#!/usr/bin/perl -w
use strict;

if (exists($ENV{'PERL_LIB'})){
	use lib $ENV{'PERL_LIB'};
} 
else{
	print "ERROR:: Not existis PERL_LIB\n";
	exit;
}

use File::Files;
use String::StringUtil;
use Database::SwissProt;

my $database = "";
my $verbose = "false";
my $outputFile = ""; 

my $PROG_NAME = "perlLib::Database::run::GetTaxonIDSwissProt.pl";

&getParameters;
&checkParameters;
&main;

############################ sub getParameters ##################################
sub getParameters {
	my $counter;
	my $ending_value = scalar(@ARGV);		#numero de parametros de entrada
	for($counter = 0 ; $counter < $ending_value ; $counter++){
		if($ARGV[$counter] eq "-database"){
			$database = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-verbose"){
			$verbose = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-outputFile"){
			$outputFile = $ARGV[$counter+1];		
		}	
	}
}

############################ sub checkParameters ##################################
sub checkParameters {
	print "\n=============================================\n";
	print "Cheking Parameters for $PROG_NAME\n";
	
	
	String->isEmpty($database, "ERROR::".$PROG_NAME.":: Parameter -database having SwissProt database required!");	
	FilesIO->checkFile($database, "ERROR::".$PROG_NAME.":: File NOT found ".$database);
	String->isEqual($verbose,"true,false", "ERROR:: ".$PROG_NAME.":: Parameter -verbose must be true or false!");
	String->isEmpty($outputFile, "ERROR::".$PROG_NAME.":: Parameter -outputFile is required it must contain the output file! ");	
	

	print "-database = $database\n";
	print "-verbose = $verbose\n";
	print "-outputFile = $outputFile\n";
	print "ok\n";
}

################################### sub main #####################################
sub main {
	my $info = "";		my @array_aux = ();
	my $ACC = "";		my $taxonID = "";
	my $lenSeq = 0;		my $count = 0;
	my $countT = 0;		my @array_ACC = ();

	if (open(FILEOUT, ">>$outputFile")){
		if (open(FILEIN, "$database")){
			while (my $line = <FILEIN>) {
				if (index($line, SwissProt->getACToken()) == 0){
					@array_aux = split(/ +/, $line);
					for (my $i=1; $i<scalar(@array_aux); $i++){
						$ACC =  $array_aux[$i];
						$ACC =~ s/;//g; 
						$ACC =~ s/\n//g;
						push(@array_ACC, $ACC);
					}
					
				}elsif (index($line, SwissProt->getTaxonIDToken()) == 0){
					@array_aux = split(/=/, $line);
					$taxonID =  $array_aux[1];
					$taxonID =~ s/;//g; 
					$taxonID =~ s/\n//g;
				}elsif (index($line, SwissProt->getSeqToken()) == 0){
					@array_aux = split(/ +/, $line);
					$lenSeq =  $array_aux[2];
					$lenSeq =~ s/ //g; 
				}elsif (index($line, SwissProt->getEndToken()) == 0){
					if ($ACC eq "" || $taxonID eq "" || $lenSeq == 0){
						print "ERROR::".$PROG_NAME.":: File ACC=$ACC or Taxon=$taxonID is empty or lenSeq=$lenSeq!\n";
					}
					for (my $i=0; $i<scalar(@array_ACC); $i++){
						print FILEOUT $array_ACC[$i]."\t".$taxonID."\t".$lenSeq."\n";
					}					
					$ACC = "";		
					$taxonID = "";
					$lenSeq = 0;
					@array_ACC = ();
					$info = "";
				}
				if ($count == 1000){
					$countT = $countT + $count;
					print "LOG::".$PROG_NAME.":: Done $countT\n";
					$count = 0;
				}else{
					$count++;
				}
			}
			close FILEIN;
		}else{
			print "ERROR::".$PROG_NAME.":: File $database NOT found!\n";
		}
		close FILEOUT;	
	}else{
		print "ERROR::".$PROG_NAME.":: File $outputFile NOT available for writting!\n";
	}		
}