#!/usr/bin/perl -w
use strict;

if (exists($ENV{'PERL_LIB'})){
	use lib $ENV{'PERL_LIB'};
} 
else{
	print "ERROR:: Not existis PERL_LIB\n";
	exit;
}

use File::Files;
use String::StringUtil;
use Sequence::sequence;
use Util::Hash;

my $database = "";
my $verbose = "false";
my $outputFile = ""; 

my $PROG_NAME = "perlLib::Database::run::getIDSizeFromFasta.pl";

&getParameters;
&checkParameters;
&main;

############################ sub getParameters ##################################
sub getParameters {
	my $counter;
	my $ending_value = scalar(@ARGV);		#numero de parametros de entrada
	for($counter = 0 ; $counter < $ending_value ; $counter++){
		if($ARGV[$counter] eq "-database" || $ARGV[$counter] eq "-db"){
			$database = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-verbose"){
			$verbose = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-outputFile" || $ARGV[$counter] eq "-o"){
			$outputFile = $ARGV[$counter+1];		
		}	
	}
}

############################ sub checkParameters ##################################
sub checkParameters {
	print "\n=============================================\n";
	print "Cheking Parameters for $PROG_NAME\n";
	
	
	String->isEmpty($database, "ERROR::".$PROG_NAME.":: Parameter -database having SwissProt database required!");	
	FilesIO->checkFile($database, "ERROR::".$PROG_NAME.":: File NOT found ".$database);
	String->isEqual($verbose,"true,false", "ERROR:: ".$PROG_NAME.":: Parameter -verbose must be true or false!");
	String->isEmpty($outputFile, "ERROR::".$PROG_NAME.":: Parameter -outputFile is required it must contain the output file! ");	
	

	print "-database = $database\n";
	print "-verbose = $verbose\n";
	print "-outputFile = $outputFile\n";
	print "ok\n";
}

################################### sub main #####################################
sub main {
	my $annotation = "";		my @array_aux = ();
	my $sequence = "";			my $info = "";
	my $lenSeq = 0;				my $count = 0;
	my $countT = 0;				my %hash = ();
	my $size;

	my @sequences = Sequences->getSequenceFromFastaArray2D($database);
	
	for (my $i=0; $i < scalar(@sequences); $i++){
			$annotation = $sequences[$i][0];
			@array_aux = split(/ +/, $annotation);
			$annotation = $array_aux[0];
			$annotation =~ s/>//g;
			$annotation =~ s/\n//g;
			if (index($annotation, "psu")){				
				$annotation =~ s/\|//g;
				$annotation =~ s/psu//g; 
			}		 
			$sequence = $sequences[$i][1];
			if (!exists($hash{$annotation})){
				$hash{$annotation} = length($sequence);
			}else{
				$size = $hash{$annotation};
				if (length($sequence) > $size ){
					$hash{$annotation} = length($sequence);
				}
			}			
	}
	Hash->hashToFile("\t", $outputFile, \%hash);
	#FilesIO->saveFile($outputFile, $info, $verbose);
}