#!/usr/bin/perl -w
use strict;

if (exists($ENV{'PERL_LIB'})){
	use lib $ENV{'PERL_LIB'};
} 
else{
	print "ERROR:: Not existis PERL_LIB\n";
	exit;
}

use File::Files;
use Util::Array;
use Sequence::sequence;
use Database::ScopPattern;

my $inputFile = "";
my $familyFile = "";
my $outputFile = "";
my $verbose = "false";
my $seq = ":";
my $countFamilyFile;
my $option = 1; #1=list File; 2=fasta File;
my $kind = 1;

my $PROG_NAME = "perlLib::Database::run::removeFromScop.pl";

&getParameters;
&checkParameters;
&main;


############################ sub getParameters ##################################
sub getParameters {
	my $counter;
	my $ending_value = scalar(@ARGV);		#numero de parametros de entrada
	for($counter = 0 ; $counter < $ending_value ; $counter++){
		if($ARGV[$counter] eq "-inputFile" || $ARGV[$counter] eq "-i"){
			$inputFile = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-verbose" || $ARGV[$counter] eq "-v"){
			$verbose = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-outputFile" || $ARGV[$counter] eq "-o"){
			$outputFile = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-option" || $ARGV[$counter] eq "-op"){
			$option = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-countFamilyFile" || $ARGV[$counter] eq "-ff"){
			$countFamilyFile = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-kind" || $ARGV[$counter] eq "-k" ){
			$kind = $ARGV[$counter+1];		
		}            	  	 			
	}
}

############################ sub checkParameters ##################################
sub checkParameters {
	print "\n=============================================\n";
	print "Cheking Parameters $PROG_NAME\n";
	
	String->isEqual($verbose,"true,false",$PROG_NAME."::ERROR::Parameter -verbose must be true or false!");
	
	String->isEmpty($inputFile, $PROG_NAME."::ERROR::Parameter -inputFile having blast output File is required!");
	FilesIO->checkFile($inputFile, $PROG_NAME."ERROR:: File $inputFile NOT found ");	
	String->isEmpty($outputFile, $PROG_NAME."::ERROR::Parameter -outputFile having output file is required!");
	String->isEmpty($countFamilyFile, $PROG_NAME."::ERROR::Parameter -countFamilyFile having families to keep in the inputFile is required!");
	FilesIO->checkFile($countFamilyFile, $PROG_NAME."ERROR:: File $countFamilyFile NOT found ");	
	
	print "-inputFile = $inputFile\n";
	print "-countFamilyFile = $countFamilyFile\n";
	print "-outputFile = $outputFile\n"; 
	print "-verbose = $verbose\n";	
	print "-option = $option\n"; 
	print "-kind = $kind (1=family, 2=super-family)\n";
	print "ok\n";
}	


########################### sub main ####################################
sub main { 
	my $info = "";				my @array = ();
	my $count = 0;				my $family1;
	my $family2;				my $sepFamily = ":";
	my $sep = " +";				my @blastLine = ();
	my $info_clean = "";
	
	my %hashFamily = FilesIO->fileToHash2($countFamilyFile, "\t", 0);
	
	if ($option == 1){#list
		@array = FilesIO->fileToArray2d($inputFile, "\t");
		for (my $i = 0; $i < scalar(@array); $i++) {
			if (exists($hashFamily{ScopPattern->getFamilyOrSupFamily($array[$i][1], $kind)})){
				$info .= $array[$i][0]."\t".ScopPattern->getFamilyOrSupFamily($array[$i][1], $kind)."\n";
				$count++;
			}
		}
	}elsif ($option == 2){#sequence
		@array = Sequences->getSequenceFromFastaArray2D($inputFile);
		for (my $i = 0; $i < scalar(@array); $i++) {
			$family1=  String->getElementArrayLine($array[$i][0], $sep, 1);
			if (exists($hashFamily{ScopPattern->getFamilyOrSupFamily($family1, $kind)})){
				$info .= $array[$i][0].Sequences->divideSeq($array[$i][1], 50)."\n";
				$count++;
			}
		}
	}elsif ($option == 3){#blast .view
		if (open(FILEIN, "$inputFile")){
			while (my $line = <FILEIN>) {
				@blastLine=  split(/\t/,  $line);
				$family1 =  String->getElementArrayLine($blastLine[0], $sepFamily, 1);
				$family2 =  String->getElementArrayLine($blastLine[1], $sepFamily, 1);
				if (exists($hashFamily{$family1}) && exists($hashFamily{$family2})){
					#print "$family1 $family2\n";
					$info .= $line;
					$line =~ s/:$family1//g;	
					$line =~ s/:$family2//g;	
					
					$info_clean .= $line;
					$count++;
				}
			
			}
			close FILEIN;
			FilesIO->saveFile($outputFile."clean", $info_clean, $verbose);
		}
	}
	FilesIO->saveFile($outputFile, $info, $verbose);
	print "Total Sequences = $count\n";
}
	
	