#!/usr/bin/perl -w
use strict;
if (exists($ENV{'PERL_LIB'})){
	use lib $ENV{'PERL_LIB'};
} 
else{
	print "ERROR:: Not existis PERL_LIB\n";
	exit;
}

use File::Files;
use Database::ScopPattern;
use Sequence::Domain;
use Util::Array;
use List::Util qw(shuffle);

my $DOMAIN_COV_THRESHOLD	= "40";
my $evalue = "1";
my $N = 5;
my $EV_MAX = 1;
my $cashArch  = "";
my $pfamArch	= "";
my $DCOList = "";
my $verbose = "false";
my $outputFile = "";

my %hashClan = ();
my %hash_DCO = ();

my %hashValidate = ();
my %hashPotential= ();
my %hashNewPot = ();
my %hash_pfam  = ();
my %hash_cash  = ();
my @potentialDom = ();
my @potential = ();
my $OriginalCert = 0;

my $columnPDom = "3";
my $columnPAcc = "5";
my $columnCDom = "4";
my $columnCAcc = "3";
my $columnPEvalue = 0;
my $columnCEvalue = 0;
my $saveDir = "";

my $PROG_NAME = "perlLib::FDR::calculeCODDFDR.pl";


&getParameters;
&checkParameters;
&main;

############################ sub getParameters ##################################
sub getParameters {
	my $counter;
	my $ending_value = scalar(@ARGV);		#numero de parametros de entrada
	for($counter = 0 ; $counter < $ending_value ; $counter++){
		if($ARGV[$counter] eq "-cashArch" || $ARGV[$counter] eq "-i"){
			$cashArch = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-pfamArch" || $ARGV[$counter] eq "-d"){
			$pfamArch= $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-verbose"){
			$verbose = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-columnPDom"){
			$columnPDom = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-columnPAcc"){
			$columnPAcc= $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-columnCDom"){
			$columnCDom = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-columnCAcc"){
			$columnCAcc= $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-DCOList"){
			$DCOList = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-N"){
			$N = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-outputFile"){
			$outputFile = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-columnCEv"){
			$columnCEvalue = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-saveDir"){
			$saveDir = $ARGV[$counter+1];
		}
	}
}
############################ sub checkParameters ##################################
sub checkParameters {
	print "\n=============================================\n";
	print "Cheking Parameters for $PROG_NAME\n";
	
	String->isEmpty($cashArch, "ERROR::".$PROG_NAME.":: Parameter -cashArch having result CASH file is required!");	
	FilesIO->checkFile($cashArch, "ERROR::".$PROG_NAME.":: File NOT found ".$cashArch);
	String->isEqual($verbose,"true,false", "ERROR:: ".$PROG_NAME.":: Parameter -verbose must be true or false!");
	String->isEmpty($pfamArch, "ERROR::".$PROG_NAME.":: Parameter -pfamArch having result Pfam file is required!");	
	FilesIO->checkFile($pfamArch, "ERROR::".$PROG_NAME.":: File NOT found ".$pfamArch);
	String->isEmpty($DCOList, "ERROR::".$PROG_NAME.":: Parameter -DCOList is required\n");
	FilesIO->checkFile($DCOList, "ERROR::".$PROG_NAME.":: File NOT found ".$DCOList);	
	String->isEmpty($outputFile, "ERROR::".$PROG_NAME.":: Parameter -outputFile is required\n");
	
	print "-pfamArch = $pfamArch\n";
	print "-cashArch = $cashArch\n";

	print "-DCOList  = $DCOList\n";
	print "-verbose = $verbose\n";	
	
	print "ok\n";
}	
################################### sub main #####################################
sub main {
	my $sum = 0;				my $FDR;
	my $info = "";		
	
	#my @FDRs = (1e-8);
	my @FDRs = (1e-10, 1e-9, 1e-8, 1e-7, 1e-6, 1e-5, 1e-4, 1e-3, 1e-2, 1);
	#print Array->printArray("\n", \@FDRs);
	
	
	%hash_DCO = FilesIO->fileToHashConcatKey($DCOList, "\t", "0,1");
	%hash_pfam = FilesIO->fileToHash($pfamArch, "\t", $columnPAcc);
	%hash_cash = FilesIO->fileToHash($cashArch, "\t", $columnCAcc);
	
	@potential = &getPotencialValidateDomains($EV_MAX);
	$OriginalCert = scalar(@potential);
	print "Num pot: $OriginalCert\n";
		

	for (my $i=0; $i<scalar(@FDRs); $i++){
		%hashValidate = ();
		%hashPotential= ();
		@potential = &getPotencialValidateDomains($FDRs[$i]);
		print $FDRs[$i]."\t"."\t".scalar(@potential)."\n";
		#print Array->printArray("\n", \@potential);
		
		$sum = 0;
		if (scalar(@potential) > 0){
			for (my $j=0; $j<$N; $j++){
				$sum += computeFDR();
				print "$i $j/$N\n";
			}
				
			$FDR = $sum/$N;
		}else{
			$FDR = 0;
		}


		print $FDRs[$i]."\t".$FDR."\t".scalar(@potential)."\n";
			
		$info .= $FDRs[$i]."\t".$FDR."\t".scalar(@potential)."\n";
	}
	FilesIO->saveFile($outputFile, $info, $verbose);
	
	
}

################################### sub main #####################################
sub computeFDR {
	my %hashUsedDomais = ();			my @array_aux = ();
	my $count = 0;						my $domPot = "";
	my $count_random_assured;			my $domain;
	my $total = 0;						my $count_random_assured_Total = 0;
	my $i = 0;							my @keysValidate = ();
	%hashNewPot = ();					my $protein;
	
	@keysValidate = keys(%hashValidate);
	#print Array->printArray("\t", \@keysValidate)."\n";
	
	my @potShuff =  shuffle(@potential);

	for (my $i=0; $i < scalar(@keysValidate); $i++){
		$protein = $keysValidate[$i];
		#print $protein."\n"; print "Pots = ".$hashPotential{$protein}."\n";
		$domPot = $hashPotential{$protein}; 
		@array_aux = split(/ +/, $domPot);
		for (my $j=0; $j < scalar(@array_aux); $j++){
			$domain = pop(@potShuff);
			#print "domain taken $domain\n";
			if (!exists($hashNewPot{$protein})){
				$hashNewPot{$protein} = $domain;
			}else{
				$hashNewPot{$protein} .= " ". $domain;
			}
		}
		#print 	"New Pots ". $hashNewPot{$protein}."\n";
	}
	
	$count_random_assured = tryAssureDomains();
	
	#print "FDR" . $count_random_assured."\t".$OriginalCert."\n";
	if ($OriginalCert > 0){
		return ($count_random_assured/$OriginalCert);
	}
	return 0;
}


################################### sub main #####################################
sub tryAssureDomains {
		
	my $validated = 0;		my $domainP;
	my $domainV;			my @arrayP = ();
	my @arrayV;
	
	while (my ($key, $value) = each(%hashNewPot)) {
	
		@arrayP = String->StringToArray($value, " +");
		@arrayV = String->StringToArray($hashValidate{$key}, " +");
		#print $key."\n";
		#print "Validated ". Array->printArray("\t", \@arrayV)."\n";
		#print "Potential ". Array->printArray("\t", \@arrayP)."\n\n";
		
		for (my $i=0; $i <scalar(@arrayP); $i++){
			$domainP = $arrayP[$i];
			$domainP =~ s/PF//g;
			#print "domainP = ".$domainP."\n";
			for (my $j=0; $j <scalar(@arrayV); $j++){
				$domainV = $arrayV[$j];
				$domainV =~ s/PF//g;
				if (exists($hash_DCO{$domainP.$domainV}) ||  exists($hash_DCO{$domainV.$domainP})){
					#if ($domainP ne "00400" && $domainV ne "00400"){
						$validated++;
						#print "Assured = ".$domainP."\t".$domainV."\n";
					#}
					$j = scalar(@arrayV);
				}
			}
		}
	}

	return $validated;
	
}
################################### sub main #####################################
sub getPotencialValidateDomains {
	my $evalue   =  $_[0];
	my @array_arch_cash;					my @array_cash_domains;
	my @array_arch_pfam;					my @array_pfam_domains;
	my $cash_arch_info;						my $pfam_arch_info;
	my $potencialDomain;					@potential = ();
	my @array_aux;							my $validateDomains;
	my $count_evaluated =0;					my $info_arch = "";
	my @matrix_aux;
	

	while ( my ($keyC, $valueC) = each(%hash_cash) ) {
		#print $keyC."\n";
		
		if (exists($hash_pfam{$keyC})){
			@array_arch_cash = String->StringToArray2d($valueC, "\n", "\t");
			@array_arch_cash =  Array->reduceMatrixSmaller2($columnCEvalue, $evalue, \@array_arch_cash);
		
			@array_cash_domains = Array->copyColMatrixToArray2($columnCDom, \@array_arch_cash);
			$cash_arch_info = Array->printArray(" ", \@array_cash_domains);
			
			@array_arch_pfam = String->StringToArray2d($hash_pfam{$keyC}, "\n", "\t");
			@array_arch_pfam =  Array->reduceMatrixSmaller2($columnPEvalue, $evalue, \@array_arch_pfam);
			@array_pfam_domains = Array->copyColMatrixToArray2($columnPDom, \@array_arch_pfam);
			$pfam_arch_info = Array->printArray(" ", \@array_pfam_domains);
			
			$potencialDomain = Domain->arch1MinusArch2Exact($cash_arch_info, $pfam_arch_info);
			$validateDomains = Domain->arch1MinusArch2Exact($cash_arch_info, $potencialDomain);
				
			
          if ($potencialDomain ne "" && $validateDomains ne "") {
          	 # print "CASH $cash_arch_info\n";
			 # print "PFAM $pfam_arch_info\n";
			 # print "validateDomains = $validateDomains\n";
			 # print "potencialDomain = $potencialDomain\n\n";
              	@array_aux = domainsAssured($validateDomains, $potencialDomain);
	      		if (scalar(@array_aux) > 0){
					$potencialDomain = Array->printArray(" ", \@array_aux);
              		$hashValidate{$keyC} = $validateDomains;
              		$hashPotential{$keyC} = $potencialDomain;
	     			#@array_aux = split(/ +/, $potencialDomain);
             		push(@potential,@array_aux);
             		@matrix_aux = Array->reduceMatrix($columnCAcc, $keyC, \@array_arch_cash);
             		$info_arch .= Array->printMatrix2("\t", "\n", \@matrix_aux);
	      		}
          }
          $count_evaluated++;
		}
	}
	if ($saveDir ne ""){
		FilesIO->saveFile($saveDir.$evalue.".txt", $info_arch, $verbose);
	}
	return @potential;
}

################################### sub main #####################################
sub domainsAssured {
	my $validatedDomain = $_[0];
	my $potentialDomain = $_[1];
		
	my @arrayV = String->StringToArray($validatedDomain, " +");
	my @arrayP = String->StringToArray($potentialDomain, " +");

	my $validated = "false";	my $domainP;
	my $domainV;			my @potAssured = ();
	
	
	for (my $i=0; $i <scalar(@arrayP); $i++){
		$domainP = $arrayP[$i];
		$domainP =~ s/PF//g;
		#print "domainP = ".$domainP."\n";
		$validated = "false";
		for (my $j=0; $j <scalar(@arrayV); $j++){
			$domainV = $arrayV[$j];
			$domainV =~ s/PF//g;
			if (exists($hash_DCO{$domainP.$domainV}) ||  exists($hash_DCO{$domainV.$domainP})){
				$validated = "true";
				$j = scalar(@arrayV);
			}
		}
		if ($validated eq "true"){
			push(@potAssured, "PF".$domainP);
		}
	}

	return @potAssured;
	
}

