#!/usr/bin/perl -w
use strict;
if (exists($ENV{'PERL_LIB'})){
	use lib $ENV{'PERL_LIB'};
} 
else{
	print "ERROR:: Not existis PERL_LIB\n";
	exit;
}

use File::Files;
use Database::ScopPattern;
use Sequence::Domain;
use Util::Array;

my $DOMAIN_COV_THRESHOLD	= "40";
my $evalue = "1";
my $inputFile  = "";
my $outputDir	= "";
my $dbIdsFile = "";
my $verbose			= "false";
my $svmProbFile = "";
my $CUT_OFF_PROB = 0.05;
my $PFAM_ACC = "";
my $CUT_OFF_ONE_DOMAIN = 1e-07;
my $CUT_OFF_EVALUE = 1;
my $CUT_OFF_SVM = -0.5;

my $COL_START_HIT = 1;
my $COL_END_HIT = 2;
my $COL_ACC = 3;
my $COL_EVALUE = 0;
my $COL_SVM = 8;

my $PROG_NAME = "perlLib::FDR::calculeFDR.pl";	

&getParameters;
&checkParameters;
&main;

############################ sub getParameters ##################################
sub getParameters {
	my $counter;
	my $ending_value = scalar(@ARGV);		#numero de parametros de entrada
	for($counter = 0 ; $counter < $ending_value ; $counter++){
		if($ARGV[$counter] eq "-inputFile" || $ARGV[$counter] eq "-i"){
			$inputFile = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-dbIdsFile" || $ARGV[$counter] eq "-d"){
			$dbIdsFile= $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-verbose"){
			$verbose = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-CUT_OFF_ONE_DOMAIN"){
			$CUT_OFF_ONE_DOMAIN = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-COL_START_HIT"){
			$COL_START_HIT = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-COL_END_HIT"){
			$COL_END_HIT = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-COL_ACC"){
			$COL_ACC = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-COL_EVALUE"){
			$COL_EVALUE = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-CUT_OFF_EVALUE"){
			$CUT_OFF_EVALUE = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-CUT_OFF_SVM"){
			$CUT_OFF_SVM = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-COL_SVM"){
			$COL_SVM = $ARGV[$counter+1];		
		}
	}
}
############################ sub checkParameters ##################################
sub checkParameters {
	print "\n=============================================\n";
	print "Cheking Parameters for $PROG_NAME\n";
	
	String->isEmpty($inputFile, "ERROR::".$PROG_NAME.":: Parameter -inputFile having result CASH file is required!");	
	FilesIO->checkFile($inputFile, "ERROR::".$PROG_NAME.":: File NOT found ".$inputFile);
	String->isEqual($verbose,"true,false", "ERROR:: ".$PROG_NAME.":: Parameter -verbose must be true or false!");
	String->isEmpty($dbIdsFile, "ERROR::".$PROG_NAME.":: Parameter -dbIdsFile having info about fasta database!");	
	FilesIO->checkFile($dbIdsFile, "ERROR::".$PROG_NAME.":: File NOT found ".$dbIdsFile);
	
	print "-inputFile = $inputFile\n";
	print "-dbIdsFile = $dbIdsFile\n";
	print "-verbose = $verbose\n";	
	
	print "ok\n";
}	

################################### sub main #####################################
sub main {
	my @arch = ();						my $hit_start	= "";
	my $hit_end = "";					my $len		= "";
	my $count_random =0 ;				my $count_real = 0;
	my $FDR;							my $accHit;
	my $count_one_domain = 0;			my $count_one_domain_conf =0;
	my $count_60 = 0;					my $count_random_one_domain = 0;
	my @new_archs = ();					my $info = "";
	my $evalue;							my $count_random_hmm = 0;
	my $count_real_svm = 0;				my $count_random_svm = 0;
	
	my %hash_IDs = FilesIO->fileToHash2($dbIdsFile, "\t", 0);
	my %hash_archs = FilesIO->fileToHash($inputFile, "\t", $COL_ACC);

	while ( my ($key, $value) = each(%hash_archs) ) {
		@arch = String->StringToArray2d($value, "\n", "\t");
		if (scalar(@arch) == 1){
			if ($arch[0][$COL_EVALUE] > $CUT_OFF_ONE_DOMAIN){
				@arch = ();
			}
			$count_one_domain++;
		}
		@new_archs = ();
		for (my $i=0; $i<scalar(@arch); $i++) {			
			$hit_start = $arch[$i][$COL_START_HIT];
			$hit_end = $arch[$i][$COL_END_HIT];
			$accHit = $arch[$i][$COL_ACC];
			$evalue = $arch[$i][$COL_EVALUE];
			
			if ($evalue <= $CUT_OFF_EVALUE){
				$len = $hash_IDs{$accHit};
				#print "LOG:: $accHit $hit_start $hit_end len=$len\n";
				
		
				if ($hit_start > $len){
					$count_random++;
					if (scalar(@arch) == 1){
						$count_random_one_domain++;
					}
					if ($arch[$i][5] eq "HMMer-3"){
						$count_random_hmm++;
					}
					if ($arch[$i][$COL_SVM] <= $CUT_OFF_SVM){
						$count_random_svm++;
						print "LOG:: $accHit $evalue $hit_start $hit_end len=$len\n";
					}
				}else{
					push(@new_archs, $arch[$i]);
					$count_real++;
					if ($arch[$i][$COL_SVM] <= $CUT_OFF_SVM){
						$count_real_svm++;
						
					}
				}
			}
			
		}
		if (scalar(@new_archs) > 0){
			$info .= Array->printMatrix2("\t", "\n", \@new_archs);
		}
	}
	
	
	if ($count_random == 0){
		$FDR = 0;
	}else{
		$FDR = $count_random/$count_real;
	}
	

	print "LOG:: Random = $count_random Real= $count_real\n";
	
	print "LOG::FRD::$FDR\n";
	print "count_one_domain= $count_one_domain\n";
	print "count_one_domain_conf= $count_one_domain_conf\n";
	print "count_60= $count_60\n";
	print "count_random_one_domain= $count_random_one_domain\n";
	print "count_real_svm= $count_real_svm\n";
	print "count_random_svm= $count_random_svm\n";
	
	FilesIO->saveFile($inputFile.".real", $info, $verbose);	
	#print "LOG::".$PROG_NAME."::Total Ramdom  $count_ramdom \n";
	
}

