#!/usr/bin/perl -w
use strict;
if (exists($ENV{'PERL_LIB'})){
	use lib $ENV{'PERL_LIB'};
} 
else{
	print "ERROR:: Not existis PERL_LIB\n";
	exit;
}

use File::Files;
use Database::ScopPattern;
use Sequence::Domain;
use Util::Array;

my $DOMAIN_COV_THRESHOLD	= "40";
my $evalue = "1";
my $inputFile  = "";
my $outputFile	= "";
my $dbIdsFile = "";
my $verbose			= "false";
my $svmProbFile = "";
my $CUT_OFF_PROB = 0.05;
my $PFAM_ACC = "";

my $PROG_NAME = "PFAM_ANNOTATION::scripts::prepare::filterRamdomDomains.pl";	

&getParameters;
&checkParameters;
&main;

############################ sub getParameters ##################################
sub getParameters {
	my $counter;
	my $ending_value = scalar(@ARGV);		#numero de parametros de entrada
	for($counter = 0 ; $counter < $ending_value ; $counter++){
		if($ARGV[$counter] eq "-inputFile" || $ARGV[$counter] eq "-i"){
			$inputFile = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-domCov"){
			$DOMAIN_COV_THRESHOLD = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-ev"){
			$evalue = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-outputFile" || $ARGV[$counter] eq "-o"){
			$outputFile= $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-dbIdsFile" || $ARGV[$counter] eq "-d"){
			$dbIdsFile= $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-verbose"){
			$verbose = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-svmProbFile"){
			$svmProbFile = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-PFAM_ACC"){
			$PFAM_ACC = $ARGV[$counter+1];	
			my @array = split(/\./, $PFAM_ACC);
			$PFAM_ACC = $array[0];
		}
	}
}
############################ sub checkParameters ##################################
sub checkParameters {
	print "\n=============================================\n";
	print "Cheking Parameters for $PROG_NAME\n";
	
	String->isEmpty($inputFile, "ERROR::".$PROG_NAME.":: Parameter -inputFile having result CASH file is required!");	
	FilesIO->checkFile($inputFile, "ERROR::".$PROG_NAME.":: File NOT found ".$inputFile);
	String->isEqual($verbose,"true,false", "ERROR:: ".$PROG_NAME.":: Parameter -verbose must be true or false!");
	String->isEmpty($outputFile, "ERROR::".$PROG_NAME.":: Parameter -outputFile is required it must contain the output file! ");	
	String->isEmpty($dbIdsFile, "ERROR::".$PROG_NAME.":: Parameter -dbIdsFile having info about fasta database!");	
	FilesIO->checkFile($dbIdsFile, "ERROR::".$PROG_NAME.":: File NOT found ".$dbIdsFile);
	#String->isEmpty($svmProbFile, "ERROR::".$PROG_NAME.":: Parameter -svmProbFile having SVM probabilty file is required!");	
	#FilesIO->checkFile($svmProbFile, "ERROR::".$PROG_NAME.":: File NOT found ".$svmProbFile);
	String->isEmpty($PFAM_ACC, "ERROR::".$PROG_NAME.":: Parameter -PFAM_ACC having PFAM domain ID is required!");
	
	print "-inputFile = $inputFile\n";
	print "-outputFile = $outputFile\n";
	print "-dbIdsFile = $dbIdsFile\n";
	print "-domCov = $DOMAIN_COV_THRESHOLD\n";
	print "-evalue = $evalue\n";
	print "-svmProbFile = $svmProbFile\n";
	print "-PFAM_ACC = $PFAM_ACC\n";
	print "-verbose = $verbose\n";	
	
	print "ok\n";
}	

################################### sub main #####################################
sub main {
	my @results = ();					my $evalueHit		= "";
	my $accHit = "";					my %hash_IDs		= ();
	my $hit_start = "";					my $hit_end = "";
	my $domain_start = "";				my $domain_end = "";
	my $len = "";						my $domain_cov;	
	my @array_aux = ();					my $info = "";
	my $count_ramdom = 0;				my $key;
	my %hash_svm = ();					my $prob;
	my @array_aux_prob = ();			my $bestModel;
	my $info_unique;
	
	@results = FilesIO->fileToArray2d($inputFile, "\t");
	%hash_IDs = FilesIO->fileToHash2($dbIdsFile, "\t", 0);
	
	for (my $i=0; $i<scalar(@results); $i++) {
		$evalueHit = $results[$i][4];
		$accHit = $results[$i][3];
		$hit_start = $results[$i][5];
		$hit_end = $results[$i][6];
		$domain_start= $results[$i][1];
		$domain_end= $results[$i][2];
		$bestModel = $results[$i][0];
		
		if ($evalueHit <= $evalue){
			if (!exists($hash_IDs{$accHit}) ){
				print "ERROR::".$PROG_NAME.":: $accHit NOT found in $dbIdsFile \n";
				exit;
			}else{
				$len = $hash_IDs{$accHit};
				if ($hit_start > $len){
					$domain_cov = Domain->calculateCoverage($domain_start, $domain_end, $hit_start, $hit_end);
					if ($domain_cov >= $DOMAIN_COV_THRESHOLD){	
						$prob = 0.05;
						$info_unique = $evalueHit."\t0\t".$hit_start."\t".$hit_end."\t".$accHit."\t".$PFAM_ACC."\t".$bestModel."\t".$domain_start."\t".$domain_end."\t".$prob."\t\n";
						print "LOGRandom::".$info_unique;
						$info .= $info_unique;							
						$count_ramdom++;		
						if ($verbose eq "true"){print "LOG::$accHit\t$hit_start\t$hit_end\n";}																			
					}					
				}else{
					#push(@array_aux, $results[$i]);	
				}				
			}
			
		}
	}
	
	
	FilesIO->saveFile($outputFile, $info, $verbose);	
	print "LOG::".$PROG_NAME."::Total Ramdom  $count_ramdom \n";
	
}

################################### sub main #####################################
sub main2 {
	my @results = ();					my $evalueHit		= "";
	my $accHit = "";					my %hash_IDs		= ();
	my $hit_start = "";					my $hit_end = "";
	my $domain_start = "";				my $domain_end = "";
	my $len = "";						my $domain_cov;	
	my @array_aux = ();					my $info = "";
	my $count_ramdom = 0;				my $key;
	my %hash_svm = ();					my $prob;
	my @array_aux_prob = ();			my $bestModel;
	my $info_unique;
	
	@results = FilesIO->fileToArray2d($inputFile, "\t");
	%hash_IDs = FilesIO->fileToHash2($dbIdsFile, "\t", 0);
	%hash_svm = FilesIO->fileToHash2($svmProbFile, "\t", "0");
	
	for (my $i=0; $i<scalar(@results); $i++) {
		$evalueHit = $results[$i][4];
		$accHit = $results[$i][3];
		$hit_start = $results[$i][5];
		$hit_end = $results[$i][6];
		$domain_start= $results[$i][1];
		$domain_end= $results[$i][2];
		$bestModel = $results[$i][0];
		
		if ($evalueHit <= $evalue){
			if (!exists($hash_IDs{$accHit}) ){
				print "ERROR::".$PROG_NAME.":: $accHit NOT found in $dbIdsFile \n";
				exit;
			}else{
				$len = $hash_IDs{$accHit};
				if ($hit_start > $len){
					$domain_cov = Domain->calculateCoverage($domain_start, $domain_end, $hit_start, $hit_end);
					if ($domain_cov >= $DOMAIN_COV_THRESHOLD){	
						$key = $accHit."_".$hit_start."_".$hit_end;
						if (!exists($hash_svm{$key})){
							print "ERROR::".$PROG_NAME.":: $key NOT found in $svmProbFile\n";
							exit;
						}
						#@array_aux_prob = split(/\t/, $hash_svm{$key});
						#print $hash_svm{$key};
						$prob = $hash_svm{$key};
						if ($prob >= $CUT_OFF_PROB){
							$info_unique = $evalueHit."\t0\t".$hit_start."\t".$hit_end."\t".$accHit."\t".$PFAM_ACC."\t".$bestModel."\t".$domain_start."\t".$domain_end."\t".$prob."\t\n";
							print "LOGRandom::".$info_unique;
							$info .= $info_unique;
							
							$count_ramdom++;		
							if ($verbose eq "true"){print "LOG::$accHit\t$hit_start\t$hit_end\n";}	
						}														
					}					
				}else{
					#push(@array_aux, $results[$i]);	
				}				
			}
			
		}
	}
	
	
	FilesIO->saveFile($outputFile, $info, $verbose);	
	print "LOG::".$PROG_NAME."::Total Ramdom  $count_ramdom \n";
	
}



