package HMMER;

if (exists($ENV{'PERL_LIB'})){
	use lib $ENV{'PERL_LIB'};
} 
else{
	print "ERROR:: Not existis PERL_LIB\n";
	exit;
}

use File::Files;
use Util::Array;
use Util::Number;
use HMMs::HMMER::hmmer;
use Align::Primary::STOCKHOLM;
use analisys::analisys;

my $space = " ";
my $build = "hmmbuild";
my $calibrate = "hmmcalibrate";
my $search_hmm = "hmmsearch";
my $HMMEMIT = "hmmemit";

my $RESULT_TOKEN = "Scores for complete sequences (score includes all domains):";
my $RESULT_DOMAIN_TOKEN ="Domain annotation for each sequence (and alignments):";
my $START_VITERBI = "start_viterbi";
my $STOP_VITERBI = "                 ------- --------";
my $EACH_DOMAIN_TOKEN ="  == domain";
my $END_DOMAINS= "Internal pipeline statistics summary:";
my $RESULT_TOKEN_ALN = "Alignments of top-scoring domains:";
my $RESULT_TOKEN_DOMAIN_HIT = "Parsed for domains:";
my $RESULT_TOKEN_QUERY_HMM = "Query HMM:";
my $RESULT_TOKEN_ACCESSION = "Accession:";

########################### sub verifyFile ##############################
sub verifyFile {
	my $file	= $_[1];
	
	if (FilesIO->isFile($file) eq "false"){ 
		print "ERROR::perlLib::HMMs::HMMER::hmmer:: File $file NOT created\n";
		exit;
	}
}

########################### sub clearHmmPfamOut ##############################
sub runJackHmmer {
	my $model_file		= $_[1];
	my $query			= $_[2];
	my $database		= $_[3];
    my $parameters		= $_[4];
    my $iteration		= $_[5];
    my $evalue			= $_[6];
    my $save			= $_[7];
    my $verbose			= $_[8];
    
    my $tmpName;
    my $ext = "";
    
    if (!defined($verbose)){
		$verbose = "true";
	}
    FilesIO->createDirFromFile($model_file);
	
	my $cmd = "jackhmmer". $space. $parameters;
	if ($save eq "1"){
		$cmd .=  " --chkhmm $model_file ";
		$ext = ".hmm";
	}elsif ($save eq "2"){
		$cmd .=  " --chkali $model_file ";
		$ext = ".sto";
	}elsif ($save eq "3"){
		$cmd .=  " -A $model_file ";
		$ext = ".aln";
	}
	
	$cmd .= " -N $iteration --incE $evalue -o /dev/null ". $space . $query. $space. $database;
	if ($verbose eq "true"){
		print $cmd."\n";
	}
	system($cmd);
	
	if ($save <=2){
		for (my $i=$iteration; $i > 0; $i--){
			$tmpName = $model_file."-".$i.$ext;
			if (FilesIO->isFile($tmpName) eq "true"){
				if ($save eq "2"){
					STOCKHOLM->convertSTOCKHOLMToFasta($tmpName, $tmpName.".fasta", "true");
					$tmpName = $tmpName.".fasta";
				}			
				$cmd = "mv $tmpName $model_file";
				if ($verbose eq "true"){
					print $cmd."\n";
				}
				system($cmd);			
				return $model_file;
			}	
		}
		print "HMMER::runJackHmmer::ERROR File $model_file not created\n";	
	}else{
		STOCKHOLM->convertSTOCKHOLMToFasta($model_file, $model_file, "true");
	}
	return $model_file;	
}

########################### sub clearHmmPfamOut ##############################
sub buildModel {
	my $model_file		= $_[1];
	my $aln_file		= $_[2];
	my $parameters		= $_[3];
    my $path_program	= $_[4];
    my $verbose			= $_[5];
    
    if (!defined($verbose)){
		$verbose = "true";
	}
    FilesIO->createDirFromFile($model_file);
	
	my $cmd = $path_program. $build. $space. $parameters. $space . $model_file. $space. $aln_file;
	if ($verbose eq "true"){
		print $cmd."\n";
	}
	system($cmd);
	
	HMMER->verifyFile($model_file);	
	
}

########################### sub clearHmmPfamOut ##############################
sub createRandomSequences {
	my $model_file		= $_[1];
	my $parameters		= $_[2];
	my $outputFile		= $_[3];
    my $accName			= $_[4];
    my $verbose			= $_[5];
    
    if (!defined($verbose)){
		$verbose = "true";
	}
	
	my $cmd = $HMMEMIT. $space. $parameters. $space . " -o $outputFile ". $model_file;
	if ($verbose eq "true"){
		print $cmd."\n";
	}
	system($cmd);
	
	HMMER->verifyFile($outputFile);	
	my @array_aux = ();	my $info = ""; my $oldId;
	my @array = FilesIO->lineFileToArray($outputFile);
	for (my $i=0; $i<scalar(@array); $i++){
		if (index($array[$i], ">") == 0){
			@array_aux = split(/-/, $array[$i]);
			$oldId = $array_aux[0];
			$oldId =~ s/>//g;
			$array[$i] =~ s/$oldId/$accName/g;
		}
		$info .= $array[$i];
	}
	FilesIO->saveFile($outputFile, $info, $verbose);
	
}
########################### sub clearHmmPfamOut ##############################
sub parseViterbiOutput {
	my $resultFile		= $_[1];
	my $outputFile		= $_[2];
	my $verbose			= $_[3];
	
	my @array_aux = ();		my $state;
	my $column;				my $amino;
	my $info ="";
	
	my @array = FilesIO->lineFileToArrayWithTokenStarStop($resultFile, $START_VITERBI, $STOP_VITERBI);
	
	for (my $i = 3; $i< scalar(@array); $i++){
		@array_aux = split(/ +/, $array[$i]);
		if (scalar(@array_aux) == 6){
			$state = $array_aux[0];
			if ($state eq "M"){
				$column = $array_aux[1];
				$amino  = $array_aux[5];
				$info .=$column."\t".$amino;
			}
		}
		
	}
	FilesIO->saveFile($outputFile, $info, $verbose);
}


########################### sub clearHmmPfamOut ##############################
sub buildModelSaveOut {
	my $model_file		= $_[1];
	my $aln_file		= $_[2];
	my $parameters		= $_[3];
    my $path_program	= $_[4];
    my $verbose			= $_[5];
	
	if (!defined($verbose)){
		$verbose = "true";
	}

	FilesIO->createDirFromFile($model_file);
	HMMER->verifyFile($aln_file);
	
	my $cmd = $path_program. $build. $space. $parameters. $space . $model_file. $space. $aln_file. " > ". $model_file.".out";
	if ($verbose eq "true"){
		print $cmd."\n";
	}
	system($cmd);
	HMMER->verifyFile($model_file);	
	my @array_file = FilesIO->lineFileToArrayWithTokenStarStop($model_file.".out","Constructing model architecture", "done.");
	my $info = Array->readArrayToString(scalar(@array_file), "", \@array_file);
	FilesIO->saveFile($model_file.".out", $info, $verbose);
	
}

########################### sub clearHmmPfamOut ##############################
sub calibrateModel {
	my $model_file		= $_[1];
	my $parameters		= $_[2];
    my $path_program	= $_[3];
    my $verbose			= $_[4];
	
	if (!defined($verbose)){
		$verbose = "true";
	}
    
    
    HMMER->verifyFile($model_file);
	
	my $cmd = $path_program. $calibrate. $space. $parameters. $space . $model_file;
	if ($verbose eq "true"){
		print $cmd."\n";
	}
	system($cmd);
}

########################### sub clearHmmPfamOut ##############################
sub search {
	my $model_file		= $_[1];
	my $result_file		= $_[2];
	my $db_file			= $_[3];
	my $parameters		= $_[4];
    my $path_program	= $_[5];
    my $verbose			= $_[6];
	
	if (!defined($verbose)){
		$verbose = "true";
	}
    
    HMMER->verifyFile($model_file);
	FilesIO->createDirFromFile($result_file);
	
	my $cmd = $path_program. $search_hmm. $space. $parameters. $space . $model_file. $space. $db_file. $space. ">". $space. $result_file;
	if ($verbose eq "true"){
		print $cmd."\n";
	}
	system($cmd);
	
	HMMER->verifyFile($result_file);
}

########################### sub clearHmmPfamOut ##############################
sub renameModel {
	my $model_file		= $_[1];
	my $new_name		= $_[2];
	
	my $info = "";
	
	if (FilesIO->isFile($model_file) eq "false"){
		print "ERROR::perlLib::HMMs::HMMER::hmmer:: file  $model_file not found!\n";
	}else{
		my @arrayLine = FilesIO->lineFileToArray($model_file);
		my @array = ();
		my $i  = 0;		
		while ($i < scalar(@arrayLine)) {
			@array = split(/ +/, $arrayLine[$i]);
			if (scalar(@array) == 2 && $array[0] eq "NAME"){
				$info .= $array[0]."  ". $new_name."\n";
			}else{
				$info .= $arrayLine[$i];
			}
			$i++;
		}
		if ($info ne ""){
			FilesIO->saveFile($model_file, $info);
		}else{
			print "ERROR $model_file file  model is Empty!\n";
		}
		
	}
}

########################### sub clearHmmPfamOut ##############################
sub getLengthModel {
	my $model_file		= $_[1];
	my $verbose			= $_[2];
	
	my $info = "";
	
	if (FilesIO->isFile($model_file) eq "false"){
		print "ERROR::perlLib::HMMs::HMMER::hmmer:: file  $model_file not found!\n";
	}else{
		my @arrayLine = FilesIO->lineFileToArrayStartWith($model_file, "LENG");
		$length = String->getElementArrayLine($arrayLine[0], " +", 1);
		return $length;
	}
	print "ERROR::perlLib::HMMs::HMMER::hmmer:: Token LENG NOT found in  $model_file!\n";
}


#################################### sub parserHmmerResults ##################################
sub parserHmmerResults{
		my $path			= $_[1];

		my $outfileName = $path.".parsed";

		HMMER->verifyFile($path);
		
		my $maxEvalue = "10";
		my $minScore = "-100";

				
		my @datas =FilesIO->lineFileToArray($path);
				
		my $start_line = "-------- -----------                                    -----    ------- ---";
		my $empty_result = "[no hits above thresholds]";
		my $start = "false";
		my $stop  = "false";
		my $line;
		my $iten;
		my @line_result;
		my @array_result_aux;
		my @array_result;
		my $i = 0;
		my $j = 0;
		my $info = "";

		while ($i < scalar(@datas) && $stop ne "true"){
			if (index($datas[$i],$start_line) >=0){
				$start = "true";				
			}elsif (index($datas[$i], $empty_result) >=0){
				$stop = "true";
				$start = "true"
			}elsif ($datas[$i] eq "\n" && $start eq "true"){
				$stop = "true";
			}elsif ($start eq "true"){
				@line_result = split (/ +/, $datas[$i]);
				$info .= $line_result[0]."\t";				 
				for (my $j = scalar(@line_result) - 3; $j < scalar(@line_result) -1; $j++){
					$info .= $line_result[$j] . "\t";
				}
				for (my $j = 1; $j < scalar(@line_result)-3; $j++){
					$info .= $line_result[$j] . " ";
				}
				@array_result_aux = split(/\t/, $info);
				$array_result[$j][0] = $array_result_aux[0];
				$array_result[$j][1] = $array_result_aux[2];
				$j++;
				$info = "";
			}
			$i++;
		}		
		return @array_result;			
}

#################################### sub parserHmmerResults ##################################
sub parserHmmer3Results{
		my $path			= $_[1];
		my $verbose			= $_[2];
		
		my @array = FilesIO->lineFileToArray($path);
		my @array_aux = ();
		my $line; my $info = ""; my $start = "false"; my $i = 0;
		while ($i <  scalar(@array)){
			if (index($array[$i], $RESULT_TOKEN) >=0){
				$start = "true";
				$i = $i + 4;
			}
			if ($start eq "true"){
				@array_aux = split(/ +/, $array[$i]);
				if (scalar(@array_aux) > 10){
					#$info .=$array_aux[9]."\t".$array_aux[10]."\t".$array_aux[1]."\n";
					$info .=$array_aux[9]."\t".$array_aux[1]."\n"
				}
			}
			if ($start eq "true" && $array[$i] eq "\n"){
				$i = scalar(@array);
			}
			$i++;
		}
		return $info;		
}

#################################### sub parserHmmerResults ##################################
sub parserHmmer3ResultDomtblout{
	my $inputFile		= $_[1];
	my $outputFile		= $_[2];
	my $verbose			= $_[3];
		
	my $C_EV = "11";			my $C_SCORE = "13";			my $C_START_Q = "17";
	my $C_END_Q = "18";			my $C_START_M = "15";		my $C_END_M = "16";
	my $C_QUERY = "0";			my $C_DOMAIN = "4";			my $C_DOMAIN_NAME = "3";
	my $C_IDENTITY = "20";		my $cols;
	
	my @domains = FilesIO->fileToArray2d($inputFile, " +", $verbose);
	@domains = Array->removeRowsMatriz("0:1:2", \@domains);
	$cols = "$C_EV:$C_SCORE:$C_START_Q:$C_END_Q:$C_QUERY:$C_DOMAIN:$C_DOMAIN:$C_START_M:$C_END_M:$C_IDENTITY:$C_DOMAIN_NAME";
	@domains = Array->getSomeColsMatrix2("$cols", \@domains);
	
	my $info = Array->printMatrix2("\t", "\n", \@domains);
	FilesIO->saveFile($outputFile, $info, $verbose);
}
#################################### sub parserHmmerResults ##################################
sub createROCFile{
	my $inputFile		= $_[1];
	my $truePosFile		= $_[2];
	my $outputFile		= $_[3];
	my $evalueStart		= $_[4];
	my $evalueRange		= $_[5];
	my $C_QUERY			= $_[6];
	my $C_EVALUE		= $_[7];
	my $totalPos		= $_[8];
	my $totalNeg		= $_[9];
	my $verbose			= $_[10];
	
	my @matrix_rates_pos = ();			my @matrix_rates_neg = ();
	my @matrix_rates = ();				my %hash_uniq = ();
	my $info = "";
	
	my @domains = FilesIO->fileToArray2d($inputFile, "\t", $verbose);
	@domains = Array->sortArray2d($C_EVALUE, \@domains);
	my %hash_true = FilesIO->fileToHash($truePosFile, "\t", 0);
	
	my @matrix_rates_log = Analisys->initialArrayCountHistogramFloatJumpLog(1, $evalueStart, $evalueRange);
	my @matrix_rates_int = Analisys->initialArrayCountHistogramFloatJump(20, 1, 1);
	push(@matrix_rates, @matrix_rates_log);
	push(@matrix_rates, @matrix_rates_int);
	@matrix_rates_pos = map { [@$_] } @matrix_rates;
	@matrix_rates = ();
	push(@matrix_rates, @matrix_rates_log);
	push(@matrix_rates, @matrix_rates_int);
	@matrix_rates_neg = map { [@$_] } @matrix_rates;
	
	for (my $i = 0; $i < scalar(@domains); $i++){
		#print $domains[$i][$C_QUERY]."\t".$domains[$i][$C_EVALUE]."\n";
		if (!exists($hash_uniq{$domains[$i][$C_QUERY]})){
			if (exists($hash_true{$domains[$i][$C_QUERY]})){
				#print ">>True\t". $domains[$i][$C_QUERY]."\t".$domains[$i][$C_EVALUE]."\n";
				@matrix_rates_pos = Analisys->fillMatrixSta(1, $domains[$i][$C_EVALUE], \@matrix_rates_pos);
			}else{
				#print ">>False\t". $domains[$i][$C_QUERY]."\t".$domains[$i][$C_EVALUE]."\n";
				@matrix_rates_neg = Analisys->fillMatrixSta(1, $domains[$i][$C_EVALUE], \@matrix_rates_neg);
			}
			$hash_uniq{$domains[$i][$C_QUERY]} = "";			
		}		
	}
	#print "\n". Array->printMatrix2("\t", "\n",\@matrix_rates_neg)."\n";
	
	@matrix_rates_pos = Array->divideValueMatrizByCol($totalPos, 1, \@matrix_rates_pos);
	@matrix_rates_neg = Array->divideValueMatrizByCol($totalNeg, 1, \@matrix_rates_neg);
	
	for (my $i = 0; $i < scalar(@matrix_rates_pos); $i++){
		$info .= $matrix_rates_neg[$i][1]."\t".	$matrix_rates_pos[$i][1]."\n"
	}
	FilesIO->saveFile($outputFile, $info, $verbose);
	#print Array->printMatrix2("\t", "\n",\@matrix_rates_neg);
	#print Array->printMatrix2("\t", "\n",\@matrix_rates_pos);
	
	
}

#################################### sub parserHmmerResults ##################################
sub getBestDomainHmmer3{
		my $path			= $_[1];
		my $acc_number		= $_[2];
		my $verbose			= $_[3];
		
		
		my @array = FilesIO->lineFileToArrayWithToken($path, $RESULT_DOMAIN_TOKEN);
		my @array_aux = ();
		my $line; my $info = ""; my $start = "false"; my $i = 1;
		my $minEvalue = 1000; my %record = (); my $evalue;
		
		while ($i <  scalar(@array)){
			if (index($array[$i], ">>") >=0 && index($array[$i], $acc_number) >=0){
				$start = "true";
				$i = $i + 3;
			}
			if ($start eq "true" && $array[$i] eq "\n"){
				#print $array[$i];
				$i = scalar(@array);
				$start = "false";
			}
			if ($start eq "true"){
				@array_aux = split(/ +/, $array[$i]);
				if (scalar(@array_aux) == 17){					
					$evalue   = $array_aux[6];					
					if ($evalue < $minEvalue){
						%record = ( "hitStart" 	=> $array_aux[10],
            		   				"hitEnd" 	=> $array_aux[11],
            		   				"score"  	=> $array_aux[3],
			           				"evalue"   	=> $evalue,
			           				"hitDomainStart" => $array_aux[7],
									"hitDomainEnd" => $array_aux[8],
									"cevalue" => $array_aux[5]
			           			   );
			           	$minEvalue = $evalue;
					}		
				}else{
					print "ERROR::HMMs::HMMER::parserHmmer3ResultsBySequences:: Line $array[$i] is not well formated\n";
				}
			}
		
			$i++;
		}
		return %record;		
}

#################################### sub parserHmmerResults ##################################
sub getListIdsHmmer3{
	my $path			= $_[1];
	
	my @array = FilesIO->lineFileToArrayWithTokenStarStop($path, $RESULT_TOKEN, $RESULT_DOMAIN_TOKEN);
	my $i = 3;
	my @array_aux;				my @array_ret = ();
	
	while ($i <  scalar(@array)){
		#print  $array[$i];
		@array_aux = String->StringToArray($array[$i], " +");
		if (scalar(@array_aux) >=9){
			push(@array_ret, $array_aux[8]);
			
		}		
		$i++;	
	}

	return @array_ret;
}

#################################### sub parserHmmerResults ##################################
sub getAllDomainHmmer3{
		my $path			= $_[1];
		my $acc_number		= $_[2];
		my $verbose			= $_[3];
		
		
		my @array = FilesIO->lineFileToArrayWithToken($path, $RESULT_DOMAIN_TOKEN);
		my @array_aux = ();
		my $line; my $info = ""; my $start = "false"; my $i = 1;
		my $minEvalue = 1000; my %record = (); my $evalue;
		my $index = 0; my $startHit = 10000; my $endHit = -1;
		my $startQuery = 10000; my $endQuery = -1;
		my $alinedHit; my $alinedQuery;
		my @array_ret = ();
		
		while ($i <  scalar(@array)){
			#print  $array[$i];
			if (index($array[$i], ">>") >=0){
				if (index($array[$i], $acc_number) >=0 && $start eq "false"){
					$start = "true";
					$i = $i + 3;
				}else{
					$start = "false";
				}
			}
			if ($start eq "true" && index($array[$i], $EACH_DOMAIN_TOKEN) ==0){
				$startHit = 10000; $endHit = -1; $startQuery = 10000; $endQuery = -1;
				$alinedHit = ""; $alinedQuery = "";
				@array_aux = split(/ +/, $array[$i]);
				if (scalar(@array_aux) == 10){	
					#$index 		= $array_aux[3] - 1;			
					$evalue   	= String->removeCaracter($array_aux[9], "\n");		
					$score 		= $array_aux[5];					
				}else{
					print "ERROR::HMMs::HMMER::parserHmmer3ResultsBySequences:: Line $array[$i] is not well formated\n";
				}
				$i++;
				while ($i <  scalar(@array) && index($array[$i], $EACH_DOMAIN_TOKEN) < 0 && index($array[$i], ">>") < 0 && index($array[$i], $END_DOMAINS) < 0){
					
					if ($array[$i] =~ m/\w\d/ && $array[$i] !~ m/\*/){
						@array_aux = String->StringToArray($array[$i], " +");
						if (scalar(@array_aux) == 4){
							#print  $array[$i];
							if ($array_aux[0] eq $acc_number) {#hit
								if (Number->isNumber3($array_aux[1]) eq "true" && $array_aux[1] < $startHit){
										$startHit = $array_aux[1];
								}
								$alinedHit .= $array_aux[2];
								if (Number->isNumber3($array_aux[3]) eq "true" && $array_aux[3] > $endHit){
										$endHit = $array_aux[3];
								}
							}
							else{#Query
								if (Number->isNumber3($array_aux[1]) eq "true" && $array_aux[1] < $startQuery){
									$startQuery = $array_aux[1];
								}
								$alinedQuery .= $array_aux[2];
								if (Number->isNumber3($array_aux[3]) eq "true" && $array_aux[3] > $endQuery){
									$endQuery = $array_aux[3];
								}
							}
						}						
					}					
					$i++;
				}
				if ($startQuery != 10000 && $endQuery != -1 && $startHit != 10000 && $endHit != -1){
					$array_ret[$index][0] = $evalue;
					$array_ret[$index][1] = $startHit;
					$array_ret[$index][2] = $endHit;
					$array_ret[$index][3] = $startQuery;
					$array_ret[$index][4] = $endQuery;
					$array_ret[$index][5] = $score;
					$array_ret[$index][6] = $startQuery;
					$array_ret[$index][7] = $endQuery;
					$array_ret[$index][8] = $alinedHit;
					$array_ret[$index][9] = $alinedQuery;
					$index++;
				}
				
				$i--;
			}		
			$i++;
		}
		return @array_ret;		
}

#################################### sub parserHmmerResults ##################################
sub getAllDomainHmmScan{
		my $path			= $_[1];
		my $outputFile		= $_[2];
		my $verbose			= $_[3];
		
		my $info = "";		my $final_info = "";
		
		if (open(FILEIN, "$path")){
			while (my $line = <FILEIN>) {
				$info .= $line;
				
				if (index($line, "//") == 0){		
					#print ">>>getAllDomainHmmScan\n". $info;		
					$final_info .= HMMER->getAllDomainByQuery($info, $verbose);
					$info = "";	
				}	
			}
			FilesIO->saveFile($outputFile, $final_info, $verbose);		
		}
		else{
			print "ERROR::".$PROGRAM_NAME.":: File $path NOT found\n";
			exit;
		}
		
}
#################################### sub parserHmmerResults ##################################
sub getAllDomainByQuery{
	my $info			= $_[1];
	my $verbose			= $_[2];
	my $i = 0;			my @array_aux = ();
	my $query;			my $info_domain = "";
	my $total_info_domain = "";
	my $startNewDomain = "false";
	
	my @array = split("\n", $info);
	
	while ($i <  scalar(@array)){
		#print $array[$i];
		if (index($array[$i], "Query: ") == 0){			
			@array_aux = split(/ +/, $array[$i]);
			$query = $array_aux[1];
		}if (index($array[$i], ">> ") == 0){
			$startNewDomain = "true";
			#print "\n\nINFO_DOMAIN\n". $info_domain."\n";
			if ($info_domain ne ""){
				
				$total_info_domain .=HMMER->getAllDomainHitsHmmer3($info_domain, $query, $verbose);
			}
			$info_domain = "";
		}
		if ($startNewDomain eq "true"){
			$info_domain .=$array[$i]. "\n";
		}			
		$i++;
	}
	$total_info_domain .= HMMER->getAllDomainHitsHmmer3($info_domain, $query, $verbose);		
}

#################################### sub parserHmmerResults ##################################
sub getAllDomainHitsHmmer3{
		my $info_domain		= $_[1];
		my $acc_number		= $_[2];
		my $verbose			= $_[3];
		
		my @array = split("\n", $info_domain);
		
		my @array_aux = ();
		my $line; my $info = ""; my $start = "false"; my $i = 1;
		my $minEvalue = 1000; my %record = (); my $evalue; my $score;
		my $index = 0; my $startHit = 10000; my $endHit = -1;
		my $startQuery = 10000; my $endQuery = -1; my $model;
		my $alinedHit; my $alinedQuery;
		my @array_ret = ();
		
		while ($i <  scalar(@array)){
			#print "line ".$array[$i]. "\n";
			if (index($array[$i], $EACH_DOMAIN_TOKEN) ==0){
				$startHit = 10000; $endHit = -1; $startQuery = 10000; $endQuery = -1;
				$alinedHit = ""; $alinedQuery = "";
				@array_aux = split(/ +/, $array[$i]);
				print "scalar".scalar(@array_aux)."\n";
				if (scalar(@array_aux) == 10){	
					$evalue   	= String->removeCaracter($array_aux[9], "\n");		
					$score 		= $array_aux[5];					
				}else{
					print "ERROR::HMMs::HMMER::parserHmmer3ResultsBySequences:: Line $array[$i] is not well formated\n";
				}
				$i++;
				while ($i < scalar(@array) && index($array[$i], $EACH_DOMAIN_TOKEN) < 0){
					
					if ($array[$i] =~ m/\w\d/ && $array[$i] !~ m/\*/){
						@array_aux = String->StringToArray($array[$i], " +");
						if (scalar(@array_aux) == 4){
							print  $array[$i]. "\n";
							if ($array_aux[0] eq $acc_number) {#hit
								if (Number->isNumber3($array_aux[1]) eq "true" && $array_aux[1] < $startHit){
										$startHit = $array_aux[1];
								}
								$alinedHit .= $array_aux[2];
								if (Number->isNumber3($array_aux[3]) eq "true" && $array_aux[3] > $endHit){
										$endHit = $array_aux[3];
								}
							}
							else{#Query
								$model = $array_aux[0];
								if (Number->isNumber3($array_aux[1]) eq "true" && $array_aux[1] < $startQuery){
									$startQuery = $array_aux[1];
								}
								$alinedQuery .= $array_aux[2];
								if (Number->isNumber3($array_aux[3]) eq "true" && $array_aux[3] > $endQuery){
									$endQuery = $array_aux[3];
								}
							}
						}						
					}					
					$i++;
				}
				if ($startQuery != 10000 && $endQuery != -1 && $startHit != 10000 && $endHit != -1){
					$array_ret[$index][0] = $evalue;
					$array_ret[$index][1] = $startHit;
					$array_ret[$index][2] = $endHit;
					$array_ret[$index][3] = $startQuery;
					$array_ret[$index][4] = $endQuery;
					$array_ret[$index][5] = $score;
					$array_ret[$index][6] = $model;
					$array_ret[$index][7] = $acc_number;
					$alinedHit = String->removeCaracter($alinedHit, "."); $alinedHit = String->removeCaracter($alinedHit, "-");
					$array_ret[$index][8] = $alinedHit;
					$alinedQuery = String->removeCaracter($alinedQuery, "."); $alinedQuery = String->removeCaracter($alinedQuery, "-");
					$array_ret[$index][9] = $alinedQuery;
					$index++;
					print "oi". Array->printArray("\t", "\n", \@array_ret);
				}				
				$i--;
			}		
			$i++;
		}
		return Array->printMatrix("\t", scalar(@array_ret), $#{$array_ret[0]} + 1, \@array_ret);	
}


#################################### sub parserHmmerResults ##################################
sub getAllDomainHitsHmmer2{
		my $inputFile		= $_[1];
		my $outputFile		= $_[2];
		my $verbose			= $_[3];
		
		my @array_aux = ();				my $domainName;
		my $domainAcc;					my $info = "";
		my @array_aux_2 = ();			my $accQuery;
		my $startQuery;					my $endQuery;
		my $startModel;					my $endModel;		
		my $score;						my $evalue;		
		
		@array_aux = FilesIO->lineFileToArrayWithToken($inputFile, $RESULT_TOKEN_QUERY_HMM);
		@array_aux = split(/ +/, $array_aux[0]);
		
		$domainName =  $array_aux[scalar(@array_aux)-1];
		
		@array_aux = FilesIO->lineFileToArrayWithToken($inputFile, $RESULT_TOKEN_ACCESSION);
		@array_aux = split(/ +/, $array_aux[0]);
		
		$domainAcc =  $array_aux[scalar(@array_aux)-1];
		$domainAcc = String->removeCaracter($domainAcc, "\n");
		
		my @array = FilesIO->lineFileToArrayWithTokenStarStop($inputFile, $RESULT_TOKEN_DOMAIN_HIT, $RESULT_TOKEN_ALN);
		
		for (my $i=0; $i< scalar(@array)-1; $i++){
			@array_aux = split(/ +/, $array[$i]);
			if (scalar(@array_aux) == 10){
				$accQuery	= $array_aux[0];
				@array_aux_2 = split(/\|/, $accQuery);
				$accQuery = $array_aux_2[1];
				$startQuery = $array_aux[2];
				$endQuery 	= $array_aux[3];
				$startModel = $array_aux[5];
				$endModel 	= $array_aux[6];
				$score		= $array_aux[8];
				$evalue		= String->removeCaracter($array_aux[9], "\n");
				$info .= $evalue."\t".$score."\t".$startQuery."\t".$endQuery."\t".$accQuery."\t".$domainAcc."\t"."HMMer-2"."\t".$startModel."\t".$endModel."\t".$domainName;
			}						
		}
		FilesIO->saveFile($outputFile, $info, $verbose);
}

1
