package SAM;

if (exists($ENV{'PERL_LIB'})){
	use lib $ENV{'PERL_LIB'};
} 
else{
	print "ERROR:: Not existis PERL_LIB\n";
	exit;
}


use Bio::AlignIO;
use String::StringUtil;
use File::Files;
use strict;

my $build_model_program = "./modelfromalign";
my $calibrate_program   = "./hmmscore";
my $search_program   = "./hmmscore";
my $target2k_program   = "./target2k";

############################ sub runFormatFiletoSam ##################################
sub formatFileToSam {
	my $input_file_name 	= $_[1];
	my $output_file_name	= $_[2];
	
	if (FilesIO->isFile($input_file_name) eq "true"){	

   		my $in  = Bio::AlignIO->new(-file => $input_file_name,'-format' => 'clustalw');     
   		my $out = Bio::AlignIO->new(-file => ">$output_file_name",'-format' => 'fasta');

   		while ( my $aln = $in->next_aln() ) {     
      	#    $aln->map_chars('\-','');     
      	    $out->write_aln($aln);     
		}
		
		if (FilesIO->isFile($output_file_name) eq "true"){
			print "The file $output_file_name was created\n";
		}else{
			print "ERROR: The file $output_file_name NOT was created\n";
		}
 	}
	else{
		print "ERROR: The file $input_file_name not found\n";
	}	
}
############################ sub runModelFromAlign ##################################
sub runModelFromAlign {
	my $model_file_name 	= $_[1];
	my $aln_file_name	= $_[2];
	my $alphabet		= $_[3];
	my $path_program	= $_[4];

	if (FilesIO->isFile($aln_file_name) eq "true"){

		my $cmd = $path_program. $build_model_program. " ".$model_file_name. " -alignfile ". $aln_file_name. " -alphabet $alphabet";
		print $cmd. "\n";
		system $cmd;
	}else{
		print "The file $aln_file_name not found\n";
	}
}

############################ sub runModelFromAlign ##################################
sub runCalibrate {
	my $model_file_name 	= $_[1];
	my $path_program	= $_[2];
	
	if (FilesIO->isFile($model_file_name) eq "true"){
		my $cmd = $path_program. $calibrate_program. " ".$model_file_name." -calibrate 1 -i ". $model_file_name;
		print $cmd. "\n";
		system $cmd;
	}
	else{
		print "The file $model_file_name not found\n";
	}	
}

############################ sub runSearch ##################################
sub runSearch{
	my $model_file_name		= $_[1];
	my $result_file_name		= $_[2];
	my $database_file_name		= $_[3];
	my $parameters			= $_[4];
	my $path_program		= $_[5];

	

	if (FilesIO->isFile($model_file_name) eq "true"){
		my $cmd .= $path_program. $search_program." ".$result_file_name. " -i ". $model_file_name;
		$cmd .=" -db ".$database_file_name. " ".$parameters;
		print $cmd;
		system $cmd;
	}
	else{
		print "file $model_file_name not found.\n";
	}	
}


############################ sub clearModelFileSAM ##################################
sub clearModelFileSAM {

	my $model_file_name 	= $_[1];
	my $path_program	= $_[2];

	my @array_model				= ();
  my $line;
  my $info							= "";
	
	if (FilesIO->isFile($model_file_name) eq "true"){
		@array_model = FilesIO->lineFileToArray($model_file_name);
    if (scalar(@array_model) == 0 ){
			print "ERROR::SAM:: The file $model_file_name is empty";
		}
		else{
    	foreach $line (@array_model){
				if ($line =~ /^[^%]/){
					$info .= $line;
					#print $line;
				}
			}
			return $info;			
    }		
	}
	else{
		print "The file $model_file_name not found\n";
	}	
}

############################ sub runFormatFiletoSam ##################################
sub runHmmscoreToLib {
	my $model_lib       	= $_[1];
	my $output_file_name	= $_[2];
  my $db_size						= $_[3];
	my $db     						= $_[4];
  my $e_max							= $_[5];
	
	if (FilesIO->isFile($model_lib) eq "true"){	
   my $cmd = $search_program ." $output_file_name -modellibrary $model_lib -db_size $db_size -db $db -rdb 1 -Emax $e_max";
   print "file $output_file_name created \n";
	 system $cmd;
  }
	else{
		print "The file $model_lib not found\n";
	}	
}

############################ sub clearResultFileSAMGroup ##################################
sub clearResultFileSAMGroup {

	my $result_file_in 	= $_[1];
	my $result_file_out	= $_[2];

	my @array_result			= ();
	my @array_result_aux  = ();
	my %hash_result				= ();
  my $line;
  my $item;
  my $line_better;
  my $info							= "";
	
	if (FilesIO->isFile($result_file_in) eq "true"){
		@array_result_aux  = FilesIO->lineFileToArray($result_file_in);
    if (scalar(@array_result_aux) == 0 ){
			print "ERROR::SAM:: The file $result_file_in is empty";
		}
		else{
			#cleaning the sam File
    	foreach $line (@array_result_aux){
				if ($line =~ /^[^#]/ && $line =~ /^[^HMM]/ && $line =~ /^[^15S]/ ){
					my @array = split(/\t/,$line);
					if ($array[7] <=1 ) {
						push(@array_result, $line);					
					}
				}
			}
			#search for better model
			foreach $line (@array_result){
					@array_result_aux = split(/\t/,$line);
					my $id_protein = $array_result_aux[2]; 
					my $evalue     = $array_result_aux[7];
					if ($evalue < 1) {
						if (exists($hash_result{$id_protein})){
							@array_result_aux = split(/\t/,$hash_result{$id_protein});
							if ($evalue <= $array_result_aux[7]){
									$hash_result{$id_protein} = $line;
							}							
						}
					  else{
							$hash_result{$id_protein} = $line;
						}				
					}
			}
			my $key ;my $value;
			while (($key, $value) = each(%hash_result)){
				$info.= $value;			
			}
#			print $info;
			FilesIO->saveFile($result_file_out.".tmp",$info);
			FilesIO->sortFile($result_file_out.".tmp",$result_file_out,7);
			FilesIO->removeFile($result_file_out.".tmp");				
    }		
	}
	else{
		print "The file $result_file_in not found\n";
	}	
}

############################ sub runModelFromAlign ##################################
sub runTarget2kHomologs {
	my $sequence_file = $_[1];
	my $output_file   = $_[2];
	my $db_size  		= $_[3];
	my $path_program  = $_[4];
	
	if (FilesIO->isFile($sequence_file) eq "true"){
		my $cmd = $path_program. $target2k_program. " -homologs ".$sequence_file." -tuneup -db_size ". $db_size ." -out " .$output_file; 
		print $cmd. "\n";
		system $cmd;
	}
	else{
		print "The file $sequence_file not found\n";
	}	
}

############################ sub runModelFromAlign ##################################
sub runTarget2k {
	my $sequence_file = $_[1];
	my $output_file   = $_[2];
	my $db  			= $_[3];
	my $path_program  = $_[4];
	
	if (FilesIO->isFile($sequence_file) eq "true"){
		my $cmd = $path_program. $target2k_program. " -seed ".$sequence_file." -db ". $db ." -out " .$output_file; 
		print $cmd. "\n";
		system $cmd;
	}
	else{
		print "The file $sequence_file not found\n";
	}	
}


1
