package BiologicalLP;
use strict;


use String::StringUtil;
use File::Files;
use ILP::Rule;
use ILP::predicate;
use ILP::BiologicalLP;
use Align::Primary::nw_align;
use Util::Hash;
use Time::HiRes qw/gettimeofday/;


############################ sub main ##################################
sub logicalProgramAling {	
	my $seqFileList 	= $_[1];
	my $prog_name		= $_[2];
	my $tmpDir		 	= $_[3];
	my $outputDir	 	= $_[4];
	my $bioToolDir		= $_[5];
	my $sfile		 	= $_[6];
	my $bgFile			= $_[7];
	
	my $input_clustalw = $tmpDir."clustalw.input";
	my $output_clustalw = $outputDir."clustalw.output";
	my $hmmModel = $outputDir."model.hmm";
	
	#align sequences with clustalw
	
	print ">>>LOG:: Preparing input for CLUSTALW for $seqFileList\n";
	my $cmd = "perl ".$ENV{'SCRIPT_DIR'}."/scripts/prepare/prepareInputFileClustalw.pl -sequenceFileList $seqFileList -inputClustalw $input_clustalw";
	#print $cmd."\n";
	system ($cmd);
	
	print ">>>LOG:: Running CLUSTALW for $seqFileList\n";
	$cmd = "perl ".$ENV{'SCRIPT_DIR'}."/scripts/run/runClustalw.pl -outputClustalw $output_clustalw -inputClustalw $input_clustalw";
	system ($cmd);
	
	print ">>>LOG:: Creating Alignment Predicates for $seqFileList\n";
	$cmd = "perl ".$ENV{'SCRIPT_DIR'}."/scripts/prepare/createPredicateAlign.pl -sequenceFileList $seqFileList -predicateDirTestSet $tmpDir/pred/ ";
	$cmd .= " -predicateKind 2 -alignFile $output_clustalw -hmmModelFile $hmmModel -bioToolDir $bioToolDir -discretizationKind 10";
	system ($cmd);
	
	print ">>>LOG:: Creating List From Directory $tmpDir/pred/ for $seqFileList\n";
	$cmd = "perl ".$ENV{'SCRIPT_DIR'}."/scripts/prepare/createListFromDirectory.pl -directory $tmpDir/pred/ -outputFileList $tmpDir/list";
	system ($cmd);
	
	print ">>>LOG:: Creating WARMR Folds for $seqFileList\n";
	$cmd = "perl ".$ENV{'SCRIPT_DIR'}."/warmrExp/scripts/prepare/createFolds.pl -trainPosList $tmpDir/list -kbFile $tmpDir/warmr/$prog_name.kb";
	$cmd .= " -sFile  $sfile  -bgFile $bgFile  output_clustalw -hmmModelFile $hmmModel -bioToolDir $bioToolDir -discretizationKind 10";
	system ($cmd);
	
	print ">>>LOG:: Running WARMR Folds for $seqFileList\n";	
	$cmd = "perl ".$ENV{'SCRIPT_DIR'}."/tildeExp/scripts/run/runTilde.pl -datasetDir $tmpDir/warmr/ -kind warmr3 -ACETildeFile ";
	$cmd .= $ENV{'TILDE_DIR'}."/ACE" ;
	system ($cmd);
	
	print ">>>LOG:: Extracting WARMR Clauses for $seqFileList\n";	
	$cmd = "perl ".$ENV{'SCRIPT_DIR'}."/warmrExp/scripts/prepare/extractWarmrClauses.pl -inFile $tmpDir/warmr/warmr3/$prog_name.freq_queries.out ";
	$cmd .= " -outFile $outputDir/clauses.table  -level 3 " ;
	system ($cmd);
	
}

############################ sub main ##################################
sub compare2BLP {	
	my $seqFile1	 	= $_[1];
	my $seqFile2	 	= $_[2];
	my $alignment	 	= $_[3];
	my $BLPFile1		= $_[4];
	my $BLPFile2		= $_[5];
	my $tmpDir			= $_[6];
	my $verbose			= $_[7];
	
	my $seconds1; my $seconds2; my $diff;
	
	my @array_rules1 = ();				my @array_rules2 = ();
	my $conf_value_comp = 0;			my @array_conf;
	my $conf_value_total = 0;
	
	my %hash_BLP1 = BiologicalLP->readBLPToHash($BLPFile1);
	my %hash_BLP2 = BiologicalLP->readBLPToHash($BLPFile2);
	
	@array_rules1 = FilesIO->fileToArray2d($BLPFile1, ";");
	@array_rules2 = FilesIO->fileToArray2d($BLPFile2, ";");	
	@array_conf = Array->copyColMatrixToArray(scalar(@array_rules1), 3, 2, \@array_rules1);
	$conf_value_total += Array->getSum(\@array_conf);
	@array_conf = Array->copyColMatrixToArray(scalar(@array_rules2), 3, 2, \@array_rules2);
	$conf_value_total += Array->getSum(\@array_conf);

	while (my ($key, $value) = each %hash_BLP1){	
		@array_rules1 = String->StringToArray2d($value, "\n", ";");
		@array_rules2 = String->StringToArray2d($hash_BLP2{$key}, "\n", ";");
		if ($key ne "col"){
			#print $key."\t". $value."\n";
			#$seconds1 = gettimeofday(); 
			$conf_value_comp += BiologicalLP->compareRules(\@array_rules1, \@array_rules2);			
			#$seconds2 = gettimeofday(); $diff = $seconds2 - $seconds1; print "LOG::compareRules Sequential BLPs $diff\n";
			#print "matrix[$i][$j]\n";
		}else{
			#$seconds1 = gettimeofday(); 
			$conf_value_comp += BiologicalLP->compareRulesAlign($seqFile1, $seqFile2, $alignment, \@array_rules1, \@array_rules2);
			#$seconds2 = gettimeofday(); $diff = $seconds2 - $seconds1; print "LOG::compareRules Alignment BLPs $diff\n";
		}
	}	
	
	#print "$conf_value_comp / $conf_value_total \n"; 
	
	$conf_value_comp = $conf_value_comp/$conf_value_total;	
	#print $conf_value_comp;
	return $conf_value_comp;
}

############################ sub main ##################################
sub compareRulesAlign {	
	my ($lixo, $seqFile1, $seqFile2, $alignment, $array_rules1, $array_rules2) = @_;
	my $output;
	
	my %hash_rules1 = BiologicalLP->arrayRulesToHash(\@{$array_rules1});
	my %hash_rules2 = BiologicalLP->arrayRulesToHash(\@{$array_rules2});
	
	my $seq1_name = substr( FilesIO->getNameFile($seqFile1), 0, index(FilesIO->getNameFile($seqFile1), ".".FilesIO->getExt($seqFile1)));
	my $seq2_name = substr( FilesIO->getNameFile($seqFile2), 0, index(FilesIO->getNameFile($seqFile2), ".".FilesIO->getExt($seqFile2)));
	#my $tmpDir = "/tmp/julianab/";
	#$output = $tmpDir."aln_$seq1_name$seq2_name.out";
	$output = $alignment;
	#NW->run($seqFile1, $seqFile2, $output, "false");
	my %hash_aln = NW->alignToHash($output, $seqFile1, $seq1_name, $seqFile2, $seq2_name);
	my @array_aln_1 = split(//,$hash_aln{$seq1_name});
	my @array_aln_2 = split(//,$hash_aln{$seq2_name});
	
	my $count_seq1 = 0;				my $count_seq2 = 0;
	my @array_aln_rules1 = ();		my @array_aln_rules2 = ();
	my $conf_value_comp = 0;
	
	for (my $i=0; $i <scalar(@array_aln_1); $i++){
		if ($array_aln_1[$i] ne "-"){
			$count_seq1++;	
		}
		if ($array_aln_2[$i] ne "-"){
			$count_seq2++;	
		}
		if ($array_aln_1[$i] ne "-" && $array_aln_2[$i] ne "-"){
			if (exists($hash_rules1{$count_seq1}) &&  exists($hash_rules2{$count_seq2})){
				#print ">>> R1".$hash_rules1{$count_seq1}."\n";
				#print ">>> R2".$hash_rules2{$count_seq2}."\n";
				$hash_rules1{$count_seq1} =~ s/\,$count_seq1\)/\,X\)/g;
				@array_aln_rules1 = String->StringToArray2d($hash_rules1{$count_seq1}, "\n", ";");
				$hash_rules2{$count_seq2} =~ s/\,$count_seq2\)/\,X\)/g;
				@array_aln_rules2 = String->StringToArray2d($hash_rules2{$count_seq2}, "\n", ";");
				$conf_value_comp += BiologicalLP->compareRules(\@array_aln_rules1, \@array_aln_rules2);			
			}			
		}
	}
	
	#my @array_aln = Hash->hashToArray(\%hash_aln);
	return $conf_value_comp;
}

############################ sub main ##################################
sub arrayRulesToHash {	
	my ($lixo, $array_rules) = @_;
	my @body = ();			my $rule;
	my @termrs;				my %hash_rules = ();
	my $position;			my $confidence;
	
	for (my $i = 0; $i < scalar(@{$array_rules}); $i++){
		$rule = @{$array_rules}->[$i]->[0];
		$confidence = @{$array_rules}->[$i]->[1];
		@body =  Rule->getBody($rule);
		#print $body[0]."\n";
		@termrs = Predicate->getTermsFromPredicate($body[0]);
		if (scalar(@termrs) != 3){
			print "ERROR::BiologicalLP::arrayRulesToHash:: $body[0] has not 3 termrs\n";
			exit;
		}
		$position = $termrs[2];
		if (exists($hash_rules{$position})){
			$hash_rules{$position} .= "\n". $rule.";".$confidence;
		}else{
			$hash_rules{$position} = $rule.";".$confidence;
		}		
	}
	return %hash_rules;
}
############################ sub main ##################################
sub compareRules {	
	my ($lixo, $array_rules1, $array_rules2) = @_;
	
	my $value = 0;
	
	
	for (my $i = 0; $i < scalar(@{$array_rules1}); $i++){
		for (my $j = 0; $j < scalar(@{$array_rules2}); $j++){
			if (@{$array_rules1}->[$i]->[0] eq @{$array_rules2}[$j]->[0]){
				$value += @{$array_rules1}->[$i]->[1] + @{$array_rules2}->[$j]->[1];
				#print "compare\t".@{$array_rules1}->[$i]->[0]."\t".@{$array_rules1}->[$i]->[1]."\t".@{$array_rules2}[$j]->[0]."\t".@{$array_rules2}->[$j]->[1]."\t".$value."\n";
				$j = scalar(@{$array_rules2});
			}
		}
	}	
	return $value;	
}


############################ sub main ##################################
sub readBLPToHash {	
	my $BLPFile		= $_[1];
	
	my $entry;			my @array_aux = ();
	my $rule;			my @body;
	my $pred_name;		my %hash_BLP = ();
	my $confidence;
	
	my @array = FilesIO->lineFileToArray($BLPFile);
	
	
	foreach $entry (@array){
		$entry = String->removeCaracter($entry, "\n");
		@array_aux = split(/;/, $entry);
		
		$rule = String->removeCaracter(String->removeCaracter($array_aux[1],".")," ");
		$confidence = $array_aux[2];
		@body = Rule->getBody($rule);
		$pred_name = Predicate->getNamePredicate($body[0]);
		if (exists($hash_BLP{$pred_name})){
			$hash_BLP{$pred_name} .= "\n". $rule.".;".$confidence;
		}else{
			$hash_BLP{$pred_name} =  $rule.".;".$confidence;
		}
	}
	
	return %hash_BLP;	
}



1