package Rule;
use strict;

use String::StringUtil;
use File::Files;
use ILP::Rule;
use ILP::predicate;
use Util::Array;
use SSM::matrix;


############################ sub run ##################################
sub consultExample {
	my $rulesFile		= $_[1];
	my $nameSequence	= $_[2];
	my $pred_file		= $_[3];
	my $verbose			= $_[4];
	my $info= "";
	my $count;
	
	my @array_rules = FilesIO->fileToArray2d($rulesFile, ";");

	for (my $i=0; $i<scalar(@array_rules); $i++){
			#print "rule".$array_rules[$i][1]."\n";			
			$count = Rule->consultOneExample($pred_file, $nameSequence, $array_rules[$i][1], $verbose);
			$info .= $count.",";
			#print $count;
	}
	return $info;
}

############################ sub run ##################################
sub consultOneExample {
	my $predicateFile	= $_[1];
	my $nameSequence	= $_[2];
	my $rule    	 	= $_[3];
	my $verbose			= $_[4];
	
	my @array = FilesIO->lineFileToArray($predicateFile);
	my $pred;my $info = "";
	my $i = 0; my $found =  "false"; my $j=0;
	my @array_aux = split(/\:\-/, $rule);
	my $fact_search = $array_aux[1];
	my $count = 0;
	@array_aux = split(/\)\,/, $fact_search); 
	
	for ($j=0; $j<scalar(@array_aux); $j++){
		$i = 0;
		$fact_search = $array_aux[$j];
		if ($j != scalar(@array_aux)-1){
			$fact_search .= ").";			
		}	
		$fact_search =~ s/A\,/$nameSequence\,/;	
		$fact_search = lc(String->removeCaracter($fact_search, " "));
		#print "fact search ".$fact_search."\n";
		while ($i < scalar(@array)){
			#print lc(String->removeCaracter($array[$i],"\n").".")."\n";
			if ($fact_search eq lc(String->removeCaracter($array[$i],"\n").".")){
				$count++;
				$i = scalar(@array);
			}
			$i++;		
		}
	}
	if ($count == $j){
		return 1;
	}else{
		return 0;
	}
}

############################ sub run ##################################
sub subsumes {
	
	my $ruleE 			= $_[1];
	my $ruleG			= $_[2];
	my $verbose			= $_[3];
	
	my @array_ruleE = Rule->getHead($ruleE);
	my @array_ruleG = Rule->getHead($ruleG);
	
	if (Array->readArrayToString(scalar(@array_ruleE), ",", \@array_ruleE) ne Array->readArrayToString(scalar(@array_ruleG), ",", \@array_ruleG)){
		return "false";	
	}
	@array_ruleE = Rule->getBody($ruleE);
	@array_ruleG = Rule->getBody($ruleG);
	
	my $count = 0; my $iten;
	my $body_ruleG =  Array->readArrayToString(scalar(@array_ruleG), ",", \@array_ruleG);
	foreach $iten (@array_ruleE){		
		if (index($body_ruleG, $iten) >=0){
			$count++;
		}		
	}
	
	if ($count == scalar(@array_ruleE)){
		return "true";	
	}	
	return "false";	
}
############################ sub run ##################################
sub entailExamples {

	my $rulesFile		= $_[1];
	my $predicateFile	= $_[2];
	my $bgFile			= $_[3];
	my $verbose			= $_[4];
	my $info= "";
	my $count;
	my @array_rules = FilesIO->fileToArray2d($rulesFile, ";");

	for (my $i=0; $i<scalar(@array_rules); $i++){
			#print "rule".$array_rules[$i][1]."\n";			
			$count = Rule->entail($array_rules[$i][1], $predicateFile, $bgFile, $verbose);
			$info .= $count.",";
			#print $count."\n";
	}
	return $info;
	
}

########################### sub run ##################################
sub entailYap {
	

	my $rule			= $_[1];
	my $predicateFile	= $_[2];
	my $bgFile			= $_[3];
	my $tmpDir			= $_[4];
	my $predProve		= $_[5];
	my $verbose			= $_[6];
	
	my $info = ""; my $pred;
	
	$rule = String->removeCaracter(String->removeCaracter($rule,".")," ");
	
	
	$info .= "assert(($rule)).\n\n";
	
	my @kb =  FilesIO->lineFileToArray($predicateFile);
	my @bg =  FilesIO->lineFileToArray($bgFile);
	
	push(@kb, @bg);	
	
	foreach $pred (@kb){
		$pred = String->replace2($pred,".", "");
		$pred = String->removeCaracter($pred,"\n");
		$pred = lc($pred);
		$info .= "assert(($pred)).\n\n";	
	}
	$info .= "call($predProve).\n";
	
	$info = "echo \"$info \" |yap -s5000 -h20000 \&> ". $tmpDir."yap.out";
	
	my $YAP_runnable = $tmpDir."yap.sh";
	FilesIO->saveFile($YAP_runnable, $info, $verbose);
	
	
	chdir $tmpDir;
	my $cmd = "bash $YAP_runnable";
	if ($verbose eq "true"){
		print $cmd."\n";
	}
	
	system ($cmd);
	
	if (FilesIO->isFile($tmpDir."yap.out") eq "false"){
		print "ERROR::perlLib::ILP::Rule:: File  ".$tmpDir."yap.out"." NOT created\n";
	}
	
	my @array = FilesIO->lineFileToArray($tmpDir."yap.out");
	if (index($array[scalar(@array)  -1], "no") >=0){
		return "0";
	}else{
		return "1";
	}
}

########################### sub run ##################################
sub entailWeigth {
	my $rule			= $_[1];
	my $predicateFile	= $_[2];
	my $sequenceFile	= $_[3];
	my $matrixFile		= $_[4];
	my $tmpDir			= $_[5];
	my $verbose			= $_[6];
	
	$rule = String->removeCaracter(String->removeCaracter($rule,".")," ");
	
	my @body = Rule->getBody($rule);
	
	if (scalar(@body) > 1){
		print "ERROR::perlLib::ILP::Rule::The body of rule $rule is big than 1\n";
		exit;
	}
	my $pred_name = Predicate->getNamePredicate($body[0]);
	my @terms_body = Predicate->getTermsFromPredicate($body[0]);
	
	#print "\nDEBUG ". $pred_name. " rule = ". $rule."\n";
	if (uc($pred_name) eq uc("col")){
		return Rule->entailcolPredicate($body[0], $predicateFile, $matrixFile, $verbose);
	}elsif (uc($pred_name) eq uc("aminoacidpairratio")) {
		my $pair = $terms_body[1] ;
		#print "Fact $pred_name\n";
		#print "pair $pair\n";
		#print "count pair ". Protein->createAminoacidPairRatio($sequenceFile, $pair)."\n";
		return Protein->createAminoacidPairRatio($sequenceFile, $pair);		
	}elsif (uc($pred_name) eq uc("aminoacidratio")){
		my $amino = $terms_body[1] ;
		#print "Fact $pred_name\n";
		#print "amino $amino\n";
		#print "amino rate ". Protein->createAminoacidRatio($sequenceFile, $amino)."\n";		
		return Protein->createAminoacidRatio($sequenceFile, $amino);		
	}else{
		#print "Fact $pred_name\n";
		#print "property rate ". Protein->findPercent($sequenceFile, $pred_name)."\n";		
		return Protein->findPercent($sequenceFile, $pred_name);
	}	
}

########################### sub run ##################################
sub scoreBLP {
	my ($lixo, $rule, $predicateFile, $confidence, $bgfile, $verbose, $matrix) = @_;
	my @localMatrix = @{$matrix};
	
	my $score;			my $pair;
	my $amino;
	
	$rule = String->removeCaracter(String->removeCaracter($rule,".")," ");
	
	my @body = Rule->getBody($rule);
	
	my $pred_name = Predicate->getNamePredicate($body[0]);
	my @terms_body = Predicate->getTermsFromPredicate($body[0]);
	
	#print "\nDEBUG ". $pred_name. " rule = ". $rule."\n";
	if (uc($pred_name) eq uc("col")){
		if (scalar(@body) == 1){
			$score =  Rule->entailcolPredicate($body[0], $predicateFile, $verbose, \@localMatrix);			
		}else{
			$score = Rule->entail($rule, $predicateFile,$bgfile,$verbose);
		}				
	}elsif (uc($pred_name) eq uc("aminoacidpairratio")) {
			$pair = $terms_body[1] ;
			$score = Rule->entailAminoRatioPredicateFast($body[0], $predicateFile, $pred_name, $pair, $verbose);	
	}elsif (uc($pred_name) eq uc("aminoacidratio")){
			$amino = $terms_body[1] ;
			$score = Rule->entailAminoRatioPredicateFast($body[0], $predicateFile,  $pred_name, $amino, $verbose);	
	}else{
		$score = Rule->entailPercentPredicateFast($body[0], $predicateFile, $pred_name, $verbose);
	}	
	$score = $score*$confidence;
	return $score;
}


########################### sub run ##################################
sub entailFast {
	my $rule			= $_[1];
	my $predicateFile	= $_[2];
	my $sequenceFile	= $_[3];
	my $tmpDir			= $_[4];
	my $predProve		= $_[5];
	my $verbose			= $_[6];
	my $bgfile			= $_[7];
	
	$rule = String->removeCaracter(String->removeCaracter($rule,".")," ");
	
	my @body = Rule->getBody($rule);
	my $pred_name = Predicate->getNamePredicate($body[0]);
	
	if (scalar(@body) > 1){
		if (uc($pred_name) eq uc("col")){
			return Rule->entail($rule, $predicateFile,$bgfile,$verbose);
		}else{
			print $pred_name."\n";
			print "ERROR::perlLib::ILP::Rule::The body of rule $rule is big than 1\n";
			exit;
		}		
	}else{
		my @terms_body = Predicate->getTermsFromPredicate($body[0]);

		if (uc($pred_name) eq uc("col")){
			return Rule->entailcolPredicateFast($body[0], $predicateFile, $verbose);
		}elsif (uc($pred_name) eq uc("aminoacidpairratio")) {
			my $pair = $terms_body[1] ;
			return Rule->entailAminoRatioPredicateFast($body[0], $predicateFile, $pred_name, $pair, $verbose);	
		}elsif (uc($pred_name) eq uc("aminoacidratio")){
			my $amino = $terms_body[1] ;
			return Rule->entailAminoRatioPredicateFast($body[0], $predicateFile,  $pred_name, $amino, $verbose);	
		}else{
			return Rule->entailPercentPredicateFast($body[0], $predicateFile, $pred_name, $verbose);
		}		
	}	
}
############################ sub run ##################################
sub entailcolPredicate {
	my ($lixo, $body, $predicateFile, $verbose, $matrix) = @_;
	my $pred;
	my @kb =  FilesIO->lineFileToArray($predicateFile);
	my @terms;
	my @terms_body = Predicate->getTermsFromPredicate($body);
	
	my @matriz_array = @{$matrix};
	
	foreach $pred (@kb){
		if (index($pred, "col(") >=0){			
			@terms = Predicate->getTermsFromPredicate($pred);			
			if ($terms_body[2] eq  $terms[2]){
				#print "Fact ".$pred;
				#print "aa rule ".$terms_body[1]."\n";	
				#print "aa fact ".$terms[1]."\n";
				if ($terms_body[1] eq  $terms[1]){
					#print "weigth = 1\n";	
					return 1;
				}else{
									
					my @array_amino =  Matrix->getAminos($terms_body[1], 24, \@matriz_array);
					my @array_aux = Array->copyColMatrixToArray(24, 2, 1 ,\@array_amino);
					my $smallerMat = Array->getSmaller(\@array_aux);
					@array_amino = Array->transladeMatrix(($smallerMat *-1) +1 ,1, \@array_amino);
					@array_aux = Array->copyColMatrixToArray(24, 2, 1 ,\@array_amino);
					my $bigger = Array->getBigger(\@array_aux);					
					my $value_amino = Matrix->getValueForAmino($terms[1], \@array_amino);
					my $weigth = $value_amino/$bigger;					
					#print "weigth =". $weigth."\n";	
					return $weigth;
				}
			}
		}		
	}
	
	#print "WARNING::perlLib::ILP::Rule::There is not predicate $body in $predicateFile\n";
	#print "weigth =0\n";	
	return 0;
}

############################ sub run ##################################
sub entailPercentPredicateFast {
	my $body			= $_[1];
	my $predicateFile	= $_[2];
	my $predProveName   = $_[3];
	my $verbose			= $_[4];
	my $pred;
	my @kb =  FilesIO->lineFileToArray($predicateFile);
	my @terms;
	my @terms_body = Predicate->getTermsFromPredicate($body);
	
	foreach $pred (@kb){
		if (index(uc($pred), uc("$predProveName(")) >=0){			
			@terms = Predicate->getTermsFromPredicate($pred);			
			if ($terms_body[1] eq  $terms[1]){
				return 1;
			}
		}		
	}	
	#print "WARNING::perlLib::ILP::Rule::entailcolPredicateFast:: There is not predicate $body in $predicateFile\n";
	#print "weigth =0\n";	
	return 0;
}

############################ sub run ##################################
sub entailAminoRatioPredicateFast {
	my $body			= $_[1];
	my $predicateFile	= $_[2];
	my $predProveName   = $_[3];
	my $amino			= $_[4];
	my $verbose			= $_[5];
	my $pred;
	my @kb =  FilesIO->lineFileToArray($predicateFile);
	my @terms;
	my @terms_body = Predicate->getTermsFromPredicate($body);
	
	foreach $pred (@kb){
		#print "$pred\n";
		if (index(uc($pred), uc("$predProveName(")) >=0){			
			@terms = Predicate->getTermsFromPredicate($pred);			
			if ($terms_body[1] eq  $terms[1]){
				return 1;
			}
		}		
	}	
	#print "WARNING::perlLib::ILP::Rule::entailcolPredicateFast:: There is not predicate $body in $predicateFile\n";
	#print "weigth =0\n";	
	return 0;
}



############################ sub run ##################################
sub entailcolPredicateFast {
	my $body			= $_[1];
	my $predicateFile	= $_[2];
	my $verbose			= $_[3];
	my $pred;
	my @kb =  FilesIO->lineFileToArray($predicateFile);
	my @terms;
	my @terms_body = Predicate->getTermsFromPredicate($body);
	
	foreach $pred (@kb){
		if (index($pred, "col(") >=0){			
			@terms = Predicate->getTermsFromPredicate($pred);			
			if ($terms_body[2] eq  $terms[2]){
				#print "Fact ".$pred;
				#print "aa rule ".$terms_body[1]."\n";	
				#print "aa fact ".$terms[1]."\n";
				if ($terms_body[1] eq  $terms[1]){
					#print "weigth = 1\n";	
					return 1;
				}else{
					return 0;
				}
			}
		}		
	}
	
	#print "WARNING::perlLib::ILP::Rule::entailcolPredicateFast:: There is not predicate $body in $predicateFile\n";
	#print "weigth =0\n";	
	return 0;
}

############################ sub run ##################################
sub entail {
	

	my $rule			= $_[1];
	my $predicateFile	= $_[2];
	my $bgFile			= $_[3];
	my $verbose			= $_[4];
	
	#assert(rule).
	#assert(bg). carrega cada fato do bg.
	#call(fact). tenta provar pelo menos  1 fato se alguma resposta for yes ta provado, caso contrrio nao foi provado.

	my @terms = (); my @terms_fact = (); 
	my $name_pred; my $pred; my $i; my $found; my $proved = "false";
	my %hash_vars = ();
		
	my @array_head = Rule->getHead($rule);
	my @array_body = Rule->getBody($rule);
	my @kb =  FilesIO->lineFileToArray($predicateFile);
	my @bg =  FilesIO->lineFileToArray($bgFile);
	
	push(@kb, @bg);	
	
	foreach $pred (@array_body){
		@terms = Predicate->getTermsFromPredicate($pred);
		$name_pred = Predicate->getNamePredicate($pred);
		$found = "false"; 
		$i = 0;
		#print "pred == > $pred\n";
		if (keys(%hash_vars) > 0){
			for (my $j =0; $j <scalar(@terms); $j++){
				if (exists($hash_vars{$terms[$j]})){
					$terms[$j] = $hash_vars{$terms[$j]};
				}
			}
		}
				
		while ($i < scalar(@kb) && $found eq "false"){
			#print "fact $i  == > $kb[$i]\n";
			if(index($kb[$i], $name_pred) >=0){				
				@terms_fact =  Predicate->getTermsFromPredicate($kb[$i]);
				for (my $j =0; $j <scalar(@terms); $j++){
					#print "Term Pred --$terms[$j]--< == > Term Fact --$terms_fact[$j]--\n";
					if ($terms[$j] =~ m/[A-Z]/) { 
						if (!exists($hash_vars{$terms[$j]})){
							$hash_vars{$terms[$j]} = $terms_fact[$j];	
						}						
					}elsif (String->removeCaracter($terms[$j],"\n") eq String->removeCaracter($terms_fact[$j],"\n")){
						$found = "true";
						$proved = "true";
						#print "proved  == > $kb[$i]\n";
					}elsif ($proved eq "false"){
						%hash_vars = ();
						$j = scalar(@terms);
					}
				}
			}
			$i++;
		}
		
		if ($found eq "false"){
			return "0";
		}		
	}
	return "1";
}



############################ sub run ##################################
sub getHead {
	
	my $rule		= $_[1];
	my $verbose		= $_[2];
	
	my @array_rule = split(/\:\-/, $rule);	
	my $head_rule = String->removeCaracter($array_rule[0], " ");
	my @array_head = split(/,/, $head_rule);
	return @array_head;
	
}

############################ sub run ##################################
sub getBody {
	
	my $rule		= $_[1];
	my $verbose		= $_[2];
	my $iten;
	my @array_body_new =();
	
	my @array_rule = split(/\:\-/, $rule);	
	my $body_rule = String->removeCaracter(String->removeCaracter($array_rule[1], "."), " ");
	my @array_body = split(/\),/, $body_rule);
	
	foreach $iten (@array_body){		
		if (index($iten, ")") < 0){
			$iten = $iten.")";
		}
		push(@array_body_new, $iten);
	}
	return @array_body_new;
	
}


1