package Aleph;
use strict;

use String::StringUtil;
use File::Files;
use SSM::matrix;
use Util::Array;
use analisys::analisys;

my $TOKEN_START_GC = "[-------------------------------------]";
my $TOKEN_GOOD_CLAUSE ="[good clause]";
my $TOKEN_COVERED = "[covered]";
my $TOKEN_NOT_COVERED ="[not covered]";

my $TRAIN_POS_FILE = "train.f";
my $TRAIN_NEG_FILE = "train.n";
my $TRAIN_BG_FILE = "train.b";

my %hash_result = ();
my %hash_rules = ();

############################ sub run ##################################
sub run {
	my $alephRunFile = $_[1];
	my $datasetDir	 = $_[2];
	my $file_result	 = $_[3];
	my $test		 = $_[4];
	my $verbose		 = $_[5];
	
	my $info = "echo \"[aleph]. read_all(train). induce. write_rules(teory). $test \" |yap -s5000 -h20000 > aleph.out";
	my $aleph_runnable = $datasetDir."aleph.sh";
	
	#test(testP, show, covered, total). test(testN, show, covered, total).
	
	FilesIO->saveFile($aleph_runnable, $info, $verbose);
	
	chdir $datasetDir;
	my $cmd = "bash $aleph_runnable";
	if ($verbose eq "true"){
		print $cmd."\n";
	}
	
	system ($cmd);
	
	if (FilesIO->isFile($file_result) eq "false"){
		print "ERROR::perlLib::ILP::Aleph:: File  $file_result NOT created\n";
	}
	elsif ($verbose eq "true"){
		print "LOG::perlLib::ILP::Aleph:: File $file_result succefuly created\n";
	}
	
	FilesIO->removeFile($aleph_runnable, $verbose);
}	

############################ sub run ##################################
sub saveGoodRules {
	my $alephFile  		= $_[1];
	my $lenClause 		= $_[2];
	my $tableFile	 	= $_[3];
	my $totalPos		= $_[4];
	my $totalNeg		= $_[5];
	my $datasetDir		= $_[6];
	my $alephRunFile    = $_[7];
	my $verbose		 	= $_[8];
	
	my $clause; my $length_total = 0;
	my $line ; my $pos_cover; my $neg_cover;
	my $sta;
	my $start = "false";
	my $info = ""; my $info_table = "";
	my $count_rules =0;
	my @array_info; my @array_aux;
	
	FilesIO->copyFile($datasetDir.$TRAIN_POS_FILE, $datasetDir."trainPos",$verbose);
	FilesIO->copyFile($datasetDir.$TRAIN_NEG_FILE, $datasetDir."trainNeg",$verbose);
	
	if (open(FILEIN, "$alephFile")){
		while (($line = <FILEIN>) && ($length_total <= $lenClause)) {
			#print $line;
			if (index($line, $TOKEN_GOOD_CLAUSE) >=0){
				$start = "true";			
			}
			if ($start eq "true"){
				$info .= $line;
			}
			if ($start eq "true" && (index($line, $TOKEN_START_GC) >=0)){
				$start = "false";
				$count_rules++;
				@array_info = split(/\n/, $info);
				
				$clause = $array_info[1]." ".$array_info[2];
				$length_total = scalar(split(/\)\,/, $clause));
				if ($length_total <= $lenClause){
					@array_aux =  split(/= /, $array_info[3]);
					@array_aux = split(/ +/,$array_aux[1]);
					$pos_cover = $array_aux[0];
					@array_aux =  split(/= /, $array_info[3]);
					@array_aux = split(/\]/,$array_aux[2]);
					$neg_cover = $array_aux[0];			
					$neg_cover = 0;			
					#$pos_cover= Aleph->getNumberCover($clause, "trainPos", $datasetDir, $alephRunFile, $verbose);
					#$neg_cover= Aleph->getNumberCover($clause, "trainNeg", $datasetDir, $alephRunFile, $verbose);
					$sta = 	Analisys->calculeFMeasure($pos_cover, $neg_cover, $totalPos -  $pos_cover);
					#FilesIO->saveFile($dirOut."rule$count_rules", $clause, $verbose);
					$info_table .= "rule$count_rules\t".$clause."\t".$sta."\n";
				}
				
				$info = "";
			}			
		}
		close FILEIN;
		FilesIO->saveFile($tableFile, $info_table, $verbose);
	}	
}


############################ sub run ##################################
sub search {
	my $tableFile  		= $_[1];
	my $alephRunFile 	= $_[2];
	my $datasetDir	 	= $_[3];
	my $resultFileOutput= $_[4];
	my $verbose		 	= $_[5];
	
	my $tmpFile = "tmp";
	my $entry; my $file_result = $datasetDir."result.out";
	chdir $datasetDir;
	
	my @array_clauses = FilesIO->lineFileToArray($tableFile);	
	my @array_line;
	foreach $entry (@array_clauses) {
		@array_line = split(/\t/, $entry);
		print $entry;
		FilesIO->saveFile($tmpFile, $array_line[1], $verbose);
		Aleph->run_search($alephRunFile, $datasetDir, $file_result, $tmpFile, "test(testP, show, covered, total). test(testN, show, covered, total).", $verbose);
		Aleph->calc_evalue($file_result, "family", String->removeCaracter($array_line[2], "\n"), $array_line[1], $verbose);		
	}
	
	my $info = "";
	while( my ($k, $v) = each %hash_result) {
        #$info .= $k.";".$v.";". $hash_rules{$k}."\n";
        $info .= $k.";".$v."\n";
    }
    FilesIO->saveFile($tmpFile, $info, $verbose);
    FilesIO->sortFileOrderDesc($tmpFile, $resultFileOutput, 2,";");

	FilesIO->removeFile($tmpFile, $verbose);
	FilesIO->removeFile($file_result, $verbose);
}

############################ sub run ##################################
sub calc_evalue {
	my $file_result = $_[1];
	my $pred_head   = $_[2];
	my $value		= $_[3];
	my $rule		= $_[4];
	my $verbose		= $_[5];
	
	my @array_result = FilesIO->lineFileToArray($file_result);	
	my $entry; my $id_prot;
	my @array_line;
	foreach $entry (@array_result) {
		if (index($entry, $pred_head) >=0){
			if (index($entry, $TOKEN_COVERED) >=0){
				@array_line = split(/\(/, $entry);
				$id_prot = String->removeCaracter($array_line[1], ")");
				$id_prot = String->removeCaracter($id_prot, "\n");
				
				if (!exists($hash_result{$id_prot})){
					$hash_result{$id_prot} = $value;
					$hash_rules{$id_prot} = $rule;
				}else{
					$hash_result{$id_prot} = $hash_result{$id_prot} + $value;
					$hash_rules{$id_prot} .= "\t".$rule;
				}				
			}
		}
	}
}

############################ sub run ##################################
sub run_search {
	my $alephRunFile = $_[1];
	my $datasetDir	 = $_[2];
	my $file_result	 = $_[3];
	my $fileTheory	 = $_[4];
	my $fileConsult	 = $_[5];
	my $verbose		 = $_[6];
	
	my $info = "echo \"[aleph]. read_all(train). consult($fileTheory). $fileConsult \" |yap -s5000 -h20000 > $file_result";
	#my $info = "echo \"[aleph]. read_all(train). consult($fileTheory). test(testP, show, covered, total). \" |yap -s5000 -h20000 > $file_result";
	my $aleph_runnable = $datasetDir."aleph.sh";
	
	#test(testP, show, covered, total). test(testN, show, covered, total).
	
	FilesIO->saveFile($aleph_runnable, $info, $verbose);
	
	chdir $datasetDir;
	my $cmd = "bash $aleph_runnable";
	if ($verbose eq "true"){
		print $cmd."\n";
	}
	
	system ($cmd);
	
	if (FilesIO->isFile($file_result) eq "false"){
		print "ERROR::perlLib::ILP::Aleph:: File  $file_result NOT created\n";
	}
	elsif ($verbose eq "true"){
		print "LOG::perlLib::ILP::Aleph:: File $file_result succefuly created\n";
	}
	
	FilesIO->removeFile($aleph_runnable, $verbose);
}

############################ sub run ##################################
sub review {
	my $tableFile  		= $_[1];
	my $matrixFile		= $_[2];
	my $alephRunFile 	= $_[3];
	my $datasetDir	 	= $_[4];
	my $totalPos		= $_[5];
	my $totalNeg		= $_[6];
	my $verbose		 	= $_[7];
	my $entry;
	my $id_prot; my $count_cover_pos = 0; my $count_cover_neg = 0; my $count = 0; my $count_rule = 0;
	my $rule; my $value_rule; my $amino; my $rule_selected;
	my @matriz_array; my @array_amino_pos; my @array_amino;
	my @array_cover = ();
	my $info = "";
	
	FilesIO->copyFile($datasetDir.$TRAIN_POS_FILE, $datasetDir."trainPos",$verbose);
	FilesIO->copyFile($datasetDir.$TRAIN_NEG_FILE, $datasetDir."trainNeg",$verbose);
	
	@matriz_array = Matrix->read($matrixFile);
	my $smallerMat = Array->getSmallerMatrix(20, 20, \@matriz_array);
	
	my @array_clauses = FilesIO->lineFileToArray($tableFile);	
	my @array_line;
	foreach $entry (@array_clauses) {
		print $entry;
		@array_line = split(/\t/, $entry);
		$count_rule = $array_line[0];
		$rule = $array_line[1];
		$value_rule = String->removeCaracter($array_line[2], "\n");
		@array_line = split(/\,/, $rule);
		$amino = $array_line[1];
		
		@array_amino_pos =  Matrix->getAminoPositive($amino, \@matriz_array);
		@array_amino =  Matrix->getAminos($amino, \@matriz_array);
		@array_amino = Array->transladeMatrix(($smallerMat *-1) +1 ,1, \@array_amino);
		@array_cover = ();
		$count = 0;
		for (my $i = 0; $i< scalar(@array_amino); $i++){
			$rule_selected = String->replace($rule, ",".$amino, ",".lc($array_amino[$i][0]));
			$count_cover_pos = Aleph->getNumberCover($rule_selected, "trainPos", $datasetDir, $alephRunFile, $verbose);
			if ($count_cover_pos > 0 ||  Array->isInArray($array_amino[$i][0],\@array_amino_pos) eq "true"){
				$array_cover[$count][0] = lc($array_amino[$i][0]);
				$count_cover_neg = Aleph->getNumberCover($rule_selected, "trainNeg", $datasetDir, $alephRunFile, $verbose);
				$count_cover_neg = 0;
				if (Array->isInArray($array_amino[$i][0],\@array_amino_pos) eq "true"){
					$array_cover[$count][1] = $value_rule;
				}
				else{
					$array_cover[$count][1] = Analisys->calculeFMeasure($count_cover_pos, $count_cover_neg, $totalPos -  $count_cover_pos);
				}				
				$array_cover[$count][2] = $array_amino[Matrix->getCodeFromAmino(lc($array_amino[$i][0]))][1];
				$array_cover[$count][3] = $count_rule.".".$count."\t".$rule_selected;
				$count++;
			}			
		}		
		$info .= Aleph->CalculeWeigthClauses($value_rule, \@array_cover);	
	}		
	FilesIO->saveFile($tableFile.".rvw", $info, $verbose);
}

############################ sub run ##################################
sub CalculeWeigthClauses {
		my ($lixo, $maxCover, $array_cover) = @_;
		my $sum_fat_subs = 0;
		my $info = "";
		
		for (my $i = 0; $i <scalar(@{$array_cover}); $i++){#i = row
			$sum_fat_subs += @{$array_cover}->[$i]->[2];
		}

		for (my $i = 0; $i <scalar(@{$array_cover}); $i++){#i = row
			@{$array_cover}->[$i]->[4]  = $maxCover*(@{$array_cover}->[$i]->[2]/$sum_fat_subs) + @{$array_cover}->[$i]->[1];
			$info .= @{$array_cover}->[$i]->[3]."\t".@{$array_cover}->[$i]->[4] ."\n";
		}
		
		return $info;
		

}
############################ sub run ##################################
sub CalculeCover {
	my $file_result = $_[1];
	my $verbose		= $_[2];
	
	my @array_result = FilesIO->lineFileToArray($file_result);	
	my $entry; my $count_cover = 0;
	my @array_line;
	foreach $entry (@array_result) {
		if (index($entry, $TOKEN_COVERED) >=0){
				$count_cover++;						
		}
	}
	return $count_cover;
}

############################ sub run ##################################
sub getNumberCover {
	my $rule			= $_[1];
	my $fileToConsult 	= $_[2];
	my $datasetDir      = $_[3];
	my $alephRunFile    = $_[4];
	my $verbose			= $_[5];

	my $file_result = $datasetDir."result.out";
	my $tmpFile = $datasetDir."tmp";
	
	FilesIO->saveFile($tmpFile, $rule, $verbose);
	Aleph->run_search($alephRunFile, $datasetDir, $file_result, "tmp", "test($fileToConsult, show, covered, total).", $verbose);
	my $count_cover = Aleph->CalculeCover($file_result, $verbose);
	
	FilesIO->removeFile($tmpFile, $verbose);
	FilesIO->removeFile($file_result, $verbose);
	return $count_cover;
}

############################ sub run ##################################
sub getMaxWeigthRuler {
	my $rulesFile			= $_[1];
	
	my $rule_name = "";my $rule_name_previus = "";
	my $value_max = 0;
	my @array_aux;
	my $value;
	my $sum = 0;
	
	my @array_rules = FilesIO->fileToArray2d($rulesFile, "\t");
	
	for (my $i=0; $i<scalar(@array_rules); $i++){
		@array_aux = split(/\./, $array_rules[$i][0]);
		$rule_name = $array_aux[0];
		if ($rule_name eq $rule_name_previus){
			if ($array_rules[$i][2] > $value_max){
				$value_max = $array_rules[$i][2];
			}
		}else{
			if ($rule_name_previus ne ""){
				$sum += $value_max;
				#print "max $value_max soma $sum \n";				
			}
			$value_max = $array_rules[$i][2];						
		}
		
		#print "rC $rule_name r Ant $rule_name_previus value = $array_rules[$i][2] valueMax $value_max\n";
		
		$value = $array_rules[$i][2];
		$rule_name_previus = $rule_name;
		
		
	}
	$sum += $value_max;		
	#print "oi $sum";
}


############################ sub run ##################################
sub getSumRuler {
	my $rulesFile			= $_[1];
	
	my $rule_name = "";my $rule_name_previus = "";
	my $value_max = 0;
	my @array_aux;
	my $value;
	my $sum = 0;
	
	my @array_rules = FilesIO->fileToArray2d($rulesFile, ";");
	
	for (my $i=0; $i<scalar(@array_rules); $i++){
		$sum += String->removeCaracter(substr($array_rules[$i][2], 0, length($array_rules[$i][2]) -1), "\n");		
	}
	return $sum;		
	#print "oi $sum";
}
############################ sub run ##################################
sub consultExample {
	my $predicateFile	= $_[1];
	my $nameSequence	= $_[2];
	my $datasetDir      = $_[3];
	my $alephRunFile 	= $_[4];
	my $rule    	 	= $_[5];
	my $verbose			= $_[6];
	
	my @array = FilesIO->lineFileToArray($predicateFile);
	my $pred;my $info = "";
	
	foreach $pred (@array){ 
		$pred = String->replace2($pred,".", "");
		$info .= String->removeCaracter($pred,"\n").".\n";			
	}
	FilesIO->saveFile($datasetDir.$TRAIN_BG_FILE, $info, $verbose);
	FilesIO->saveFile($datasetDir."testT", $nameSequence, $verbose);
	
	my $count_cover = Aleph->getNumberCover($rule, "testT", $datasetDir, $alephRunFile, $verbose);
	FilesIO->removeFile($datasetDir.$TRAIN_BG_FILE, $verbose);
	FilesIO->removeFile($datasetDir."testT", $verbose);
	return $count_cover;
}

############################ sub run ##################################
sub consultExample2 {
	my $predicateFile	= $_[1];
	my $nameSequence	= $_[2];
	my $datasetDir      = $_[3];
	my $alephRunFile 	= $_[4];
	my $rule    	 	= $_[5];
	my $verbose			= $_[6];
	
	my @array = FilesIO->lineFileToArray($predicateFile);
	my $pred;my $info = "";
	my $i = 0; my $found =  "false"; my $j=0;
	my @array_aux = split(/\:\-/, $rule);
	my $fact_search = $array_aux[1];
	my $count = 0;
	@array_aux = split(/\)\,/, $fact_search); 
	
	for ($j=0; $j<scalar(@array_aux); $j++){
		$i = 0;
		$fact_search = $array_aux[$j];
		if ($j != scalar(@array_aux)-1){
			$fact_search .= ").";			
		}	
		$fact_search =~ s/A\,/$nameSequence\,/;	
		$fact_search = lc(String->removeCaracter($fact_search, " "));
		#print "fact search ".$fact_search."\n";
		while ($i < scalar(@array)){
			#print lc(String->removeCaracter($array[$i],"\n").".")."\n";
			if ($fact_search eq lc(String->removeCaracter($array[$i],"\n").".")){
				$count++;
				$i = scalar(@array);
			}
			$i++;		
		}
	}
	if ($count == $j){
		return 1;
	}else{
		return 0;
	}
}
	

1