package KNN;

if (exists($ENV{'PERL_LIB'})){
	use lib $ENV{'PERL_LIB'};
} 
else{
	print "ERROR:: Not existis PERL_LIB\n";
	exit;
}

use File::Files;
use Util::Hash;

my $PROG_NAME = "PerlLib::KNN::knn.pm";


############################ sub loadAlign1d ####################################
sub run{
	
	my $trainFile    	= $_[1];
	my $testFile    	= $_[2];
	my $outputFile    	= $_[3];
	my $k		    	= $_[4];
	my $ignoreAttTrain 	= $_[5];#sample identifier or other attributes (more than 1 use ':' for separeting them)
	my $ignoreAttTest 	= $_[6];
	my $weightAtt    	= $_[7];
	my $class	    	= $_[8];
	my $verbose	    	= $_[9];
	
	my @matrixDist = ();			my $line_i; 			my $line_j;
	my $removeCols = "";			my @trainData = ();		my @testData = ();
	my $predictedClass;				my $info = "";
			
	if ($ignoreAttTrain ne ""){$removeCols = $ignoreAttTrain.":";}
	if ($ignoreAttTest ne ""){$removeCols = $ignoreAttTest.":";}
	if ($weightAtt ne ""){
		$removeCols .= $weightAtt.":";		
	}else{
		$weightAtt =-1;
	}
	
	my @train = FilesIO->fileToArray2d($trainFile, "\t");
	@trainData = Array->removeColsMatriz($removeCols.$class, \@train);	
	
	my @test = FilesIO->fileToArray2d($testFile, "\t");
	@testData = Array->removeColsMatriz($removeCols.$class, \@test);	
	
	for (my $i = 0; $i <scalar(@testData); $i++){
		$line_i = Array->copyRowsMatrixToStringSep(",", $i, \@testData);
		for (my $j = 0; $j <scalar(@trainData); $j++){
			$line_j = Array->copyRowsMatrixToStringSep(",", $j, \@trainData);
			$matrixDist[$j][0] = Number->euclianDistance($line_i, $line_j);
			$matrixDist[$j][1] = $train[$j][$class];
			if ($weightAtt>=0){
				$matrixDist[$j][2] = $train[$j][$weightAtt];
			}else{
				$matrixDist[$j][2] = 1;
			}			
		}
	
		$predictedClass = KNN->classify($k, \@matrixDist);
		$line_i = Array->copyRowsMatrixToStringSep("\t", $i, \@testData)."\t".$predictedClass."\n";
		$info .= $line_i;
	}
	
	if ($outputFile ne ""){
		FilesIO->saveFile($outputFile, $info, $verbose);
	}	
	return $info;
}

############################ sub loadAlign1d ####################################
sub classify {
	my ($lixo, $k, $matrixDist) = @_;
	my @lmatrixDist = @{$matrixDist};
	@lmatrixDist = Array->sortArray2d(0, \@lmatrixDist);
	#print Array->printMatrix2("\t", "\n", \@lmatrixDist);
	my %hashClassify = ();
	
	for (my $j = 0; $j < $k; $j++){
		if (!exists($hashClassify{$lmatrixDist[$j][1]})){
			$hashClassify{$lmatrixDist[$j][1]} = $lmatrixDist[$j][2];
		}else{
			$hashClassify{$lmatrixDist[$j][1]} = $hashClassify{$lmatrixDist[$j][1]} + $lmatrixDist[$j][2];
		}
	}
	my @matrixClass = Hash->hashToArray(\%hashClassify);
	@matrixClass = Array->sortArray2dDesc(1, \@matrixClass);
	return $matrixClass[0][0];	
}

############################ sub loadAlign1d ####################################
sub leaveOneOut{
	my $trainFile    	= $_[1];
	my $k		    	= $_[2];
	my $ignoreAttTrain	= $_[3];#sample identifier or other attributes (more than 1 use ':' for separeting them)
	my $ignoreAttTest	= $_[4];
	my $weightAtt    	= $_[5];
	my $classAtt    	= $_[6];
	my $tmpDir			= $_[7];
	my $verbose	    	= $_[8];
	
	my @trainData = ();		my @testData = ();
	my $line_test = "";		my $line_train = "";
	my $result;				my @array_aux;
	my $real_class;			my $predicted_class;
	my $countTP = 0;		my $total;
	my %hash_true_class = ();my %hash_class = ();
	
	my @train = FilesIO->fileToArray2d($trainFile, "\t");
	if ($ignoreAttTrain eq ""){
		@trainData = @train;	
	}else{
		@trainData = Array->removeColsMatriz($ignoreAttTrain, \@train);	
	}

	$total = scalar(@trainData);
	
	my @test = FilesIO->fileToArray2d($trainFile, "\t");
	if ($ignoreAttTest eq ""){
		@testData = @test;	
	}else{
		@testData = Array->removeColsMatriz($ignoreAttTest, \@test);	
	}
	
	for (my $i = 0; $i <scalar(@testData); $i++){
		$line_test = Array->copyRowsMatrixToStringSep("\t", $i, \@testData)."\n";
		$real_class = $test[$i][$classAtt];
		for (my $j = 0; $j <scalar(@trainData); $j++){
			if ($j != $i){
				$line_train .= Array->copyRowsMatrixToStringSep("\t", $j, \@trainData)."\n";
			}
		}
		FilesIO->saveFile("$tmpDir/test$i.txt", $line_test, $verbose);
		FilesIO->saveFile("$tmpDir/train$i.txt", $line_train, $verbose);
		$result = KNN->run("$tmpDir/train$i.txt", "$tmpDir/test$i.txt", "", $k, $ignoreAttTrain, $ignoreAttTest, $weightAtt, $classAtt, $verbose)."\n";
		#print $line_test."\t".$result;
		@array_aux = String->StringToArray($result, "\t");
		$predicted_class = $array_aux[scalar(@array_aux) -1];
		print "Real=$real_class\tPred=$predicted_class\t";
		if (!exists($hash_true_class{$real_class})){
			$hash_true_class{$real_class} = 0;
		}
		if (!exists($hash_class{$real_class})){
			$hash_class{$real_class} = 1;
		}else{
			$hash_class{$real_class} = $hash_class{$real_class} + 1;
		}
		if ($predicted_class eq $real_class){
			$countTP++;		
		
			print "True";
			if ($real_class == 1){
				print $line_test."\n";
			}	
			$hash_true_class{$real_class} = $hash_true_class{$real_class} + 1;
		}
		print "\n";
		#print "Test\t".$line_test."\n";
		
		$line_train = "";
	}
	print Hash->printHash(\%hash_class);
	print Hash->printHash(\%hash_true_class);
	$acc = $countTP/$total;
	return $acc;
}
