package PHYLIP;

if (exists($ENV{'PERL_LIB'})){
	use lib $ENV{'PERL_LIB'};
} 
else{
	print "ERROR:: Not existis PERL_LIB\n";
	exit;
}

use File::Files;
use Align::Primary::clustalw;
use Util::Number;

my $TREE_EXT = "_phyml_tree.txt";
my $BINARY_TREE_PROGRAM = "retree";


########################### sub buildPSSM ##############################
sub getTREE_EXT {

	return $TREE_EXT;

}

########################### sub buildPSSM ##############################
sub run {
	my $programName		= $_[1];
    my $alignment		= $_[2];
    my $parameter		= $_[3];
    my $verbose			= $_[4];
    
    my $cmd = $programName." ".$alignment." ". $parameter;
    
    if ($verbose eq "true"){
    	print $cmd."\n"
    }
    system $cmd;
    
    if (FilesIO->isFile($alignment.$TREE_EXT) eq "false"){
		print "ERROR::perlLib::PhyTree::phylip:: $alignment$TREE_EXT NOT created!\n";
		exit;
	}elsif ($verbose eq "true") {
		print "perlLib::PhyTree::phylip:: $alignment$TREE_EXT created!\n";
	}

}

########################### sub buildPSSM ##############################
sub convertBinaryTree {
	my $input			= $_[1];
    my $output			= $_[2];
    my $verbose			= $_[3];
    
    my $cmd = $BINARY_TREE_PROGRAM." ".$input." ".$output;
    
    if ($verbose eq "true"){
    	print $cmd."\n"
    }
    system $cmd;
    
    if (FilesIO->isFile($output) eq "false"){
		print "ERROR::perlLib::PhyTree::phylip:: $output NOT created!\n";
		exit;
	}elsif ($verbose eq "true") {
		print "perlLib::PhyTree::phylip:: $output created!\n";
	}

}

############################ sub formatAlign1d ####################################
sub convertAlnIntoPhylipFormat {
 	my $aln 	 = $_[1];
 	my $output 	 = $_[2];
 	my $verbose	 = $_[3];


	my $numSeqs = 0; my $lenAln;
	my @array = FilesIO->fileToArray2d($aln, " ");  
	my $info = "";
	for (my $i=0; $i<scalar(@array); $i++){
		$lenAln = Sequences->lengthSequence($array[$i][1]);
		$info .= $array[$i][0]." ". $array[$i][1]."\n";
		$numSeqs++;
	}	
	
  	$info = $numSeqs." ".$lenAln."\n".$info;
	FilesIO->saveFile($output, $info, $verbose);	
 }
 
 
############################ sub formatAlign1d ####################################
sub renameSequenceFromAln {
 	my $aln 	 = $_[1];
	
	my @array = FilesIO->fileToArray2d($aln, " ");  
	my $info = "";
	for (my $i=0; $i<scalar(@array); $i++){
		
	}
}

############################ sub formatAlign1d ####################################
sub runProtDist {
 	my $aln 	 = $_[1];
	my $output   = $_[2];
	my $tmpDir   = $_[3];
	my $format   = $_[4];
	my $verbose  = $_[5];
	
	my $outputLocalFile = $tmpDir."outfile";
	
	FilesIO->saveFile($tmpDir."par.txt", "Y\n", $verbose);
	if (FilesIO->isFile($outputLocalFile) eq "true"){
		FilesIO->removeFile($outputLocalFile, $verbose);
	}	
	FilesIO->copyFile($aln, $tmpDir."infile", $verbose);
	chdir $tmpDir;
	
	my $cmd = "protdist infile < ".$tmpDir."par.txt";
	if ($verbose eq "true"){print $cmd;}
	system $cmd;
	
	FilesIO->checkFile($outputLocalFile, "ERROR::perlLib::PhyTree::phylip::runProtDist:: File not created $outputLocalFile\n");
	
	if ($format eq "true"){
		my $info = ""; 					my $line = "";
		my $head = "matrix";
		
		my @array_aux = FilesIO->fileToArray2d($outputLocalFile, " +");
		for (my $i = 1; $i < scalar(@array_aux); $i++){
			for (my $j = 0; $j <= $#{$array_aux[$i]}; $j++){
				#print "----before---".$array_aux[$i][$j]."----\n";
				if (Number->isNumber2($array_aux[$i][$j]) eq "false"){
					#print "----after---".$array_aux[$i][$j]."----\n";
					if ($line ne ""){
						$info .= substr($line, 0, length($line) -1)."\n"; 
						$line = "";
					}				
					$head .= "\t".$array_aux[$i][$j];
				}
				if ($array_aux[$i][$j] ne ""){
					$line .= $array_aux[$i][$j]. "\t";					
				}						
			}			
		}
		$info .=  substr($line, 0, length($line) -1)."\n";
		$info  = $head."\n".$info;
		#print $info;
		FilesIO->saveFile($output, $info, $verbose);
	}else{
		FilesIO->copyFile($outputLocalFile, $output, $verbose);
	}
	
}


############################ sub formatAlign1d ####################################
sub runNeighbor {
 	my $fileDist = $_[1];
	my $output   = $_[2];
	my $tmpDir   = $_[3];
	my $verbose  = $_[4];

	
	my $outputLocalFile = $tmpDir."outtree";
	
	FilesIO->saveFile($tmpDir."par.txt", "Y\n", $verbose);
	if (FilesIO->isFile($outputLocalFile) eq "true"){
		FilesIO->removeFile($outputLocalFile, $verbose);
		FilesIO->removeFile($tmpDir."infile", $verbose);
	}	
	FilesIO->copyFile($fileDist, $tmpDir."infile", $verbose);
	chdir $tmpDir;
	
	my $cmd = "neighbor infile < ".$tmpDir."par.txt";
	if ($verbose eq "true"){print $cmd;}
	system $cmd;
	
	FilesIO->checkFile($outputLocalFile, "ERROR::perlLib::PhyTree::phylip::runNeighbor:: File not created $outputLocalFile\n");
	FilesIO->copyFile($outputLocalFile, $output, $verbose);
	
}

############################ sub formatAlign1d ####################################
sub formatMatDist {
 	my $fileDist = $_[1];
	my $output   = $_[2];
	my $tmpDir   = $_[3];
	my $verbose  = $_[4];

	my $info = ""; 					my $line = "";
	my $head = "matrix";
		
	my @array_aux = FilesIO->fileToArray2d($fileDist, " +");
	for (my $i = 1; $i < scalar(@array_aux); $i++){
		for (my $j = 0; $j <= $#{$array_aux[$i]}; $j++){
			#print "----before---".$array_aux[$i][$j]."----\n";
			if (Number->isNumber2($array_aux[$i][$j]) eq "false"){
				#print "----after---".$array_aux[$i][$j]."----\n";
				if ($line ne ""){
					$info .= substr($line, 0, length($line) -1)."\n"; 
					$line = "";
				}				
				$head .= "\t".$array_aux[$i][$j];
			}
			if ($array_aux[$i][$j] ne ""){
				$line .= $array_aux[$i][$j]. "\t";					
			}						
		}			
	}
	$info .=  substr($line, 0, length($line) -1)."\n";
	$info  = $head."\n".$info;
		#print $info;
	FilesIO->saveFile($output, $info, $verbose);
}

1