package LIBSVM;

if (exists($ENV{'PERL_LIB'})){
	use lib $ENV{'PERL_LIB'};
} 
else{
	print "ERROR:: Not existis PERL_LIB\n";
	exit;
}

use File::Files;
use Util::Array;
use Util::Number;


my $PROG_NAME = "perlLib::SVM::libsvm.pm::";
my $TRAIN_PROGRAM = "svm-train";
my $PREDICT_PROGRAM = "svm-predict";
my $SCALE_PROGRAM = "svm-scale";



############################ sub lineFileToArray ###################################
sub createDataset {
	my $datasetFile	= $_[1];
	
	my $info = "";			my $last;
	my $label;				my $value;
		
	my @array = FilesIO->fileToArray2d($datasetFile, "\t");	

	
  for (my $i=0; $i < scalar(@array); $i++){
                $last = $#{$array[$i]};
                $label = $array[$i][$last];
                $info .= $label."  ";
                for (my $j=0; $j <= $last-1; $j++){
                        if ($array[$i][$j] ne "NA"){
                                $value = $array[$i][$j];
                                $info .= ($j+1).":".$value;
                                if ($j != $last-1){
                                        $info .= " ";
                                }       
                        }       
                }
                $info .= "\n";
        }
	
	return $info;	
}

############################ sub lineFileToArray ###################################
sub train {
	my $trainFile	= $_[1];
	my $svm_type	= $_[2];
	my $kernel_type = $_[3];
	my $trainmodel	= $_[4];
	my $parameters	= $_[5];
	my $verbose		= $_[6];
	
	my $cmd = $TRAIN_PROGRAM. $parameters." -s ".$svm_type." -t ".$kernel_type." ".$trainFile." ".$trainmodel;
	
	if ($verbose eq "true"){
		print $cmd."\n";
	}
	system($cmd);
	
	FilesIO->checkFile($trainmodel, "ERROR::".$PROG_NAME.":: File NOT created ".$trainmodel);	
	
}

############################ sub lineFileToArray ###################################
sub scale {
	my $inputFile		= $_[1];
	my $outputFile		= $_[2];
	my $upper			= $_[3];
	my $low				= $_[4];
	my $verbose			= $_[5];
	
	my $cmd = $SCALE_PROGRAM. $parameters. " -u $upper -l $low ". $inputFile. " > ".$outputFile;
	
	if ($verbose eq "true"){
		print $cmd."\n";
	}
	system($cmd);
	
	FilesIO->checkFile($outputFile, "ERROR::".$PROG_NAME.":: File NOT created ".$outputFile);	
	
}


############################ sub lineFileToArray ###################################
sub normalizeMinMax {
	my $inputFile		= $_[1];
	my $outputFile		= $_[2];
	my $classColumn		= $_[3];
	my $posClass		= $_[4];
	my $negClass		= $_[5];
	my $verbose			= $_[6];
	print $inputFile."\n";
	
	my @array = FilesIO->fileToArray2d($inputFile, "\t");	
	@array = Array->normalizeByColuns($negClass, $posClass, "", $classColumn, \@array);
	
	
	my $output = Array->printMatrix2("\t", "\n", \@array);
	FilesIO->saveFile($outputFile, $output, $verbose);
}



############################ sub lineFileToArray ###################################
sub predict {
	my $testFile	= $_[1];
	my $trainmodel	= $_[2];
	my $classOutput	= $_[3];
	my $confValueOutput = $_[4];
	my $parameters	= $_[5];
	my $verbose		= $_[6];
	
	my $cmd = $PREDICT_PROGRAM. $parameters." ".$testFile." ".$trainmodel." ".$classOutput." > ".$confValueOutput;
	
	if ($verbose eq "true"){
		print $cmd."\n";
	}
	system($cmd);
	
	FilesIO->checkFile($classOutput, "ERROR::".$PROG_NAME.":: File NOT created ".$classOutput);	
	
}
############################ sub lineFileToArray ###################################
sub getResults {
	my ($lixo, $rConfFile, $rClassFile, $outputFile, $posClass, $array) = @_;
	my @larray = @{$array};	
	
	my @array_cv = FilesIO->fileToArray2d($rConfFile, "=");
	my @array_class = FilesIO->fileToArray2d($rClassFile, " +");
	my @array_aux;			my $conf_value;
	my @array_results = ();
	my $small;
	my $bigger;
	my $absValue;
	
	for (my $i=0; $i < scalar(@array_cv) -1; $i++){
		$conf_value = $array_cv[$i][1];
		$id = $larray[$i];
		$class = $array_class[$i][0];
		
		if (Number->isNumber3($class) eq "true"){
			$absValue = abs($conf_value);
			if ($class == $posClass){
				$conf_value = $absValue;
			}else{
				$conf_value = (-1)*$absValue;
			}
			#print $id."\t".$conf_value."\t".$class."\n";
			$array_results[$i][0] = $id;
			$array_results[$i][1] = $conf_value;	
		}						
	}
	my $info = Array->printMatrix("\t", scalar(@array_results), 2,  \@array_results);
	FilesIO->saveFile($outputFile, $info, "false");
}


############################ sub lineFileToArray ###################################
sub getResultsProbability {
	my ($lixo, $resultFile, $outputFile, $colProbPos, $array) = @_;
	my @larray = @{$array};	
	
	my @array_cv = FilesIO->fileToArray2d($resultFile, " +");
	my @array_aux;			my $conf_value;
	my @array_results = ();
	my $small;
	my $bigger;
	
	if ($array_cv[0][1] == 1){
		$colProbPos = 1;
	}else{
		$colProbPos = 2;
	}
	
	for (my $i=1; $i < scalar(@array_cv); $i++){
		$conf_value = $array_cv[$i][$colProbPos];
		$id = $larray[$i-1];
		#print $id."\t".$conf_value."\n";
		$array_results[$i-1][0] = $id;
		$array_results[$i-1][1] = $conf_value;					
	}
	my $info = Array->printMatrix("\t", scalar(@array_results), 2,  \@array_results);
	FilesIO->saveFile($outputFile, $info, "false");
}

############################ sub lineFileToArray ###################################
sub balanceData {
	my ($lixo, $posClass, $negClass, $fileInput, $fileOutput) = @_;
	print "posClass = $posClass negClass = $negClass\n";
	my @larray = FilesIO->fileToArray2d($fileInput, "\t"); ;
	my $total;					my $classMinority;
	my $classMajority;
	
	my $last = $#{$larray[0]};
	my $totalPos = Array->countCaracterInColumnMatrix(scalar(@larray), $last, $posClass, \@larray);
	my $totalNeg = Array->countCaracterInColumnMatrix(scalar(@larray), $last, $negClass, \@larray);
	
	if ($totalNeg < $totalPos){
		$total = $totalNeg;
		$classMinority = $negClass;
		$classMajority = $posClass;
	}else{
		$total = $totalPos;
		$classMinority = $posClass;
		$classMajority = $negClass;
	}
	
	my @array_Min = Array->reduceMatrix($last, $classMinority, \@larray);
	my @array_Max = Array->reduceMatrix($last, $classMajority, \@larray);
	
	@array_Max = Array->getRandon($total,\@array_Max);
	
	print "TPos $totalPos TNeg $totalNeg $total $classMinority\n";
	my $info = Array->printMatrix2("\t", "\n", \@array_Min);
	$info .= Array->printMatrix2("\t", "\n", \@array_Max);
	FilesIO->saveFile($fileOutput, $info, "false");	
}

############################ sub lineFileToArray ###################################
sub balanceDataWithReplace {
	my ($lixo, $posClass, $negClass, $fileInput, $fileOutput) = @_;
	
	print "posClass = $posClass negClass = $negClass\n";
	my @larray = FilesIO->fileToArray2d($fileInput, "\t");
	my $total;					my $classMinority;
	my $classMajority;			my $replace;
	my @array_aux = ();			my $newReplace = 1000;
	
	my $last = $#{$larray[0]};
	my $totalPos = Array->countCaracterInColumnMatrix(scalar(@larray), $last, $posClass, \@larray);
	my $totalNeg = Array->countCaracterInColumnMatrix(scalar(@larray), $last, $negClass, \@larray);
	
	if ($totalNeg > $totalPos){
		$total = $totalNeg;
		$replace = $totalNeg - $totalPos;
		$classMinority = $posClass;
		$classMajority = $negClass;
	}else{
		$total = $totalPos;
		$replace = $totalPos - $totalNeg;
		$classMinority = $negClass;
		$classMajority = $posClass;
	}
	print "TPos = $totalPos TNeg = $totalNeg Total = $total Minority Class = $classMinority Total Replace = $replace \n";
	my @array_Min = Array->reduceMatrix($last, $classMinority, \@larray);
	my @array_Max = Array->reduceMatrix($last, $classMajority, \@larray);
	
	print "Array_min = ".scalar(@array_Min)." \n";
	print "Array_max = ".scalar(@array_Max)." \n";
	
	if ($replace != 0){
		if (scalar(@array_Min) < $total && scalar(@array_Min) >= $replace ){
			@array_aux = Array->getRandon($replace,\@array_Min);
		}else{
			@array_aux = Array->getRandonWithReplacement($replace,\@array_Min);	
		}	
	}	
	
	my $info = Array->printMatrix2("\t", "\n", \@array_Min);
	$info .= Array->printMatrix2("\t", "\n", \@array_aux);
	$info .= Array->printMatrix2("\t", "\n", \@array_Max);
	FilesIO->saveFile($fileOutput, $info, "false");	
}


############################ sub main ##################################
sub normalize{

		my $test_file_in 	= $_[1];
		my $test_file_out 	= $_[2];
		my $upper		 	= $_[3];
		my $bound		 	= $_[4];
				
		my @array_aux;				my $info_test = "";
		my $info_train = "";		
		my @array_inverse_cols = ();
		$array_inverse_cols[0] = 1;
		
		my @arrar_par_test = FilesIO->fileToArray2d($test_file_in, "\t");
	
		my @array = @arrar_par_test;
		my $cols = $#{$array[0]};		
		
		for (my $i=1; $i < $cols; $i++){
			if (Array->isInArray($i, \@array_inverse_cols) eq "true"){
				@array 	= Array->logMatrizByCol(10, $i, \@array);
				@array =  Array->multiplyValueMatrizByCol((-1), 1, \@array);
			}				
		}
		
		@array = Array->normalizeByColunsMinMax($bound, $upper, 0, 200, "", "0:2:3:4:5:6", \@array);
		@array = Array->removeColsMatriz(0, \@array);
		$info_test = Array->printMatrix2("\t", "\n", \@array);
		FilesIO->saveFile($test_file_out, $info_test, $verbose);
		
}


############################ sub main ##################################
sub normalizeEvalue{

		my $test_file_in 	= $_[1];
		my $test_file_out 	= $_[2];
		my $maxEvalue 		= $_[3];
		my $minEvalue 		= $_[4];
		my $colEv			= $_[5];
		my $upper		 	= $_[6];
		my $bound		 	= $_[7];
		
				
		my @array_aux;				my $info_test = "";
		my $info_train = "";		
		my $maxValue;				my $minValue;
		my @array_inverse_cols = ();
		$array_inverse_cols[0] = 1;
		
		my @arrar_par_test = FilesIO->fileToArray2d($test_file_in, "\t");
	
		my @array = @arrar_par_test;
		my $cols = $#{$array[0]};		
		
		for (my $i=0; $i < scalar(@array); $i++){
			$array[$i][$colEv] =  Number->logMax($array[$i][$colEv] , 10, $minEvalue);
			$array[$i][$colEv] = $array[$i][$colEv] * (-1);				
		}
		
		$maxValue = Number->log($minEvalue, 10, $minEvalue)*(-1);
		$minValue = Number->log($maxEvalue, 10, $minEvalue)*(-1);
		
		
		@array = Array->normalizeByColunsMinMaxCol($bound, $upper, $minValue, $maxValue, $colEv, \@array);
		@array = Array->removeColsMatriz(0, \@array);
		$info_test = Array->printMatrix2("\t", "\n", \@array);
		FilesIO->saveFile($test_file_out, $info_test, $verbose);
		
}
############################ sub main ##################################
sub setClass{
	my $train_file 	= $_[1];	
	my $out_file 	= $_[2];	
	my $class 		= $_[3];
	my $numberAtt	= $_[4];
	
	my $info = "";
	
	my @arrar_par_train = FilesIO->fileToArray2d($train_file, "\t", " ");
	my $cols = $#{$arrar_par_train[0]} + 1;
	if ($cols < $numberAtt){
		print "ERROR::".$PROG_NAME.":: File $train_file there are not enougth parameters\n";
		exit;
	}
	for (my $i=0; $i < scalar(@arrar_par_train); $i++){
		for (my $j=0; $j < $cols; $j++){
			$info .=$arrar_par_train[$i][$j]."\t";
		}
		$info .="$class\n";
	}	
	FilesIO->saveFile($out_file, $info, $verbose);	
}

1