package SVM;

if (exists($ENV{'PERL_LIB'})){
	use lib $ENV{'PERL_LIB'};
} 
else{
	print "ERROR:: Not existis PERL_LIB\n";
	exit;
}

use File::Files;

my $ReadRawGramMatrixExe = "ReadRawGramMatrix.exe";
my $processMatrixExe     = "process_matrix.exe";
my $svdcmdExe			 = "svdcmd.exe";
my $v2svmvecExe			 = "V2svmvec.exe";
my $test2svmvecExe		 = "test2svmvec.exe";
my $trainSvmExe			 = "gist-train-svm";
my $classifySvmExe		 = "gist-classify";
my $space = " ";



########################### sub createLabel ##############################
sub getResults {
	my $file_in						= $_[1];
	my $file_out					= $_[2];
	my $verbose						= $_[3];
	my $info = "";
	my @array_aux = ();
	
	my @array = FilesIO->lineFileToArrayWithToken($file_in, "classification	discriminant");
	my $number;
	for (my $i=1; $i < scalar(@array); $i++){
		@array_aux = split(/\t/, $array[$i]);
		if (scalar(@array_aux) != 3){
			print "ERROR::perlLib::SVM::svm.pm::getResults():: File $file_in out of format\n";
			exit;
		}
		$info .= $array_aux[0]."\t".$array_aux[2];
	}	
	 FilesIO->saveFile($file_out, $info, $verbose);	
}


########################### sub createLabel ##############################
sub createLabel {
	my $num_seq						= $_[1];
	my $start_seq					= $_[2];
	my $label						= $_[3];
	my $verbose						= $_[4];
	my $info = "";
	
	my $number;
	for (my $i=0; $i < $num_seq; $i++){
		$number = $i + $start_seq;
		$info .= $number."\t". $label."\n";
	}	
	return $info;	
}


########################### sub createLabel ##############################
sub createLabelFromFile {
	my $file						= $_[1];
	my $label						= $_[2];
	my $verbose						= $_[3];
	my $info = "";
	
	my @array = FilesIO->fileToArray2d($file, "\t");
	for (my $i=0; $i < scalar(@array); $i++){
		$info .= $array[$i][0]. "\t$label\n";
	}	
	return $info;	
}


########################### sub createLabel ##############################
sub createDataset {
	my $file						= $_[1];
	my $start_seq					= $_[2];
	my $verbose						= $_[3];
	my $info = "";
	
	my @array = FilesIO->fileToArray2d($file, "\t");
	my $cols = $#{$array[0]} + 1;
	
	$info = "seq";
	for (my $i=1; $i< $cols; $i++){
		$info .="\tfeature".($i);
	}
	$info .="\n";
	
	my $number;
	for (my $i=0; $i < scalar(@array); $i++){
		#$number = $i + $start_seq;
		$number =  $array[$i][0];
		$info .= $number;
		for (my $j=1; $j < $cols; $j++){
			$info .="\t". $array[$i][$j];
		}
		$info .= "\n";
	}	
	return $info;	
}

########################### sub ReadRawGramMatrix ##############################
# Count the N-gram vectors of all the protein sequences in a input file and 
# output for subsequent processing.
################################################################################
sub readRawGramMatrix {
	my $N_gram						= $_[1];
	my $sequenceFile				= $_[2];
	my $RawGramMatrixOutPutFile		= $_[3];
	my $verbose						= $_[4];
	
	if (!defined($verbose)){
		$verbose = "true";
	}
	FilesIO->createDirFromFile($RawGramMatrixOutPutFile);
	
	if (FilesIO->isFile($sequenceFile) eq "false"){ 
		print "ERROR::perlLib::SVM::svm::ReadRawGramMatrix:: File $sequenceFile NOT found\n";
		exit;
	}
	
	my $cmd = $ReadRawGramMatrixExe.$space.$N_gram.$space.$sequenceFile.$space.$RawGramMatrixOutPutFile;
	if ($verbose eq "true") {
		print "LOG:: ReadRawGramMatrix to  $sequenceFile \n";
		print $cmd."\n";
	}	
	system $cmd;
	
	
	if (FilesIO->isFile($RawGramMatrixOutPutFile) eq "false"){ 
		print "ERROR::perlLib::SVM::svm::ReadRawGramMatrix::  File $RawGramMatrixOutPutFile NOT created\n";
		exit;
	}	
}

########################### sub processMatrix ##############################
# Pre-process the word-document matrix. Each element in the matrix is 
#replaced by the weight of each word in documents.
################################################################################

sub processMatrix {
	my $rawGramMatrixFile			= $_[1];
	my $proOuPutFile				= $_[2];
	my $wordentropyOutPutFile		= $_[3];
	my $verbose						= $_[4];
	
	if (!defined($verbose)){
		$verbose = "true";
	}
	if (FilesIO->isFile($rawGramMatrixFile) eq "false"){ 
		print "ERROR::perlLib::SVM::svm::processMatrix:: File $rawGramMatrixFile NOT found\n";
		exit;
	}
	
	FilesIO->createDirFromFile($rawGramMatrixFile);	
	
	my $cmd = $processMatrixExe.$space.$rawGramMatrixFile.$space.$proOuPutFile.$space.$wordentropyOutPutFile;
	if ($verbose eq "true") {
		print "LOG:: processMatrix to  $rawGramMatrixFile \n";
		print $cmd."\n";
	}	
	system $cmd;	
	
	if (FilesIO->isFile($proOuPutFile) eq "false"){ 
		print "ERROR::perlLib::SVM::svm::processMatrix:: File $proOuPutFile NOT created\n";
		exit;
	}	
	
	if (FilesIO->isFile($wordentropyOutPutFile) eq "false"){ 
		print "ERROR::perlLib::SVM::svm::processMatrix:: File $wordentropyOutPutFile NOT created\n";
		exit;
	}	
}

########################### sub svdcmd ##############################
# Doing the singular value decomposition (SVD)
################################################################################

sub svdcmd {
	my $proProcessMatrix			= $_[1];
	my $uOuPutFile					= $_[2];
	my $sOuPutFile					= $_[3];
	my $vOuPutFile					= $_[4];
	my $verbose						= $_[5];
	
	if (!defined($verbose)){
		$verbose = "true";
	}
	if (FilesIO->isFile($proProcessMatrix) eq "false"){ 
		print "ERROR::perlLib::SVM::svm::svdcmd:: File $rawGramMatrixFile NOT found\n";
		exit;
	}
	
	FilesIO->createDirFromFile($uOuPutFile);	
	
	my $cmd = $svdcmdExe.$space.$proProcessMatrix.$space.$uOuPutFile.$space.$sOuPutFile.$space.$vOuPutFile;
	if ($verbose eq "true") {
		print "LOG:: svdcmd to  $proProcessMatrix \n";
		print $cmd."\n";
	}	
	system $cmd;	
	
	if (FilesIO->isFile($uOuPutFile) eq "false"){ 
		print "ERROR::perlLib::SVM::svm::svdcmd:: File $uOuPutFile NOT created\n";
		exit;
	}	
	
	if (FilesIO->isFile($sOuPutFile) eq "false"){ 
		print "ERROR::perlLib::SVM::svm::svdcmd:: File $sOuPutFile NOT created\n";
		exit;
	}	
	
	if (FilesIO->isFile($vOuPutFile) eq "false"){ 
		print "ERROR::perlLib::SVM::svm::svdcmd:: File $vOuPutFile NOT created\n";
		exit;
	}	
}
########################### sub v2svmvec ##############################
# extracts the low dimension vectors from the results of SVD, which 
# can be used to train a SVM classifier
################################################################################

sub v2svmvec {
	my $Vsvdcmd				= $_[1];
	my $Ssvdcmd				= $_[2];
	my $R					= $_[3];
	my $svmVecOuPutFile		= $_[4];
	my $verbose				= $_[5];
	
	if (!defined($verbose)){
		$verbose = "true";
	}
	if (FilesIO->isFile($Vsvdcmd) eq "false" || FilesIO->isFile($Ssvdcmd) eq "false" ){ 
		print "ERROR::perlLib::SVM::svm::v2svmvec:: File $Vsvdcmd OR $Ssvdcmd NOT found\n";
		exit;
	}
	
	FilesIO->createDirFromFile($svmVecOuPutFile);	
	
	my $cmd = $v2svmvecExe.$space.$Vsvdcmd.$space.$Ssvdcmd.$space.$R.$space.$svmVecOuPutFile;
	if ($verbose eq "true") {
		print "LOG:: v2svmvec to  $svmVecOuPutFile \n";
		print $cmd."\n";
	}	
	system $cmd;	
	
	if (FilesIO->isFile($svmVecOuPutFile) eq "false"){ 
		print "ERROR::perlLib::SVM::svm::v2svmvec:: File $svmVecOuPutFile NOT created\n";
		exit;
	}	
	
}

########################### sub test2svmvec ##############################
# Convert the testing vectors to representation vectors which can be 
# evaluated by SVM classifier.
################################################################################

sub test2svmvec {
	my $R							= $_[1];
	my $Usvdcmd						= $_[2];
	my $rawGramMatrixFile			= $_[3];
	my $svmVecOuPutFile				= $_[4];
	my $processMatrixWordentropy	= $_[5];
	my $verbose						= $_[6];
	
	if (!defined($verbose)){
		$verbose = "true";
	}
	if (FilesIO->isFile($Usvdcmd) eq "false" || FilesIO->isFile($rawGramMatrixFile) eq "false" || FilesIO->isFile($processMatrixWordentropy) eq "false"){ 
		print "ERROR::perlLib::SVM::svm::test2svmvec:: File $Usvdcmd OR $rawGramMatrixFile OR $processMatrixWordentropy NOT found\n";
		exit;
	}
	
	FilesIO->createDirFromFile($svmVecOuPutFile);	
	
	my $cmd = $test2svmvecExe.$space.$R.$space.$Usvdcmd.$space.$rawGramMatrixFile.$space.$svmVecOuPutFile.$space.$processMatrixWordentropy;
	if ($verbose eq "true") {
		print "LOG:: test2svmvec to  $svmVecOuPutFile \n";
		print $cmd."\n";
	}	
	system $cmd;	
	
	if (FilesIO->isFile($svmVecOuPutFile) eq "false"){ 
		print "ERROR::perlLib::SVM::svm::test2svmvec:: File $svmVecOuPutFile NOT created\n";
		exit;
	}	
	
}

########################### sub test2svmvec ##############################
# Convert the testing vectors to representation vectors which can be 
# evaluated by SVM classifier.
################################################################################
sub trainSvm{
	my $trainFile					= $_[1];
	my $labelFile					= $_[2];
	my $learnWeigthOutPutFile		= $_[3];
	my $verbose						= $_[4];
	
	if (!defined($verbose)){
		$verbose = "true";
	}
	if (FilesIO->isFile($trainFile) eq "false" || FilesIO->isFile($labelFile) eq "false"){ 
		print "ERROR::perlLib::SVM::svm::trainSvm:: File $trainFile OR $labelFile NOT found\n";
		exit;
	}
	
	FilesIO->createDirFromFile($learnWeigthOutPutFile);	
	
	my $cmd = $trainSvmExe.$space." -radial -maxtime 600 -threshold 0.001 -train".$space.$trainFile.$space."-class".$space.$labelFile.$space.">".$space.$learnWeigthOutPutFile;
	if ($verbose eq "true") {
		print "LOG:: trainSvm to  $learnWeigthOutPutFile \n";
		print $cmd."\n";
	}	
	system $cmd;	
	
	if (FilesIO->isFile($learnWeigthOutPutFile) eq "false"){ 
		print "ERROR::perlLib::SVM::svm::trainSvm:: File $learnWeigthOutPutFile NOT created\n";
		exit;
	}	
	
}

########################### sub test2svmvec ##############################
# SVM classifier.
################################################################################
sub classifySvm{
	my $trainFile					= $_[1];
	my $svmTrainedFile				= $_[2];
	my $testFile					= $_[3];
	my $outputFile					= $_[4];
	my $verbose						= $_[5];
	
	if (!defined($verbose)){
		$verbose = "true";
	}
	if (FilesIO->isFile($trainFile) eq "false" || FilesIO->isFile($svmTrainedFile) eq "false" || FilesIO->isFile($testFile) eq "false"){ 
		print "ERROR::perlLib::SVM::svm::classifySvm:: File $trainFile OR $svmTrainedFile OR $testFile NOT found\n";
		exit;
	}
	
	FilesIO->createDirFromFile($outputFile);	
	
	my $cmd = $classifySvmExe.$space."-train".$space.$trainFile.$space."-learned".$space.$svmTrainedFile;
	$cmd .= $space."-test".$space.$testFile.$space.">".$space.$outputFile;
	if ($verbose eq "true") {
		print "LOG:: classifySvm to  $outputFile \n";
		print $cmd."\n";
	}	
	system $cmd;	
	
	if (FilesIO->isFile($outputFile) eq "false"){ 
		print "ERROR::perlLib::SVM::svm::classifySvm:: File $outputFile NOT created\n";
		exit;
	}	
	
}

########################### sub test2svmvec ##############################
# SVM score.
################################################################################
sub scoreSvm{
	my $testLabelFile				= $_[1];
	my $classifySvmFile				= $_[2];
	my $svmTrainedFile				= $_[3];
	my $outputFile					= $_[4];
	my $pathScoreFunction			= $_[5];
	my $verbose						= $_[6];
	
	if (!defined($verbose)){
		$verbose = "true";
	}
	if (FilesIO->isFile($testLabelFile) eq "false" || FilesIO->isFile($classifySvmFile) eq "false" || FilesIO->isFile($svmTrainedFile) eq "false"){ 
		print "ERROR::perlLib::SVM::svm::scoreSvm:: File $testLabelFile OR $classifySvmFile OR $svmTrainedFile NOT found\n";
		exit;
	}
	
	FilesIO->createDirFromFile($outputFile);	
	
	my $cmd = "perl".$space.$pathScoreFunction.$space."-test".$space.$testLabelFile.$space.$classifySvmFile.$space.$svmTrainedFile;
	$cmd .= $space.">".$space.$outputFile;
	if ($verbose eq "true") {
		print "LOG:: scoreSvm to  $outputFile \n";
		print $cmd."\n";
	}	
	system $cmd;	
	
	if (FilesIO->isFile($outputFile) eq "false"){ 
		print "ERROR::perlLib::SVM::svm::scoreSvm:: File $outputFile NOT created\n";
		exit;
	}	
	
}


########################### sub main ####################################
sub getSequencesFromResults { 
	
	my $file  		= $_[1];
	my $posTestList = $_[2];
	my $negTestList = $_[3]; 
	my $type        = $_[4]; #pos = 1 ; neg = -1
	
	my @array_seqs_find = ();
	
	my $indice; my $name_seq; my $scop_id; my $family; my $seqFile; my $j = 0;

	my @array_results = FilesIO->fileToArray2d($file, "\t");
	my @array_positives =  FilesIO->lineFileToArray($posTestList);
	my @array_negatives =  FilesIO->lineFileToArray($negTestList);
	
	
	for (my $i=12; $i<scalar(@array_results); $i++){			
		if (String->removeCaracter($array_results[$i][1]," ") eq $type){
			$indice = $i -12;
			#print "i = ".$i. "indice = ".$indice."\t". "pos " .scalar(@array_positives)."\n";
			if ($indice < scalar(@array_positives)){
				$seqFile = String->removeCaracter($array_positives[$indice], "\n");	
			}
			else{
				$indice = $indice - scalar(@array_positives);
				#print "i = ".$i. "indice = ".$indice."\t". "pos " .scalar(@array_positives)."\n";
				if ($indice < scalar(@array_negatives)){
					$seqFile = String->removeCaracter($array_negatives[$indice], "\n");	
					#print "neg ";
				} 
			} 
			#print $seqFile."\n";
			$name_seq = FilesIO->getNameFile($seqFile);
			$name_seq = substr($name_seq, 0, index($name_seq, ".".FilesIO->getExt($seqFile)));
			$scop_id = substr($name_seq,0, rindex($name_seq,"_"));
			$family  = substr($name_seq,rindex($name_seq,"_")+1, length($name_seq));
			
			$array_seqs_find[$j][0] = $scop_id;
			$array_seqs_find[$j][1] = $family;
			$array_seqs_find[$j][2] = $seqFile;	
			$j++;	
		}
	}
	return @array_seqs_find;
}

########################### sub main ####################################
sub getScoreDistribution { 
	my $attFile  		= $_[1];
	my $colEvalue		= $_[2];
	my $numAtt			= $_[3];
	
	my @arrayWeight = (4, 1, 1, 1);
	my @dist = ();
	
	my @atts = FilesIO->fileToArray2d($attFile, "\t");
	@atts = Array->logMatrizByCol(10, $colEvalue, \@atts);
	@atts = Array->multiplyValueMatrizByCol(-1, 1, \@atts);
	
	@att_array = Array->copyColMatrixToArray2(1, \@atts);
	$max =  Array->getBigger(\@att_array);
	$min =  Array->getSmaller(\@att_array);
	print "max=$max min=$min\n";
	@atts = Array->normalizeByColunsMinMaxCol(0, 1, 0, 300, $colEvalue, \@atts);
	
	for (my $i=0; $i<scalar(@atts); $i++){
		@array_aux = Array->copyRowMatrixToArrayInterval($i, "1:$numAtt", \@atts);
		#print Array->printArray("\t", \@array_aux)."\n";
		$avgWeight = Array->getAverageWeigth(\@arrayWeight, \@array_aux);
		#print $avgWeight."\n";
		push(@dist, $avgWeight);
	}
	#print Array->printArray("\n", \@dist);
	return (Array->getAverage(\@dist) - Array->getSTD(\@dist));		
}


1