#!/usr/bin/perl -w
use strict;

if (exists($ENV{'PERL_LIB'})){
	use lib $ENV{'PERL_LIB'};
} 
else{
	print "ERROR:: Not existis PERL_LIB\n";
	exit;
}

use File::Files;
use String::StringUtil;
use Sequence::sequence;

my $inputFile = "";
my $outputFile = "";
my $verbose = "false";
my $numberRandomSeqs = 20; 
my $lengthRandomSeqs = 100;
my $factorRandomSeqs = 1;
my @array_seqs = ();

my $PROG_NAME = "perlLib::run::run_sequence.pl::";

&getParameters;
&checkParameters;
&main;

############################ sub getParameters ##################################
sub getParameters {
	my $counter;
	my $ending_value = scalar(@ARGV);		#numero de parametros de entrada
	for($counter = 0 ; $counter < $ending_value ; $counter++){
		if($ARGV[$counter] eq "-inputFile"){
			$inputFile = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-outputFile"){
			$outputFile = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-verbose"){
			$verbose = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-numberRandomSeqs"){
			$numberRandomSeqs = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-lengthRandomSeqs"){
			$lengthRandomSeqs = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-factorRandomSeqs"){
			$factorRandomSeqs = $ARGV[$counter+1];		
		}		
	}
}

############################ sub checkParameters ##################################
sub checkParameters {
	print "\n=============================================\n";
	print "Cheking Parameters for $PROG_NAME\n";
	
	
	String->isEmpty($inputFile, "ERROR::".$PROG_NAME.":: Parameter -inputFile having the input file in fasta format is required!");	
	FilesIO->checkFile($inputFile, "ERROR::".$PROG_NAME.":: File NOT found ".$inputFile);
	String->isEqual($verbose,"true,false", "ERROR:: ".$PROG_NAME.":: Parameter -verbose must be true or false!");
	String->isEmpty($outputFile, "ERROR::".$PROG_NAME.":: Parameter -outputFile is required it must contain the output file! ");	
	String->isEmpty($outputFile, "ERROR::".$PROG_NAME.":: Parameter -option (what do you wish to do) is required! ");	
	

	print "-inputFile = $inputFile\n";
	print "-outputFile = $outputFile\n";
	print "-numberRandomSeqs = $numberRandomSeqs\n";
	print "-lengthRandomSeqs = $lengthRandomSeqs\n";
	print "-factorRandomSeqs = $factorRandomSeqs\n";
	print "-verbose = $verbose\n";
		
	print "ok\n";
}

################################### sub main #####################################
sub main {
	@array_seqs = Sequences->getSequenceFromFastaArray2D($inputFile);
	
	my $shuffled;			my $info = "";
	
	print "LOG::".$PROG_NAME." Generating ramdom sequences\n";
	
	for (my $i = 0; $i < scalar(@array_seqs); $i++){
		#$info .= $array_seqs[$i][0].Sequences->divideSeq($array_seqs[$i][1], 60)."\n";
		print "LOG::".$PROG_NAME." Generating ramdom sequence $i\n";
		for (my $j = 0; $j < $numberRandomSeqs; $j++){
			if ($factorRandomSeqs > 0){
				$lengthRandomSeqs = $factorRandomSeqs*length($array_seqs[$i][1]);
			}
			$shuffled = Sequences->generateSequenceShuffled($array_seqs[$i][1], $lengthRandomSeqs, $verbose);
			$info .= $array_seqs[$i][0].Sequences->divideSeq($shuffled, 60)."\n";		
		}
	}
	FilesIO->saveFile($outputFile, $info, $verbose);
		
}

