#!/usr/bin/perl -w
use strict;
if (exists($ENV{'PERL_LIB'})){
	use lib $ENV{'PERL_LIB'};
} 
else{
	print "ERROR:: Not existis PERL_LIB\n";
	exit;
}

use File::Files;
use Database::ScopPattern;
use Sequence::Domain;
use Util::Array;
use Align::Primary::blast;
use Database::PFAM;

my $DOMAIN_COV_THRESHOLD	= "40";
my $evalue = "1";
my $inputFile  = "";
my $outputDir	= "";
my $dbIdsFile = "";
my $verbose			= "false";
my $svmProbFile = "";
my $CUT_OFF_PROB = 0.05;
my $PFAM_ACC = "";
my $CUT_OFF_ONE_DOMAIN = 1e-07;
my $pfamFile = "";
my $outputFile = "";

my $C_DOMAIN_HMMSCAN = 4;
my $C_START_HMMSCAN = 1;
my $C_END_HMMSCAN = 2;

my $C_DOMAIN_CODD = 1;
my $C_START_CODD = 17;
my $C_END_CODD = 18;
my $C_EV_CODD = 11;
my $C_ACC_QUERY = 3;

my $PROG_NAME = "PERL_LIB::Tools::CODD::getPotentialDomains.pl";	

&getParameters;
&checkParameters;
&main;

############################ sub getParameters ##################################
sub getParameters {
	my $counter;
	my $ending_value = scalar(@ARGV);		#numero de parametros de entrada
	for($counter = 0 ; $counter < $ending_value ; $counter++){
		if($ARGV[$counter] eq "-inputFile" || $ARGV[$counter] eq "-i"){
			$inputFile = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-pfamFile" || $ARGV[$counter] eq "-d"){
			$pfamFile= $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-verbose"){
			$verbose = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-outputFile"){
			$outputFile = $ARGV[$counter+1];		
		}
	}
}
############################ sub checkParameters ##################################
sub checkParameters {
	print "\n=============================================\n";
	print "Cheking Parameters for $PROG_NAME\n";
	
	String->isEmpty($inputFile, "ERROR::".$PROG_NAME.":: Parameter -inputFile having result CASH file is required!");	
	FilesIO->checkFile($inputFile, "ERROR::".$PROG_NAME.":: File NOT found ".$inputFile);
	String->isEmpty($pfamFile, "ERROR::".$PROG_NAME.":: Parameter -pfamFile having result CASH file is required!");	
	FilesIO->checkFile($pfamFile, "ERROR::".$PROG_NAME.":: File NOT found ".$pfamFile);
	String->isEqual($verbose,"true,false", "ERROR:: ".$PROG_NAME.":: Parameter -verbose must be true or false!");

	String->isEmpty($outputFile, "ERROR::".$PROG_NAME.":: Parameter -outputFile having output File is required!");	
	
	print "-inputFile = $inputFile\n";
	print "-pfamFile = $pfamFile\n";
	print "-outputFile = $outputFile\n";
	print "-verbose = $verbose\n";	
	
	print "ok\n";
}	

################################### sub main #####################################
sub main {
	my @arrayHmmscan	= ();			my @arrayIds = ();
	my @arrayCodd 		= ();			my @arrayPotential = ();
	my $count = 0;						my $overlap = 0;				
	my $info_codd = "";
	my $save;							my $info = "";
	my $countTotal;						my $countIds = 0;
	my @arrayCodd_aux;		
	
	my $id_prot; 						my $id_prot_old = "";
	
	
	if (open(FILEIN, "$inputFile")){
		while (my $line = <FILEIN>) {
			if (index($line, "#") < 0){
				@arrayCodd =split(/ +/, $line);
				$id_prot = $arrayCodd[$C_ACC_QUERY];
				$countIds++;
				if ($id_prot_old ne "" && $id_prot ne $id_prot_old){
					#print "PFAM\n".$info_pfam."\n"; print "CODD\n".$info_codd."\n";
					$info .= getPotDomains($id_prot_old, $info_codd);
					#print "CODD Pot\n".$info."\n";
				
					$info_codd = "";
					if ($countIds == 100){
						FilesIO->apendFile($outputFile, $info, $verbose);
						$countTotal += $countIds;
						print "LOG::Saved ". $countTotal. "\n";
						$info = ""; $countIds= 0;
					}	
				}
				$info_codd .= $line;
				$id_prot_old = $id_prot;
			}
	
		}
	}			
	$info .= getPotDomains($id_prot_old, $info_codd);
	FilesIO->apendFile($outputFile, $info, $verbose);
	
}

################################### sub main #####################################
sub getPotDomains {
		my $id_prot		= $_[0];
		my $info_codd	= $_[1];
		
		my @arrayCodd;			my @arrayHmmscan;
		my @arrayCodd_aux;		my @arrayPotential = ();
		my $save;				my $overlap;
		my $count = 0;			my $info = "";
		
		my $info_pfam = FilesIO->getLineInFile($pfamFile, $id_prot, $verbose);
		
		if ($info_pfam ne ""){
			@arrayHmmscan = String->StringToArray2d($info_pfam, "\n", "\t");
		}
		if ($info_codd ne ""){
			@arrayCodd =split(/\n/, $info_codd);
			for (my $i=0; $i < scalar(@arrayCodd); $i++){
				@arrayCodd_aux = split(/ +/, $arrayCodd[$i]);
				$save = "true";
				for (my $j=0; $j < scalar(@arrayHmmscan); $j++){
					if (PFAM->getAccNumber($arrayCodd_aux[$C_DOMAIN_CODD]) eq $arrayHmmscan[$j][$C_DOMAIN_HMMSCAN]){
						$overlap = Domain->calculateOverlapSize($arrayHmmscan[$j][$C_START_HMMSCAN], $arrayHmmscan[$j][$C_END_HMMSCAN], $arrayCodd_aux[$C_START_CODD], $arrayCodd_aux[$C_END_CODD]);
						if ($overlap > 5){
							$save = "false";
							$j = scalar(@arrayHmmscan);							
						}
					}					
				}
				if ($save eq "true"){
					$arrayPotential[$count][0] = $arrayCodd_aux[$C_EV_CODD];
					$arrayPotential[$count][1] = $arrayCodd[$i];
					$count++;
				}							
			}
			if (scalar(@arrayPotential) > 0){
				@arrayPotential = Array->sortArray2d(0, \@arrayPotential);
				$info = Array->printColMatrix(1, "\n", \@arrayPotential)."\n";
			}
		}
		return $info;
}
