#!/usr/bin/perl -w
use strict;

use lib "/usr/local/bioperl14/";
use Bio::SearchIO;

#Variaveis que comportam os parametros de entrada
my $in_file 		= "";
my $db_file;

my @hmmer3d_ids 		= ();
my @true_ids 		= ();

my @hmmer3d_ids_tp 	= ();#hmmer3d ids true positive
my @hmmer3d_ids_fp 	= ();#hmmer3d ids false positive
my @hmmer3d_ids_fn	= ();#hmmer3d ids false negatives

my @hmmer3d_info_parsed = ();#result file hmmer3d
my $info_statistics = "";

my $alin_er   = "clustalw";
my $calibrate = "nocalibrated";
my $work_fold = "hmmer3d";

#my $alin_er   = "";
#my $calibrate = "";


&getParameters;
&main;

############################ sub getParameters ##################################
sub getParameters {
	my $counter;
	my $ending_value = scalar(@ARGV);		#numero de parametros de entrada
	for($counter = 0 ; $counter < $ending_value ; $counter++){
		if($ARGV[$counter] eq "-filein"){
			$in_file = $ARGV[$counter+1];		
		} 		
	}
}

#################################### sub main ##################################
sub main {

	my @families 		= ();
	my @family 			= ();
	my $entry;
	my $old_sample		= "";
	my $new_sample		= "";
	my $dir;
		
	#read family file
	unless (open(FAMILYFILE, "$in_file")) {
       	print "Could not open file $in_file !\n";
        	exit;
    }
	@families = <FAMILYFILE>;
	
	foreach $entry (@families) {
		@family	= split(/ +/,$entry);
		$new_sample = $family[0];
		if ($new_sample ne $old_sample){
			$dir = substr($family[4],0,index($family[4],"\n")). "/" ;	
			&getTruePositivesFound($new_sample,$dir); 
			&writeFileRitsFound($new_sample,$dir);
			&writeFileRitsNotFound($new_sample,$dir);	

			@hmmer3d_ids 		= ();
			@true_ids 		= ();
			@hmmer3d_ids_tp 	= ();
			@hmmer3d_ids_fp 	= ();	
			@hmmer3d_info_parsed = ();
			@hmmer3d_ids_fn	= ();
		}
		$old_sample = $new_sample;					
	}
	print $info_statistics;
	&writeStatistics($info_statistics);
	close FAMILYFILE;
}

############################ sub getTruePositivesFound ##################################
sub getTruePositivesFound{
	my $sample		= $_[0];
	my $dir			= $_[1];
		

	my $dir_result 		= $dir.$sample."/result/$alin_er/$calibrate/";
	my $dir_result_hmmer3d = $dir_result."$work_fold/";
	my $result_file_name = "hmmer3d_$sample.result";
	my $real_true_positive_file = &getPattern(3, ".", $sample)."true";

	my	$item_true_id;
	my $item_hmmer3d_id;
	my $count_true_positive_found		= 0;
	my $count_false_positive	= 0;
	my $found = "false";


	&readhmmer3dIds($dir_result_hmmer3d.$result_file_name);
	&readTrueIds($dir.$real_true_positive_file);

	foreach $item_hmmer3d_id (@hmmer3d_ids){
		foreach $item_true_id (@true_ids){	
			if ($item_true_id eq $item_hmmer3d_id){
				push(@hmmer3d_ids_tp, $item_true_id);
				$count_true_positive_found++;
				$found = "true";				
			}
		}
		if ($found eq "false"){
			push(@hmmer3d_ids_fp, $item_hmmer3d_id);
			$count_false_positive++;
		}
		$found = "false";
	}
	#$count_false_positive = scalar(@hmmer3d_ids) - $count_true_positive_found;
	$info_statistics .= "$sample FP $count_false_positive TP $count_true_positive_found";	
	
}

##################################### sub writeFileRitsFound ##############################
sub writeFileRitsFound{
	
	my $sample		= $_[0];
	my $dir			= $_[1];

	my $hmmer3d	= "hmmer3d";

	my $path_tp_file = $dir.$sample."/result/$alin_er/$calibrate/$work_fold/true_positives_found.txt";
	my $path_fp_file = $dir.$sample."/result/$alin_er/$calibrate/$work_fold/false_positives_found.txt";
	
	my $item;
	my $i = 0;

	my @result = ();
	my $found  = "false";
	my $info_true = "";
	my $info_false = "";

   foreach $item(@hmmer3d_info_parsed){
		@result	= split(/ +/,$item);		
		#print $item."\n";
		while ($i < scalar(@hmmer3d_ids_tp) && $found eq "false"){
			#print $hmmer3d_ids_tp[$i].$result[0]."\n";
			if ($hmmer3d_ids_tp[$i] eq $result[0]){
				$info_true.= $item;
				$found = "true";	
			}
			$i++;	
		}
		$i = 0; 
		if ($found eq "false"){
			$info_false .= $item;	
		}
		$found = "false";			
	}
	&saveFile($path_tp_file, $info_true);
	&saveFile($path_fp_file, $info_false);
}

##################################### sub writeFileRitsNotFound ##############################
sub writeFileRitsNotFound{
	
	my $sample		= $_[0];
	my $dir			= $_[1];


	my $path_fn_file = $dir.$sample."/result/$alin_er/$calibrate/$work_fold/false_negatives_found.txt";
		
	my $item_true_id;
	my $item_hmmer3d_id;
		
	my $found  = "false";
	my $info_false_neg = "";
	my $count_false_negative = 0;

  foreach $item_true_id (@true_ids){
		foreach $item_hmmer3d_id (@hmmer3d_ids_tp){	
			if ($item_true_id eq $item_hmmer3d_id){
				$found = "true";				
			}
		}
		if($found eq "false"){
			$info_false_neg.= $item_true_id ."\n";
			$count_false_negative++;
		}
		$found = "false";
	}
	$info_statistics .= " FN $count_false_negative\n";
	&saveFile($path_fn_file, $info_false_neg);
}
############################ sub writeStatistics ##################################
sub writeStatistics{

	my @infos = split(/\n/,$info_statistics);
	my @each_info  = ();
	my $old_pattern = "";
	my $new_pattern = "";
	my $new_sample = "";
	my $info_file ="";

	my $item;
	my $sum_tp = 0;
	my $sum_fp = 0;
	my $sum_fn = 0;
	my $num_same_samples = 0;

	foreach $item (@infos){
		@each_info 	= split(/ +/,$item); 
		$new_sample = $each_info[0];
		$new_pattern = &getPattern(3, ".", $new_sample);

		if ($old_pattern ne $new_pattern && $old_pattern ne ""){			
			$info_file.= &calcTx($num_same_samples,$sum_tp,$sum_fp,$sum_fn,$old_pattern);
			$num_same_samples = 0;
			$sum_tp = 0;
			$sum_fp = 0;
			$sum_fn = 0;
		}
		$sum_fp += $each_info[2];
		$sum_tp += $each_info[4];
		$sum_fn += $each_info[6];
		$num_same_samples++;
		$old_pattern = $new_pattern;		
	}
	$info_file.= &calcTx($num_same_samples,$sum_tp,$sum_fp,$sum_fn,$old_pattern);
	print $info_file;
	&saveFile("hmmer3d_statistics.txt",$info_file);
}

############################ sub calcTx ##################################
sub calcTx{
	my $num_same_samples	= $_[0];
	my $sum_tp	= $_[1];
	my $sum_fp	= $_[2];
	my $sum_fn	= $_[3];
	my $old_pattern = $_[4]; 

	my $num_tp;
	my $num_fp;
	my $num_fn;

	my $precision;
	my $recall;
	my $tx;

	if ($num_same_samples == 0){
				$num_same_samples = 1;
	}
	$num_tp = $sum_tp / $num_same_samples;
	$num_fp = $sum_fp / $num_same_samples;
	$num_fn = $sum_fn / $num_same_samples;
	$precision = $num_tp /($num_tp + $num_fn);
	$recall =  $num_tp /($num_tp + $num_fp);
	if ($num_tp == 0){
		$precision = 0;
		$recall	= 0;
	}else{
		$precision = $num_tp /($num_tp + $num_fn);
		$recall =  $num_tp /($num_tp + $num_fp);
	}
	return "$old_pattern $num_tp $num_fp $num_fn $precision $recall\n";
	
}
##################################### sub readhmmer3dIds ##################################
sub readhmmer3dIds{
	my $path			= $_[0];

	my $hmmer3d_file_parsed = $path.".parsed";

	my $item;
	my	@hmmer3d_file = ();
	my	@hmmer3d_info = ();
	my $i;

	if (open(hmmer3dRESFILE,$path)){
		&parserhmmer3dResults($path);
		close hmmer3dRESFILE;
		#print "path $hmmer3d_file_parsed\n";
		if (open(hmmer3dPARFILE,$hmmer3d_file_parsed)){
			@hmmer3d_file = <hmmer3dPARFILE>;
			$i = 0;
			foreach $item (@hmmer3d_file){
				$hmmer3d_info_parsed[$i++] = $item;
				@hmmer3d_info	= split(/ +/,$item);
				push(@hmmer3d_ids, $hmmer3d_info[0]);						
			}
			close hmmer3dPARFILE;
		}
		else{
			print "ERROR : file $hmmer3d_file_parsed not found\n";
		}
		
	}
	else{
		print "ERROR : file $path not found\n";
	}
}
##################################### sub readTrueIds ##################################
sub readTrueIds{
		my $path	= $_[0];
	
		my @lines_file = ();
		my $line;	
		my @align		= ();	
		my @true_positives = ();
	
		my %hash_seq	= ();
		my $value = "";
		if (open(FILESEQ,"$path")){
			@lines_file = <FILESEQ>;		
			foreach $line (@lines_file){
				@true_positives = split (/ +/, $line);
				push(@true_ids, $true_positives[0]);			
			}
			close FILESEQ;
		}
		else{
			print "File $path not found!\n";
		}
}
#################################### sub parserhmmer3dResults ##################################
sub parserhmmer3dResults{
		my $path			= $_[0];

		my $outfileName = $path.".parsed";

		my $maxEvalue = "10";
		my $minScore = "-100";
		print "path $outfileName\n";
		unless (open(PARSEDFILE, ">$outfileName")) {
                print "Could not open file $outfileName !\n";
                exit;
       }

		my $in = new Bio::SearchIO(-format => 'hmmer', -file =>$path);

		my $result = $in->next_result;
		while (my $hit = $result->next_hit) {
			while (my $hsp = $hit->next_hsp ) {
	      	if ($hit->raw_score >= $minScore){
	  				if ($hit->significance <= $maxEvalue){
          			print PARSEDFILE $hit->name, " ",
			 			 $hsp->length('hit')," ",
						 $hit->raw_score," ",
						 $hit->significance," ",
						 $hit->description, "\n";
       			}
				}	
	 		}
		}
		close (PARSEDFILE);
}

############################ sub getPattern ##################################
sub getPattern {
	my $num 		= $_[0];
	my $sep		= $_[1];
	my $string 	= $_[2];
	my $counter;
	my $indice;
	my $new_string 	= "";

	for($counter = 0 ; $counter < $num  ; $counter++){
		$indice 	= index($string, $sep);
		if ($indice >=0){
			$new_string 	= $new_string . substr($string,0,$indice+1);
			$string		= substr($string,$indice+1, length($string));			
		}
	}
	return $new_string;	
}

############################ sub saveFile ###################################
sub saveFile{
	my $path 	= $_[0];
	my $info  	= $_[1];

	if (open(FILESEQ,">$path")){
		print FILESEQ $info;
		print "file $path was created\n";
	}
	else{
		print "file $path not found\n";
	}
	close FILESEQ;
}
