#!/usr/bin/perl -w
use strict;

if (exists($ENV{'PERL_LIB'})){
	use lib $ENV{'PERL_LIB'};
} 
else{
	print "ERROR:: Not existis PERL_LIB\n";
	exit;
}

use File::Files;
use PSSM::psi_blast_source_code;
use Database::ScopPattern;
use Align::Primary::blast;
use Sequence::sequence;


my $fastaFile	= "";
my $genesList	 = "";
my $verbose = "false";
my $format = "";
my $outputFile = "";
my $sep = "\\|";
my $field = 2;

my $PROG_NAME = "getSequencesFromFasta.pl";


&getParameters;
&checkParameters;
&main;

############################ sub getParameters ##################################
sub getParameters {
	my $counter;
	my $ending_value = scalar(@ARGV);		#numero de parametros de entrada
	for($counter = 0 ; $counter < $ending_value ; $counter++){
		if($ARGV[$counter] eq "-fastaFile"){
			$fastaFile = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-genesList"){
			$genesList = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-verbose"){
			$verbose = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-outputFile"){
			$outputFile= $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-sep"){
			$sep= $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-field"){
			$field= $ARGV[$counter+1];		
		}          	  	 			
	}
}

############################ sub checkParameters ##################################
sub checkParameters {
	print "\n=============================================\n";
	print "Cheking Parameters $PROG_NAME\n";
	
	String->isEmpty($fastaFile, "ERROR::".$PROG_NAME.":: Parameter -fastaFile having FASTA file pathway is required!");	
	FilesIO->checkFile($fastaFile, "ERROR::".$PROG_NAME.":: File NOT found ".$fastaFile);
	
	if ($fastaFile eq ""){
		print $PROG_NAME."::ERROR::Parameter -fastaFile having FASTA file pathway is required!\n";
	}
	if ($genesList eq ""){
		print $PROG_NAME."::ERROR::Parameter -genesList having genes list file is required!\n";
	}
	if ($outputFile eq ""){
		print $PROG_NAME."::ERROR::Parameter -outputFile having output file name is required!\n";
	}
	

	print "fastaFile=$fastaFile\n";
	print "genesList=$genesList\n";
	print "outputFile=$outputFile\n";
	print "ok\n";
}	


########################### sub main ####################################
sub main { 
	my $geneID;					my $id;
	my $info = "";				my $annotation;
	my $count_saved = 0;
	 
	my @sequences = Sequences->getSequenceFromFastaArray2D($fastaFile, "true");
	my %hashIds = FilesIO->fileToHash($genesList, "\t", 0);
	
	for (my $i=0; $i<scalar(@sequences); $i++){
		$annotation = $sequences[$i][0];
		
		$id = String->getElementArrayLine($annotation, $sep, $field);
		#print $annotation."\n".$id;
		
		if (exists($hashIds{$id})){
			$info .=$annotation. $sequences[$i][1];
			$count_saved++;
		}
	}
	FilesIO->saveFile($outputFile, $info, $verbose);
	print "Saved = ".$count_saved."\n";
}
