#!/usr/bin/perl -w
use strict;
if (exists($ENV{'PERL_LIB'})){
	use lib $ENV{'PERL_LIB'};
} 
else{
	print "ERROR:: Not existis PERL_LIB\n";
	exit;
}

use File::Files;
use Database::ScopPattern;


my $fileTestSet	= "";
my $virtualPath = "";
my $outputFileList  = "";
my $exceptSupFamily			= "";
my $kindRun 		= "";
my $defaultFamily	= "";
my $verbose			= "false";
my $sep = "\n"; 

&getParameters;
&checkParameters;
&main;

############################ sub getParameters ##################################
sub getParameters {
	my $counter;
	my $ending_value = scalar(@ARGV);		#numero de parametros de entrada
	for($counter = 0 ; $counter < $ending_value ; $counter++){
		if($ARGV[$counter] eq "-fileTestSet"){
			$fileTestSet = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-virtualPath"){
			$virtualPath= $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-outputFileList"){
			$outputFileList= $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-exceptSupFamily"){
			$exceptSupFamily= $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-kindRun"){
			$kindRun = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-defaultFamily"){
			$defaultFamily = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-verbose"){
			$verbose = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-sep"){
			$sep = $ARGV[$counter+1];		
		}
	}
}
############################ sub checkParameters ##################################
sub checkParameters {
	print "\n=============================================\n";
	print "Cheking Parameters\n";
	
	if ($fileTestSet eq ""){
		print "ERROR::prepareListDataSet:: Parameter -fileTestSet having SCOP parseable file that correspond to dataset is required!\n";
		exit;
	}
	if (FilesIO->isFile($fileTestSet) eq "false"){
		print "ERROR::prepareListDataSet:: $fileTestSet is NOT a File !\n";
		exit;
	}
	
	print "-fileTestSet = $fileTestSet\n";
	
	if ($virtualPath eq ""){
		print "ERROR::prepareListDataSet:: Parameter -virtualPath having virtual path of list files is required!\n";
		exit;
	}
	print "-virtualPath = $virtualPath\n";
	
	if ($outputFileList eq ""){
		print "ERROR::prepareListDataSet:: Parameter -outputFileList having output file name is required!\n";
		exit;
	}
	print "-outputFileList = $outputFileList\n";
	
	if ($exceptSupFamily ne ""){
		print "-exceptSupFamily = $exceptSupFamily\n";
	}
	
	if ($kindRun eq ""){
		print "WARNING::prepareListDataSet:: Parameter -kindRun saying how to run (1 = by Family (default); 2 = bySequence) is required!\n";
		$kindRun = 2;
	}
	if ($kindRun ne "1" && $kindRun ne "2"){
		print "ERROR::prepareListDataSet:: Parameter -kindRun must be 1 or 2.\n";
		exit;
	}	
	print "-kindRun = $kindRun\n";
	
	if ($verbose ne "true" && $verbose ne "false"){
		print "ERROR::prepareListDataSet:: Parameter -verbose must be true or false!\n";
		exit;
	}	
	
	print "-verbose = $verbose\n";	
	print "-sep = $sep\n";	
	
	print "ok\n";
}	

################################### sub main #####################################
sub main {
	my @families 		= ();
	my @family 		= ();
	my $entry		= "";
	my $id_family		= "";
	my $id_sup		= "";
	my $pdbId		= "";
	my $chain;
	my @array_line;
	my $scopId = "";
	my $value2;
	my $id_family_old = "";
	my $info	= "";	
	@families = FilesIO->lineFileToArray($fileTestSet);
	foreach $entry (@families) {
	@array_line = split(/\t/, $entry);
			
		if (index($array_line[0],"\n") != 0){
			$id_family = String->removeCaracter(ScopPattern->getPatternFamily($array_line[3]),"\n");
			$id_sup = ScopPattern->getPattern(3,".", $id_family);
			$pdbId = $array_line[1];
			$scopId = $array_line[0];	
			$chain 	= $array_line[2];
			
			if ($kindRun == 1 && $id_family ne $id_family_old){
				#by family
				if ($verbose eq "true"){
					print "LOG:: by family  $id_family \n";
				}
				$info .= &runFamily($id_family, $id_sup, $pdbId, $scopId, $chain, 1, $virtualPath);
			}elsif ($kindRun == 2) {
				#by sequence
				#print "parameter ".$id_family."\t". $id_sup."\t". $pdbId."\t". $scopId."\t". $chain."\n";
				if ($verbose eq "true"){
					print "LOG:: by sequence  $id_family  $scopId\n";
				}
				$info .= &runFamily($id_family, $id_sup, $pdbId, $scopId, $chain, 2, $virtualPath);
			}
			
			$id_family_old = $id_family;				
		}
	}
	
	if ($info ne ""){
		FilesIO->saveFile(ScopPattern->replaceDataFamily($id_family, $id_sup, $pdbId, $scopId, $chain, $outputFileList), $info, $verbose);	
	}else{
		print "ERROR::prepareListDataSet:: $fileTestSet has a format invalid\n";
		exit;
	}
}

############################ sub main ##################################
sub runFamily {
	my $id_family		= $_[0];
	my $id_sup			= $_[1];	
	my $pdbId			= $_[2];
	my $scopId			= $_[3];		
	my $chain			= $_[4];		
	my $kind			= $_[5];	
	my $virtualPath		= $_[6];	

	my $value2 = ScopPattern->replaceDataFamily($id_family, $id_sup, $pdbId, $scopId, $chain, $virtualPath, $defaultFamily);
	if ($exceptSupFamily ne ""){
		if ($id_sup ne $exceptSupFamily){
			return $value2.$sep;
		}
	}else{
		return $value2.$sep;
	}
}			
