#!/usr/bin/perl -w
use strict;

#Variaveis que comportam os parametros de entrada
my $in_file 		= "";
my $db_file;
my $calib			= "yes";

my $linker_program		= "linker_Hmmer_PDB";
my $build_program			= "hmmbuild";
my $calibrate_program 	= "hmmcalibrate";
my $search_program 		= "hmmsearch";
my $path_hmmer				= "/home/juliana/tese/desenvolvimento/hmmer/linux/Debug_Build/./";
#my $path_hmmer				= "";

my $kind_Align				= "mammoth";
my $kind_calib				= "calibrated";	

my $kind_pdb				= "mammoth";

&getParameters;
&main;

############################ sub getParameters ##################################
sub getParameters {
	my $counter;
	my $ending_value = scalar(@ARGV);		#numero de parametros de entrada
	for($counter = 0 ; $counter < $ending_value ; $counter++){
		if($ARGV[$counter] eq "-filein"){
			$in_file = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-db"){
			$db_file = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-calib"){
			$calib = $ARGV[$counter+1];		
		} 		 		
	}
}

############################ sub getParameters ##################################
sub main {
	my @families 		= ();
	my @family 			= ();
	my $entry;
	my $old_sample		= "";
	my $new_sample		= "";
	my $dir;
		
	#read family file
	unless (open(FAMILYFILE, "$in_file")) {
       		print "Could not open file $in_file !\n";
        	exit;
    	}
	@families = <FAMILYFILE>;
	print "\n =================\n Inicio ";
	system("date");
	print "\n =================\n";
	foreach $entry (@families) {
		@family	= split(/ +/,$entry);
		$new_sample = $family[0];
		#print $new_sample ."\n";
		if ($new_sample ne $old_sample){
			$dir = substr($family[4],0,index($family[4],"\n")). "/" ;	
			&runLinkerHmmer($new_sample,$dir);
                        
# 			&runHmmBuild($new_sample,$dir); 			
# 			if ($calib eq "yes"){ 
# 				&runCalibrate($new_sample,$dir); 
# 			}
# 			&runSearch($new_sample,$dir); 
		}
		$old_sample = $new_sample;					
	}
	print "\n =================\n fim ";
	system("date");
	print "\n =================\n";

	close FAMILYFILE;
}
############################ sub runLinkerHmmer ##################################
sub runLinkerHmmer{
	my $sample		= $_[0];
	my $dir			= $_[1];

	my $model_file_name   		= "hmmer3d_$sample.hmm";
	my $aln_file_name     		=  $sample."-FINAL.aln";
	#my $aln_file_name     		=  $sample.".clustalw";
	my $aln3d_new_file_name   	=  $sample.".mat";
	my $aln3d_file_name   		=  $sample."-FINAL.pdb";

	my $dir_aln	  					=  $dir. $sample. "/alignment/$kind_Align/";
	my $dir_aln_pdb				=  $dir. $sample. "/alignment/$kind_pdb/";
	my $parameters					=  "-align_$kind_pdb $dir_aln$aln_file_name -".$kind_pdb."_pdb $dir_aln_pdb$aln3d_file_name";

	#if (open(ALNFILE,$dir_aln.$aln3d_new_file_name)){
	#	print "file $dir_aln$aln3d_new_file_name already created.\n";
	#	close ALNFILE;
	#}
	#else{
	
		if (open(ALNFILE,$dir_aln_pdb.$aln3d_file_name)){
			my $cmd = $path_hmmer.$linker_program. " ".$parameters;
			print  "\n linker ". $cmd. "\n";
		        system $cmd;		
			close ALNFILE;
		}
		else{
			print "File $dir_aln_pdb$aln3d_file_name not found\n";
		}		
	#}	
}

############################### sub runHmmBuild ####################################
sub runHmmBuild{
	my $sample		= $_[0];
	my $dir			= $_[1];

	my $model_file_name 	= "hmmer3d_$sample.hmm";
	my $aln_file_name   	=  $sample.".clustalw";
   my $matriz_file_name =  $sample.".mat";
	my $dir_model = "";

	my $dir_aln	  			= $dir. $sample. "/alignment/$kind_Align/";

	$dir_model = $dir. $sample. "/models/$kind_Align/";
	

	
	my $parameters 		= "--pdb $dir_aln$matriz_file_name";

	#if (open(MODELFILE,$dir_hmmer_model.$model_file_name)){
	#	print "file $dir_model$model_file_name already created.\n";
	#	close MODELFILE;
	#}
	#else{
		if (&isDirectory($dir_model) eq "false"){
			mkdir $dir_model;		
		}
		$dir_model .= "$kind_calib/";
		if (&isDirectory($dir_model) eq "false"){
				mkdir $dir_model;
		}
		my $dir_hmmer_model 	= $dir_model."hmmer3d/";
		if (&isDirectory($dir_hmmer_model) eq "false"){
			mkdir $dir_hmmer_model;	
		}
		
		my $cmd = $path_hmmer.$build_program. " ".$parameters. " ".$dir_hmmer_model.$model_file_name. " ". $dir_aln.$aln_file_name;
		print $cmd. "\n";
		system $cmd;
	#}	
}
############################ sub runCalibrate ##################################
sub runCalibrate{
	my $sample		= $_[0];
	my $dir			= $_[1];

	my $model_file_name = "hmmer3d_$sample.hmm";
	my $dir_model = $dir. $sample. "/models/$kind_Align/$kind_calib/";
	my $dir_hmmer_model = $dir_model."hmmer3d/";

	if (open(MODELFILE,$dir_hmmer_model.$model_file_name)){
		my $cmd = $calibrate_program. " ". $dir_hmmer_model .$model_file_name;
		print $cmd. "\n";
		system $cmd;
		close MODELFILE;
	}
	else{
		print "ERROR: file $dir_model$model_file_name have not created.\n";
	}	
}
############################ sub runSearch ##################################
sub runSearch{
	my $sample		= $_[0];
	my $dir			= $_[1];

	my $model_file_name = "hmmer3d_$sample.hmm";
	my $result_file_name = "hmmer3d_$sample.result";
	
	my $dir_hmmer_model = "";
	my $dir_result = "";
	
	
	
	$dir_hmmer_model	= $dir. $sample. "/models/$kind_Align/$kind_calib/hmmer3d/";	
	$dir_result	= $dir. $sample. "/result/$kind_Align/";
	

	

	if (open(MODELFILE,$dir_hmmer_model.$model_file_name)){
		if (&isDirectory($dir_result) eq "false"){
			mkdir $dir_result;		
		}
		$dir_result .= "$kind_calib/";
		if (&isDirectory($dir_result) eq "false"){
			mkdir $dir_result;		
		}
		my $dir_hmmer_result	= $dir_result. "hmmer3d/";
		if (&isDirectory($dir_hmmer_result) eq "false"){
			mkdir $dir_hmmer_result;		
		}

		my $param = " -E 10 -T -100 ";	
		my $cmd = $search_program. " $param ".$dir_hmmer_model.$model_file_name. " ". $db_file. " > ".$dir_hmmer_result.$result_file_name ;
		print $cmd. "\n";
		system $cmd;
		close MODELFILE;
	}
	else{
		print "ERROR: file $dir_hmmer_model.$model_file_name have not created.\n";
	}	
}

############################ sub isDirectory ##################################
sub isDirectory{
	my $dir		= $_[0];
	opendir MYDIR, $dir;
	my @contents = readdir MYDIR;
	closedir MYDIR;
	if (scalar(@contents) == 0){
		return "false";	
	}
	return "true";
}
