#!/usr/bin/perl -w
use strict;

use lib "/usr/local/bioperl14";
use Bio::AlignIO;

#Variaveis que comportam os parametros de entrada
my $in_file 		= "";
my $db_file;
my $calib		= "yes";	

my $build_model_program	= "modelfromalign";
my $search_program 	= "hmmscore";

my $kind_Align		= "mammoth";
my $kind_calib		= "calibrated";
my $work_fold		= "hmmer_sam";

&getParameters;
&main;

############################ sub getParameters ##################################
sub getParameters {
	my $counter;
	my $ending_value = scalar(@ARGV);		#numero de parametros de entrada
	for($counter = 0 ; $counter < $ending_value ; $counter++){
		if($ARGV[$counter] eq "-filein"){
			$in_file = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-db"){
			$db_file = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-calib"){
			$calib = $ARGV[$counter+1];		
		} 	 		
	}
}
############################ sub main ##################################
sub main {
	my @families 		= ();
	my @family 			= ();
	my $entry;
	my $old_sample		= "";
	my $new_sample		= "";
	my $dir;
		
	#read family file
	unless (open(FAMILYFILE, "$in_file")) {
       		print "Could not open file $in_file !\n";
        	exit;
    }
	@families = <FAMILYFILE>;
	
	foreach $entry (@families) {
		@family	= split(/ +/,$entry);
		$new_sample = $family[0];
		#print $new_sample ."\n";
		if ($new_sample ne $old_sample){
			$dir = substr($family[4],0,index($family[4],"\n")). "/" ;
			system("date");
			#&runFormatFiletoSam($new_sample,$dir); 
			#&runBuildModel($new_sample,$dir); 
			&runSearch($new_sample,$dir); 
			system("date");
		}
		$old_sample = $new_sample;					
	}
	close FAMILYFILE;
}

############################ sub runBuildModel ##################################
sub runBuildModel{
	my $sample		= $_[0];
	my $dir			= $_[1];

	my $model_file_name = "sam_$sample";
	my $aln_file_name   =  $sample."-FINAL.aln";

	my $dir_aln	  = $dir. $sample. "/alignment/$kind_Align/";
	my $dir_model = $dir. $sample. "/models/$kind_Align/";
	

	#if (open(MODELFILE,$dir_sam_model.$model_file_name.".mod")){
	#	print "file 1 $dir_sam_model$model_file_name already created.\n";
	#	close MODELFILE;
	#}
	#else{
		if (&isDirectory($dir_model) eq "false"){
			mkdir $dir_model;		
		}
		$dir_model .= "$kind_calib/";
		if (&isDirectory($dir_model) eq "false"){
				mkdir $dir_model;
		}
		my $dir_sam_model = $dir_model."sam/";
		if (&isDirectory($dir_sam_model) eq "false"){
				mkdir $dir_sam_model;
		}
		my $cmd = $build_model_program. " ". $dir_sam_model.$model_file_name. " -alignfile ". $dir_aln.$aln_file_name. " -alphabet protein";
		print $cmd. "\n";
		system $cmd;
	#}	
}

############################ sub runSearch ##################################
sub runSearch{
	my $sample		= $_[0];
	my $dir			= $_[1];

	my $model_file_name 	= "sam_$sample.mod";
	my $aln_file_name   	=  $sample.".clustalw.out";
	my $result_file_name	=  "sam_".$sample;

	my $dir_aln	 	= $dir. $sample. "/alignment/$kind_Align/";
	my $dir_sam_model 	= $dir. $sample. "/models/$kind_Align/$kind_calib/$work_fold/";
	my $dir_result 		= $dir. $sample."/result/$kind_Align/";
	

	if (open(MODELFILE,$dir_sam_model.$model_file_name)){
	#	if (open(RESFILE,$dir_sam_result.$result_file_name.".dist")){
	#		print "file   $dir_sam_result$result_file_name already created.\n";
	#		close RESFILE;
	#	}
	#	else{			
			if (&isDirectory($dir_result) eq "false"){
				mkdir $dir_result;		
			}
			$dir_result .= "$kind_calib/";		
			if (&isDirectory($dir_result) eq "false"){
				mkdir $dir_result;		
			}
			my $dir_sam_result 	= $dir_result."$work_fold/";
			if (&isDirectory($dir_sam_result) eq "false"){
				mkdir $dir_sam_result;		
			}
			my $cmd = $search_program. " ".$dir_sam_result.$result_file_name. " -db ".$db_file." -i ";
			if ($calib eq "no"){
				$cmd .= $dir_sam_model.$model_file_name. " -select_score 4 -Emax 10";
			}
			else{
				$cmd .= $dir_sam_model.$model_file_name. " -dpstyle 1 -calibrate 1 -select_score 4 -Emax 10";
			}
			print $cmd. "\n";

			system $cmd;
			
		#}
		close MODELFILE;
	}
	else{
				print "file $dir_sam_model$model_file_name not found.\n";		
	}	
}
############################ sub runFormatFiletoSam ##################################
sub runFormatFiletoSam{
	my $sample		= $_[0];
	my $dir			= $_[1];

	my $aln_file_name   =  $sample.".clustalw";
	my $dir_aln	  = $dir. $sample. "/alignment/$kind_Align/";

	my $inputfilename = $dir_aln.$aln_file_name;

	if (open(ALNFILE,$inputfilename)){	
		close ALNFILE;
		#my $dotposition = index($inputfilename,".");
		#my $outputfilename = substr($inputfilename,0,$dotposition).".out";
		my $outputfilename = $inputfilename.".out";

   	my $in  = Bio::AlignIO->new(-file => $inputfilename,'-format' => 'clustalw');     
   	my $out = Bio::AlignIO->new(-file => ">$outputfilename",'-format' => 'fasta');

   	while ( my $aln = $in->next_aln() ) {     
      	#    $aln->map_chars('\-','');     
      	    $out->write_aln($aln);     
		}
  	}
	else{
		print "The file $inputfilename not found";
	}	
}

############################ sub isDirectory ##################################
sub isDirectory{
	my $dir		= $_[0];
	opendir MYDIR, $dir;
	my @contents = readdir MYDIR;
	closedir MYDIR;
	if (scalar(@contents) == 0){
		return "false";	
	}
	return "true";
}
