/*
 *   C++ sockets on Unix and Windows
 *   Copyright (C) 2002
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "PracticalSocket.h"  // For Socket and SocketException
#include <iostream>           // For cerr and cout
#include <cstdlib>            // For atoi()
#include <fstream>
#include <sstream>

using namespace std;

int main(int argc, char *argv[]) {
  if (argc != 2) {     // Test for correct number of arguments
    cerr << "Usage: " << argv[0]
         << " <string to find>" << endl;
    exit(1);
  }
  stringstream serverLocation;
  serverLocation << SERVER_LOCATION[0] << SERVER_LOCATION[1];

  char *echoString = argv[1];   // Second arg: string to echo

  char tempString[RCVBUFSIZE];
  string servAddress; // First arg: server address
  int echoStringLen = strlen(echoString), echoServPort;   // Determine input length

  std::ifstream file(serverLocation.str().data(), std::ios::in);
  file.getline(tempString,RCVBUFSIZE);
  servAddress = tempString;
  file.getline(tempString,RCVBUFSIZE);
  echoServPort = atoi(tempString);
  file.close();

  try {
    // Establish connection with the echo server
    TCPSocket sock(servAddress, echoServPort);
  
    // Send the string to the echo server
    sock.send(echoString, echoStringLen);
    int bytesReceived;
  
    char echoBuffer[RCVBUFSIZE + 1];    // Buffer for echo string + \0
      if ((bytesReceived = (sock.recv(echoBuffer, RCVBUFSIZE))) <= 0) {
        cerr << "Unable to read";
        exit(1);
      }
      echoBuffer[bytesReceived] = '\0';        // Terminate the string!
      cout << echoBuffer;                      // Print the echo buffer
    cout << endl;

    // Destructor closes the socket

  } catch(SocketException &e) {
    cerr << e.what() << endl;
    exit(1);
  }

  return 0;
}
