/*
 *   C++ sockets on Unix and Windows
 *   Copyright (C) 2002
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "PracticalSocket.h"  // For Socket, ServerSocket, and SocketException
#include <iostream>           // For cout, cerr
#include <cstdlib>            // For atoi()  
#include <pthread.h>          // For POSIX threads  
#include <fstream>
#include <vector>
#include <algorithm>
#include <sstream>

struct ThreadParameters
{
	TCPSocket *clntSock;
	std::vector < std::vector < std::string > > * sortedSearchSpace;
	std::vector < std::vector < int > > * sortedSearchSpaceAmount;
	std::vector < std::vector < std::string > > * searchSpace;
};

namespace Support
{
	/*comment*///ordem das rotas
	static const bool ASCENDING = true;
	static const bool DESCENDING = false;

	static const char FILE_NAME_SEPARATOR[2] = ".";

   static const unsigned int MAX_READS = 1024*256;
   static const unsigned int INVALID_IDENTIFICATION = -1;
   static const char END_OF_INFORMATION[2] = ";";
   static char SEPARATOR[2] = "	";
   static char VALUE_SEPARATOR[2] = ",";
   static const char END_OF_LINE = '\0';
   static const char COMMENTARY[2] = "#";
   static const double CONFIDENCE_FACTOR = 0.95;
}

template<class FIterator, typename T>
FIterator binarySearch(FIterator begin, FIterator end, const T& key)
{
    // Keep halving the search space until we reach the end of the vector
    FIterator NotFound = end;

    while(begin < end)
    {
        // Find the median value between the iterators
        FIterator Middle = begin + (std::distance(begin, end) / 2);

        // Re-adjust the iterators based on the median value
        if(*Middle == key)
        {
            return Middle;
        }
        else if(*Middle > key)
        {
            end = Middle;
        }
        else
        {
            begin = Middle + 1;
        }
    }

    return NotFound;
}

template< typename T >
bool readCSVfileN(const char * fileName, std::vector < std::vector < T > > * table, char separator = Support::SEPARATOR[0])
{
	std::ifstream file(fileName, std::ios::in);
	char in[Support::MAX_READS];
	std::string value;
	std::vector < T > sample(0);
	unsigned int bytesRead, position;

	if (not file)
	{
		std::cerr << fileName << " not found!" << std::endl;
		return NULL; //exit(1);
	}

	do
		file.getline(in, Support::MAX_READS);
	while (in[0] == Support::COMMENTARY[0] and not file.eof());
	if (file.eof()) return NULL;
	bytesRead = file.gcount();
	while(bytesRead >= 2)
	{
		table->push_back(sample);
		position = 0;
		do
		{
			value.clear();
			do
				value.push_back(in[position++]);
			while (position < bytesRead and in[position] != separator and in[position] != Support::END_OF_INFORMATION[0]);
			table->back().push_back(atof(value.data()));
			position++;
		}
		while (position < bytesRead and in[position -1] != Support::END_OF_INFORMATION[0]);

		do file.getline(in, Support::MAX_READS);
		while (in[0] == Support::COMMENTARY[0] and not file.eof());
		if (file.eof()) break;
		bytesRead = file.gcount();
	}
	file.close();

	return true;
}

template< typename T >
bool readCSVfileA(const char * fileName, std::vector < std::vector < T > > * table, char separator = Support::SEPARATOR[0])
{
	std::ifstream file(fileName, std::ios::in);
	char in[Support::MAX_READS];
	std::string value;
	std::vector < T > sample(0);
	unsigned int bytesRead, position;

	if (not file)
	{
		std::cerr << fileName << " not found!" << std::endl;
		return NULL; //exit(1);
	}

	do
		file.getline(in, Support::MAX_READS);
	while (in[0] == Support::COMMENTARY[0] and not file.eof());
	if (file.eof()) return NULL;
	bytesRead = file.gcount();
	while(bytesRead >= 2)
	{
		table->push_back(sample);
		position = 0;
		do
		{
			value.clear();
			do
				value.push_back(in[position++]);
			while (position < bytesRead and in[position] != separator and in[position] != Support::END_OF_INFORMATION[0]);
			table->back().push_back(value.data());
			position++;
		}
		while (position < bytesRead and in[position -1] != Support::END_OF_INFORMATION[0]);

		do file.getline(in, Support::MAX_READS);
		while (in[0] == Support::COMMENTARY[0] and not file.eof());
		if (file.eof()) break;
		bytesRead = file.gcount();
	}
	file.close();

	return true;
}

void loadSearchSpace(char * searchSpaceFileName, std::vector < std::vector < std::string > > * sortedSearchSpace, std::vector < std::vector < int > > * sortedSearchSpaceAmount, std::vector < std::vector < std::string > > * searchSpace)
{
	std::vector < std::string > sampleLineString;
	std::vector < int > sampleLineInt;
	readCSVfileA(searchSpaceFileName, searchSpace);
	std::vector < std::vector < std::string > > tempSearchSpace;

	//parsing and sorting the search space
	for (unsigned int line=0; line < searchSpace->size(); line++)
	{
		tempSearchSpace.push_back(sampleLineString);
		tempSearchSpace.back().push_back("");
		for (unsigned int character=0; character < searchSpace->at(line).at(0).size(); character++)
		{
			if (searchSpace->at(line).at(0).at(character) == '-')
				tempSearchSpace.back().push_back("");
			else tempSearchSpace.back().back().push_back(searchSpace->at(line).at(0).at(character));
		}
		std::sort(tempSearchSpace.back().begin(), tempSearchSpace.back().end());
	}

	//removing repetitions and counting them
	for (unsigned int line=0; line < tempSearchSpace.size(); line++)
	{
		sortedSearchSpace->push_back(sampleLineString);
		sortedSearchSpace->back().push_back(tempSearchSpace.at(line).at(0));
		sortedSearchSpaceAmount->push_back(sampleLineInt);
		sortedSearchSpaceAmount->back().push_back(1);
		for (unsigned int column=1; column < tempSearchSpace.at(line).size(); column++)
		{
			if (sortedSearchSpace->back().back() == tempSearchSpace.at(line).at(column))
				sortedSearchSpaceAmount->back().back()++;
			else
			{
				sortedSearchSpace->back().push_back(tempSearchSpace.at(line).at(column));
				sortedSearchSpaceAmount->back().push_back(1);
			}
		}
	}
}

unsigned long int findString(const char * toFind, int sizeToFind, std::vector < std::vector < std::string > > * sortedSearchSpace, std::vector < std::vector < int > > * sortedSearchSpaceAmount, std::vector < std::vector < std::string > > * searchSpace)
{
	unsigned long int total = 0;
	bool found;
	std::vector < std::string > tempListToFind;
	std::vector < std::string > listToFind;
	std::vector < int > listToFindAmount;
	std::vector< std::string >::iterator foundPosition, begining;

	//loading the list to find in the search space
	tempListToFind.push_back("");
	for (int character=0; character < sizeToFind; character++)
	{
		if (toFind[character] == ' ')
			tempListToFind.push_back("");
		else tempListToFind.back().push_back(toFind[character]);
	}
	std::sort(tempListToFind.begin(), tempListToFind.end());

	//removing repetitions and counting them
	listToFind.push_back(tempListToFind.at(0));
	listToFindAmount.push_back(1);
	for (unsigned int string = 1; string < tempListToFind.size(); string++)
	{
		if (listToFind.back() == tempListToFind.at(string))
			listToFindAmount.back()++;
		else
		{
			listToFind.push_back(tempListToFind.at(string));
			listToFindAmount.push_back(1);
		}
	}

	//searching
	for (unsigned int line=0; line < sortedSearchSpace->size(); line++)
	{
		found = true;
		begining = sortedSearchSpace->at(line).begin();
		for (unsigned int string=0; string < listToFind.size() and found; string++)
		{
			foundPosition = binarySearch(begining, sortedSearchSpace->at(line).end(), listToFind.at(string));
			if (foundPosition == sortedSearchSpace->at(line).end() or sortedSearchSpaceAmount->at(line).at(foundPosition - sortedSearchSpace->at(line).begin()) < listToFindAmount.at(string))
				found = false;
//			else if (sortedSearchSpaceAmount->at(line).at(foundPosition - sortedSearchSpace->at(line).begin()) < listToFindAmount.at(string))
//				found = false;
			begining = foundPosition +1;
		}
		if (found)
			total += atoi(searchSpace->at(line).back().data());
	}

	return total;
}

void HandleTCPClient(TCPSocket *sock, std::vector < std::vector < std::string > > * sortedSearchSpace, std::vector < std::vector < int > > * sortedSearchSpaceAmount, std::vector < std::vector < std::string > > * searchSpace);     // TCP client handling function
void *ThreadMain(void *arg);               // Main program of a thread

int main(int argc, char *argv[]) {
  unsigned short echoServPort = 0;    // First arg:  local port
  char * searchSpaceFileName = argv[1];
  std::stringstream serverLocation;
  serverLocation << SERVER_LOCATION[0] << SERVER_LOCATION[1];

  std::vector < std::vector < std::string > > * sortedSearchSpace = new std::vector < std::vector < std::string > > ();
  std::vector < std::vector < int > > * sortedSearchSpaceAmount = new std::vector < std::vector < int > > ();
  std::vector < std::vector < std::string > > * searchSpace = new std::vector < std::vector < std::string > > ();

  loadSearchSpace(searchSpaceFileName, sortedSearchSpace, sortedSearchSpaceAmount, searchSpace);

  //std::cout << "ready" << endl;
  try {
    TCPServerSocket servSock(echoServPort);   // Socket descriptor for server

    char szHostName[255];
    std::ofstream file(serverLocation.str().data(), std::ios::out);
    gethostname(szHostName, 255);
    file << szHostName << endl;
    file << servSock.getLocalPort() << endl;
    file.close();

    for (;;) {      // Run forever  
      // Create separate memory for client argument  
      TCPSocket * clntSock = servSock.accept();
      ThreadParameters * tp = new ThreadParameters;

      tp->clntSock = clntSock;
      tp->searchSpace = searchSpace;
      tp->sortedSearchSpace = sortedSearchSpace;
      tp->sortedSearchSpaceAmount = sortedSearchSpaceAmount;
  
      // Create client thread  
      pthread_t threadID;              // Thread ID from pthread_create()
      if (pthread_create(&threadID, NULL, ThreadMain, 
              (void *) tp) != 0) {
        cerr << "Unable to create thread" << endl;
        exit(1);
      }
    }
  } catch (SocketException &e) {
    cerr << e.what() << endl;
    exit(1);
  }
  // NOT REACHED

  return 0;
}

// TCP client handling function
void HandleTCPClient(TCPSocket *sock, std::vector < std::vector < std::string > > * sortedSearchSpace, std::vector < std::vector < int > > * sortedSearchSpaceAmount, std::vector < std::vector < std::string > > * searchSpace) {
  // Send received string and receive again until the end of transmission
  char echoBuffer[RCVBUFSIZE];
  std::stringstream end;
  int recvMsgSize;
  while ((recvMsgSize = sock->recv(echoBuffer, RCVBUFSIZE)) > 0) { // Zero means
                                                         // end of transmission
    // Echo message back to client
    end << findString(echoBuffer, recvMsgSize, sortedSearchSpace, sortedSearchSpaceAmount, searchSpace);
    sock->send(end.str().data(), end.str().size());
    //cout << end.str() << endl;
    end.str(echoBuffer);
    if (end.str().compare("end") == 0) exit(0);
  }
  // Destructor closes socket
}

void *ThreadMain(void *tp) {
  // Guarantees that thread resources are deallocated upon return  
  pthread_detach(pthread_self()); 

  // Extract socket file descriptor from argument
  HandleTCPClient((TCPSocket *) ((ThreadParameters *) tp)->clntSock, (std::vector < std::vector < std::string > > *) ((ThreadParameters *) tp)->sortedSearchSpace, (std::vector < std::vector < int > > *) ((ThreadParameters *) tp)->sortedSearchSpaceAmount, (std::vector < std::vector < std::string > > *) ((ThreadParameters *) tp)->searchSpace);

  delete (TCPSocket *) ((ThreadParameters *) tp)->clntSock;
  delete (ThreadParameters *) tp;
  return NULL;
}
