package PSIBLAST;

if (exists($ENV{'PERL_LIB'})){
	use lib $ENV{'PERL_LIB'};
} 
else{
	print "ERROR:: Not existis PERL_LIB\n";
	exit;
}

use File::Files;
use Align::Primary::blast;
use Database::ScopPattern; 
use Util::Array;

my $build = "blastpgp";
my $search = "blastall";
my $TAG_RESULT = "Sequences producing significant alignments:";

########################### sub buildPSSM ##############################
sub buildPSSM {
	my $seq_file		= $_[1];
    my $database		= $_[2];
    my $iteration		= $_[3];
    my $path_data		= $_[4];
    my $pssm_file		= $_[5];
    my $seq_file_out	= $_[6];
    my $parameter		= $_[7];
    my $verbose			= $_[8];
	
	if (!defined($verbose)){
		$verbose = "true";
	}
	#verify directory

	FilesIO->createDirFromFile($seq_file_out);
	
	#print "path data" . $path_data."\n";
	
	chdir($path_data);
	my $cmd =$build." -d ".$database." -i ". $seq_file;
	if ($iteration ne "") {
		$cmd .= " -j ". $iteration;
	}
	
	if ($pssm_file ne ""){
		FilesIO->createDirFromFile($pssm_file);
		$cmd .= " -C ".$pssm_file;
	}
	
	$cmd .= " -o ".$seq_file_out.$parameter;
	
	if ($verbose eq "true") {
		print $cmd."\n";
	}	
	system $cmd;
	
	if ($pssm_file ne ""){
		if (FilesIO->isFile($pssm_file) eq "false"){
			print "ERROR::perlLib::psi_blast:: $pssm_file NOT created!\n";
			exit;
		}elsif ($verbose eq "true") {
			print "perlLib::psi_blast:: $pssm_file created!\n";
		}
	}
	
	if (FilesIO->isFile($seq_file_out) eq "false"){
		print "ERROR::perlLib::psi_blast:: $seq_file_out NOT created!\n";
		exit;
	}elsif ($verbose eq "true") {
		print "perlLib::psi_blast:: $seq_file_out created!\n";
	}
	
}

########################### sub searchPSSM ##############################
sub searchPSSM {
	my $seq_file		= $_[1];
    my $database		= $_[2];
    my $path_data		= $_[3];
    my $pssm_file		= $_[4];
    my $search_file_out	= $_[5];
    my $parameter		= $_[6];
    my $verbose			= $_[7];
	
	if (!defined($verbose)){
		$verbose = "true";
	}
   
    chdir($path_data);
	my $cmd = $build. " -i ". $seq_file." -d ". $database. " -R ". $pssm_file .$parameter. " -o ". $search_file_out;
	
	if ($verbose eq "true") {
		print $cmd."\n";
	}
	system $cmd;
	
	if (FilesIO->isFile($search_file_out) eq "false"){
		print "ERROR::perlLib::psi_blast:: $search_file_out is NOT created!\n";
		exit;
	}elsif ($verbose eq "true") {
		print "perlLib::psi_blast:: File $search_file_out is created!\n";
	}
	
	
}

########################### sub searchPSSM ##############################
sub searchOneByOnePSSM {
	my $seq_file		= $_[1];
    my $database		= $_[2];
    my $path_data		= $_[3];
    my $pssm_file		= $_[4];
    my $search_file_out	= $_[5];
    my $parameter		= $_[6];
    my $verbose			= $_[7];
    
    my $seq; my $cmd; my $hit;my @array_hits; my @array_aux; my $scop_id;
    my $family_id; my $info = "";
	
	if (!defined($verbose)){
		$verbose = "true";
	}
   
    chdir($path_data);
    my $dir = FilesIO->getDirectory($database);
    
    my @sequences = Sequences->getSequenceFromFastaArray2D($database);
    for (my $i=0; $i <scalar(@sequences); $i++){
    	$seq = $sequences[$i][0]. Sequences->divideSeq($sequences[$i][1], 50);
    	FilesIO->saveFile($dir."seq.tmp", $seq, $verbose);
    	Blast->formatdb($dir."seq.tmp", "", $verbose);
        $cmd = $build. " -i ". $seq_file." -d ". $dir."seq.tmp". " -R ". $pssm_file .$parameter. " -o ". $search_file_out;
        if ($verbose eq "true") {
			print $cmd."\n";
		}
		system $cmd;
		
		FilesIO->checkFile($search_file_out, "ERROR::perlLib::psi_blast:: $search_file_out is NOT created!\n");
		@array_hits = Array->lineStringToMatriz($sequences[$i][0], " +", "\>");
		$evalue = Blast->getEvalue($search_file_out);
		$info .= "$array_hits[0][0]\t$array_hits[0][1]\t$evalue\n";
		FilesIO->removeFiles($dir."seq.tmp", $verbose);
    }
	return $info;
}


########################### sub searchPSSM ##############################
sub getResultPSSM {
	my $result_file		= $_[1];
	
	my @results = FilesIO->lineFileToArray($result_file);
	my $i = 0;
	my $info = "";
	my @array_line;
	my @array_result = ();
	my $j = 0;
	my $evalue;
	
	while($i < scalar(@results)) {
		#print $results[$i];
		if (index($results[$i], $TAG_RESULT) >=0){
			$i += 2;
			#print "oi = $i ".$results[$i];
			
			while ($results[$i] ne "\n" && $i < scalar(@results)){	
				@array_line = split(/ +/, $results[$i]);	
				$array_result[$j] = $array_line[0]."\t".$array_line[1]."\t";
				$evalue = String->trim(substr($results[$i],76,length($results[$i])));
				$evalue = String->removeCaracter($evalue, "\n");
				$array_result[$j] .=$evalue; 
				$j++;	
				$info .= $results[$i];
				$i++;
			}	
			#print $info;		
		}
		$i++;
	}
	#print $info;	
	return @array_result;
}
