#!/usr/bin/perl -w
use strict;

if (exists($ENV{'PERL_LIB'})){
	use lib $ENV{'PERL_LIB'};
} 
else{
	print "ERROR:: Not existis PERL_LIB\n";
	exit;
}

use File::Files;
use PSSM::psi_blast_source_code;
use Database::ScopPattern;
use Align::Primary::blast;
use Sequence::sequence;


my $seqFile 	= "";
my $dbBuild 	= "";
my $iteration 	= "5";
my $pssmFile	= "";
my $seqFileOut	= "";
my $parameter   = " -J T -u 1 ";
my $evalue = "0.001"; 
my $verbose = "false";
my $tmpDir = "";
my $run = "true";
my $FamilyID = "";

my $PROG_NAME = "perlLib::run::run_psi_blast.pl";

&getParameters;
&checkParameters;
&main;

############################ sub getParameters ##################################
sub getParameters {
	my $counter;
	my $ending_value = scalar(@ARGV);		#numero de parametros de entrada
	for($counter = 0 ; $counter < $ending_value ; $counter++){
		if($ARGV[$counter] eq "-seqFile"){
			$seqFile = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-dbBuild"){
			$dbBuild = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-iteration"){
			$iteration = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-pssmFile"){
			$pssmFile = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-seqFileOut"){
			$seqFileOut = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-evalue"){
			$evalue = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-verbose"){
			$verbose = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-tmpDir"){
			$tmpDir = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-run"){
			$run = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-FamilyID"){
			$FamilyID = $ARGV[$counter+1];		
		}	           	  	 			
	}
}

############################ sub checkParameters ##################################
sub checkParameters {
	print "\n=============================================\n";
	print "Cheking Parameters $PROG_NAME\n";
	
	String->isEqual($verbose,"true,false",$PROG_NAME."::ERROR::Parameter -verbose must be true or false!");
	
	String->isEmpty($seqFile, $PROG_NAME."::ERROR::Parameter -seqFile having query sequence is required!");
	FilesIO->checkFile($seqFile, $PROG_NAME."ERROR:: File $seqFile NOT found ");	
	String->isEmpty($dbBuild, $PROG_NAME."::ERROR::Parameter -dbBuild having database for buildind PSSM is required!");	
	String->isEmpty($pssmFile, $PROG_NAME."::ERROR::Parameter -pssmFile having PSSM file output is required!");
	String->isEmpty($tmpDir, $PROG_NAME."::ERROR::Parameter -tmpDir having temporary directory is required!");
	FilesIO->createDirectory($tmpDir, $verbose);
	
	print "-seqFile = $seqFile\n";
	print "-dbBuild = $dbBuild\n";
	print "-pssmFile = $pssmFile\n";
	print "-iteration = $iteration\n";
	print "-evalue = $evalue\n";
	print "-seqFileOut = $seqFileOut\n";
	print "-tmpDir = $tmpDir\n";
	print "-run = $run\n";
	print "-verbose = $verbose\n";
	
	
	print "ok\n";
}	


########################### sub main ####################################
sub main { 
	my $entry;					my $info = "";
	my $dir = "";				my $pssmFile_name;
	
	my $out = $tmpDir."seq.out";
	
	if (FilesIO->isFile($pssmFile) eq "true"){
		print "LOG::".$PROG_NAME.":: File $pssmFile already created\n"; 
	}
	elsif ($run eq "true"){
		$dir = FilesIO->getDirectory($pssmFile);
		FilesIO->createDirectory($dir);

		PSIBLAST2->buildPSSM($seqFile, $dbBuild, $iteration, $pssmFile, $seqFileOut, $evalue, " -outfmt 0 ", $verbose);		
		
	}else{
		$pssmFile_name = FilesIO->getNameFile($pssmFile);
		print "LOG::".$PROG_NAME."::Running::$FamilyID\t$pssmFile_name\n";
	}
}